/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "CentralWidget/centraltoolbar.h"
#include "Widget/spacer.h"

#include <QtGui>

namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}

//! ---------- global instance -------------------------------------------------
CentralToolBar* CentralToolBar::global_instance = 0;

namespace GLOBAL
{
    CentralToolBar* centralToolBar()
    {
        return CentralToolBar::instance();
    }
}


//! ---------- class CentralToolBar --------------------------------------------
CentralToolBar::CentralToolBar(QWidget *parent) : QToolBar( "CentralToolBar", parent )
{
  global_instance = this;
  _menuModel = MenuModel::instance();
  
  this->setObjectName(QString::fromUtf8("CentralToolBar"));
  this->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum );
  this->setMaximumHeight(28);
  this->setFloatable(false);
  this->setMovable(false);
  this->setContentsMargins( 0, 0, 0, 0);
  this->setToolButtonStyle( Qt::ToolButtonIconOnly );
  this->setIconSize( QSize( 20, 20) );
  
  _currentBrowserIdx = 0;
  
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  //! Breadcrumb 
  _breadcrumb = new BreadCrumb();
  _breadcrumb->setModel(_menuModel);
  _breadcrumb->setIconSize(QSize( 20, 20));
  _breadcrumb->setRootIndex(_menuModel->indexFromItem(_menuModel->invisibleRootItem()));
  _breadcrumb->setCurrentIndex(_menuModel->indexFromItem(_menuModel->invisibleRootItem()->child(0)));
     
  //! Prev/Next
  QAction *goPrevAct  = new QAction(QIcon::fromTheme("go-previous", QIcon(":/images/go-previous.png")),tr("Go back"),this);  
  QAction *goNextAct  = new QAction(QIcon::fromTheme("go-next", QIcon(":/images/go-next.png")),tr("Go forward"),this);  
  actions->insert("goPrev", goPrevAct);
  actions->insert("goNext", goNextAct);
  QObject::connect(goPrevAct, SIGNAL(triggered()), this, SLOT(activePrev()));
  QObject::connect(goNextAct, SIGNAL(triggered()), this, SLOT(activeNext()));
  
      
  //! Up/Down Actions
  goUpAct  = new QAction(QIcon::fromTheme("go-up", QIcon(":/images/go-up.png")),tr("Go category up"),this);
  goDownAct = new QAction(QIcon::fromTheme("go-down", QIcon(":/images/go-down.png")),tr("Go category down"),this);
  actions->insert("goUp", goUpAct);
  actions->insert("goDown", goDownAct);
  
  //! Search field widget
  _collectionSearch = new SearchField(this);
  _collectionSearch->setInactiveText(tr("Search in Collection"));
  QObject::connect(_collectionSearch, SIGNAL(signalSearchTriggered()), this, SLOT(sendCollectionSearch()));

  _playlistSearch = new SearchField(this);
  _playlistSearch->setInactiveText(tr("Search in Playlist"));
  QObject::connect(_playlistSearch, SIGNAL(signalSearchTriggered()), this, SLOT(sendPlaylistSearch()));
  
  _radioSearch = new SearchField(this);
  _radioSearch->setInactiveText(tr("Search in Radio"));
  QObject::connect(_radioSearch, SIGNAL(signalSearchTriggered()), this, SLOT(sendRadioSearch()));

  //! Playlist Menu & Button
  QToolButton* playlistButton = new QToolButton(this);
  playlistButton->setDefaultAction( actions->value("showPlaylist") );
  playlistButton->setAutoRaise(true);  
  playlistButton->setPopupMode(QToolButton::MenuButtonPopup);
 
  QMenu *_playlistMenu = new QMenu(this);
  _playlistMenu->addAction(actions->value("addfile"));
  _playlistMenu->addAction(actions->value("adddir"));
  _playlistMenu->addAction(actions->value("addurl"));    
  _playlistMenu->addAction(actions->value("clear"));
  _playlistMenu->addAction(actions->value("save"));
  _playlistMenu->addAction(actions->value("autosave"));
  _playlistMenu->addSeparator();
    QMenu *menuMode = _playlistMenu->addMenu(tr("&Choose playlist mode"));
    menuMode->addAction(actions->value("playlistModeTitle"));
    menuMode->addAction(actions->value("playlistModeAlbum"));
    menuMode->addAction(actions->value("playlistModeArtist"));
  _playlistMenu->addSeparator();
  _playlistMenu->addAction(actions->value("showPlaylist"));
  
  playlistButton->setMenu(_playlistMenu);

  //!Tool Bar Settup
  QFrame* separator1 = new QFrame( this );
  separator1->setFrameShape( QFrame::VLine );
  QFrame* separator2 = new QFrame( this );
  separator2->setFrameShape( QFrame::VLine );
      
  QWidget *expandSpacerWidget = new QWidget(this);
  expandSpacerWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
  
  this->addWidget( new FixedSpacer( this, QSize(5, 0)) );
  this->addAction( goPrevAct );
  this->addAction( goNextAct );
  this->addWidget( separator1 );
  this->addWidget( _breadcrumb );
  this->addWidget( expandSpacerWidget);
  this->addAction(goUpAct);
  this->addAction(goDownAct);
  _actCSearch = this->addWidget( _collectionSearch );
  _actCSearch->setVisible(false);  
  _actPSearch = this->addWidget( _playlistSearch );
  _actPSearch->setVisible(false);
  _actRSearch = this->addWidget( _radioSearch );
  _actRSearch->setVisible(false);
  this->addWidget( separator2 ) ;
  this->addWidget( playlistButton ) ;
  this->addWidget( new FixedSpacer( this, QSize(5, 0)) );
}

//! ------------ Search field method -------------------------------------------
void CentralToolBar::sendCollectionSearch() 
{
  emit collectionSearch(_collectionSearch->text());
}

void CentralToolBar::sendPlaylistSearch() 
{
  emit playlistSearch(_playlistSearch->text());
}

void CentralToolBar::sendRadioSearch() 
{
  emit radioSearch(_radioSearch->text());
}

//! ------------ update Mode ---------------------------------------------------
void CentralToolBar::updateMode(Main::MainMode mode)
{
  goUpAct->setVisible(false); 
  goDownAct->setVisible(false);
  _actPSearch->setVisible(false);
  _actRSearch->setVisible(false);                      
  _actCSearch->setVisible(false);
  
  goUpAct->setVisible(mode != Main::FilerMode); 
  goDownAct->setVisible(mode != Main::FilerMode);
  _actCSearch->setVisible(mode == Main::CollectionMode);
  _actPSearch->setVisible(mode == Main::PlaylistMode);            
  _actRSearch->setVisible(mode == Main::RadioMode);
}



//! --------------- breadCrumb method ------------------------------------------
void CentralToolBar::updateCurrentIndex(const QModelIndex &idx)
{
  //! nouveau dclenchement --> on supprime les next
  for(int i = 0; i < _currentBrowserIdx; i++) {
    _browserList.takeAt(i);
  }
  _breadcrumb->setCurrentIndex(idx);
  QString text = idx.data(Qt::DisplayRole).toString();
  _browserList.prepend(text);
  _currentBrowserIdx = 0;

  GLOBAL::globalActions()->value("goPrev")->setEnabled(_currentBrowserIdx < _browserList.size() -1);
  GLOBAL::globalActions()->value("goNext")->setEnabled(_currentBrowserIdx > 0);
  
  //! limite de la taille de l'historique de navigation
  if(_browserList.size() >= 20 && _currentBrowserIdx != _browserList.size() -1)
    _browserList.takeLast();
}

void CentralToolBar::activePrev()
{
   if(_currentBrowserIdx < _browserList.size() -1) {
      _currentBrowserIdx++;
      _breadcrumb->restorePath( _browserList.at(_currentBrowserIdx) );
      const QModelIndex idx = _breadcrumb->currentIndex();
      emit itemActivated(idx);

      GLOBAL::globalActions()->value("goPrev")->setEnabled(_currentBrowserIdx < _browserList.size() -1);
      GLOBAL::globalActions()->value("goNext")->setEnabled(_currentBrowserIdx > 0);
   }
}

void CentralToolBar::activeNext()
{
   if(_currentBrowserIdx > 0) {
      _currentBrowserIdx--;
      _breadcrumb->restorePath( _browserList.at(_currentBrowserIdx) );
      const QModelIndex idx = _breadcrumb->currentIndex();
      emit itemActivated(idx);
      
      GLOBAL::globalActions()->value("goPrev")->setEnabled(_currentBrowserIdx < _browserList.size() -1);
      GLOBAL::globalActions()->value("goNext")->setEnabled(_currentBrowserIdx > 0);
   }
}

