/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CentralWidget/centralwidget.h"

#include "constants.h"
#include <QSettings>

namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}


CentralWidget::CentralWidget(
                       QWidget *parent,
                       CentralToolBar   *centralToolBar,
                       MenuView         *menuView,
                       NowPlayingView   *nowPlayingView,
                       PlaylistWidget   *playlistWidget,
                       CollectionWidget *collectionWidget,
                       FileViewer       *fileViewer,
                       RadioWidget      *radioWidget
		                  ) : QWidget(parent)
{
  //! get all widgets
  _centraltoolbar   = centralToolBar;
  _menuView         = menuView;
  _nowplayingview   = nowPlayingView;
  _playlistwidget   = playlistWidget;
  _collectionwidget = collectionWidget;
  _fileViewer       = fileViewer;
  _radioWidget      = radioWidget;

  //! configure stacked widget
  _stackedLeftWidget = new QStackedWidget(this);
  _stackedLeftWidget->addWidget(_collectionwidget);
  _stackedLeftWidget->addWidget(_radioWidget);
  _stackedLeftWidget->addWidget(_fileViewer);

  //! Gui setup
  createGui();
  
  //! Connect Actions 
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  connect(actions->value("showPlaylist"), SIGNAL(triggered()), SLOT(showPlaylist()));
  connect(actions->value("showMenu"), SIGNAL(triggered()), SLOT(showMenu()));
  
  connect(_menuView,SIGNAL(doubleClicked(QModelIndex)),this, SLOT(menuItemActivated(QModelIndex)));  
  connect(_centraltoolbar,SIGNAL(itemActivated(QModelIndex)),this, SLOT(toolbarItemActivated(QModelIndex)));  

  //! Restore previous state
  restoreState();

}


//! --------- createGui --------------------------------------------------------
void CentralWidget::createGui()
{
  this->setObjectName(QString::fromUtf8("Main widget"));
  this->setAutoFillBackground(false);
  
  QVBoxLayout* vl0 = new QVBoxLayout(this);
  vl0->setSpacing(0);
  vl0->setContentsMargins(0, 0, 0, 0);
  
  //! top Frame
  QWidget *topFrame = new QFrame(this);
  topFrame->setMaximumHeight(28);

  QHBoxLayout* hl1 = new QHBoxLayout(topFrame);
  hl1->setSpacing(0);
  hl1->setContentsMargins(0, 0, 0, 0);
  hl1->addWidget(_centraltoolbar);

  //! main Frame
  QFrame *mainFrame = new QFrame(this);
  mainFrame->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
  
  QHBoxLayout* hl2 = new QHBoxLayout(mainFrame);
  hl2->setSpacing(6);
  hl2->setContentsMargins(2, 2, 2, 2);
    
  _viewsSplitter = new QSplitter(this);
  _viewsSplitter->setObjectName(QString::fromUtf8("viewsSplitter"));
  _viewsSplitter->setLineWidth(0);
  _viewsSplitter->setOrientation(Qt::Horizontal);
  _viewsSplitter->setOpaqueResize(true);
  _viewsSplitter->setHandleWidth(1);
  _viewsSplitter->setChildrenCollapsible(false);
  
  //! right pane
  _rightWidget = new QFrame(this);
  _rightWidget->setFrameShape(QFrame::StyledPanel);
  _rightWidget->setFrameShadow(QFrame::Sunken);
  
  QVBoxLayout* verticalLayout = new QVBoxLayout(_rightWidget);
  verticalLayout->setSpacing(0);
  verticalLayout->setContentsMargins(0, 0, 0, 0);
  verticalLayout->addWidget(_nowplayingview);
  verticalLayout->addWidget(_playlistwidget);
    
  //! splitter population
  _viewsSplitter->addWidget(_menuView);
  _viewsSplitter->addWidget(_stackedLeftWidget);
  _viewsSplitter->addWidget(_rightWidget);
    
  hl2->addWidget(_viewsSplitter);

  vl0->addWidget(topFrame);
  vl0->addWidget(mainFrame);
}


//! --------- ~CentralWidget ---------------------------------------------------
CentralWidget::~CentralWidget()
{
  //! state storage
  QMap<QString, QAction*> *actions = GLOBAL::globalActions(); 
  QSettings *_settings = new QSettings(_CONFIGDIR + QDir::separator() + "main.conf",QSettings::IniFormat,this);
 
  _settings->setValue("Window/ViewsSplit",_viewsSplitter->saveState());
  _settings->setValue("Playlist/show",(actions->value("showPlaylist"))->isChecked());
  _settings->setValue("CentralWidget/menu",(actions->value("showMenu"))->isChecked());
  _settings->setValue("CentralWidget/mode",_mainMode );
  
  //! save breadcrumb activated item
  QModelIndex idx = _centraltoolbar->_breadcrumb->currentIndex();
  _settings->setValue("CentralWidget/breadcrumb", idx.data(Qt::DisplayRole).toString() );
}

  
  
//! --------- restoreState -----------------------------------------------------
void CentralWidget::restoreState()
{
  //! restore QSplitter state    
  QSettings *_settings = new QSettings(_CONFIGDIR + QDir::separator() + "main.conf",QSettings::IniFormat,this);
  if(_settings->contains("Window/ViewsSplit"))
    _viewsSplitter->restoreState(_settings->value("Window/ViewsSplit").toByteArray());

  //! restore Playlist Hide/Show state 
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  if(_settings->contains("Playlist/show"))
    (actions->value("showPlaylist"))->setChecked(_settings->value("Playlist/show").toBool());
  else
    (actions->value("showPlaylist"))->setChecked(true);
    
  showPlaylist();

  //! restore Menu hide/show  
  if(_settings->contains("CentralWidget/menu"))
    (actions->value("showMenu"))->setChecked(_settings->value("CentralWidget/menu").toBool());
  else
    (actions->value("showMenu"))->setChecked(true);

  showMenu();

  //! restore MainWidget mode (collection, filer, ...)  
  if(_settings->contains("CentralWidget/mode"))
    _mainMode = Main::MainMode(_settings->value("CentralWidget/mode").toInt());
  else 
    _mainMode = Main::CollectionMode;

  //! restore BreadCrumb state
  if(_settings->contains("CentralWidget/breadcrumb")) {
    QString breadCrumbItemText;
    breadCrumbItemText = _settings->value("CentralWidget/breadcrumb").toString();
    
    QModelIndex idx = static_cast<MenuModel*>(_menuView->model())->IndexFromItemText(breadCrumbItemText);
    _centraltoolbar->updateCurrentIndex(idx);
  }
  
  _centraltoolbar->updateMode(_mainMode);
  updateMode(_mainMode);  
}


//! --------- menuItemActivated ------------------------------------------------
void CentralWidget::menuItemActivated(QModelIndex index)
{
    QStandardItem *item = static_cast<QStandardItemModel*>( _menuView->model() )->itemFromIndex( index );
    MenuItem *mi = dynamic_cast<MenuItem*>(item);
    if(!mi) return;
    
    switch(mi->type()) 
    {
      case MenuCustomRadioType :
         _radioWidget->_radioFile =  index.data(FileRole).toString();
         qDebug() << "--- CentralWidget --> menuItemActivated radio file " << _radioWidget->_radioFile;
      case MenuRadioType :   
         _centraltoolbar->updateMode(Main::RadioMode);
         _centraltoolbar->updateCurrentIndex(index);   
         this->updateMode(Main::RadioMode);
         break; 
      case MenuPlaylistType :   
         _centraltoolbar->updateMode(Main::PlaylistMode);
         _centraltoolbar->updateCurrentIndex(index);   
         this->updateMode(Main::PlaylistMode);
         break; 
      case MenuCollectionType : 
         _centraltoolbar->updateMode(Main::CollectionMode);
         _centraltoolbar->updateCurrentIndex(index);   
         this->updateMode(Main::CollectionMode);
         break; 
      case MenuFilerType: 
         _centraltoolbar->updateMode(Main::FilerMode);
         _centraltoolbar->updateCurrentIndex(index);   
         this->updateMode(Main::FilerMode);
         break; 
     
     default: break;
   }
   
   QAction *action = 0;
   action = index.data(ActionRole).value<QAction*>();
   if(action) action->trigger();
}


//! --------- toolbarItemActivated ---------------------------------------------
void CentralWidget::toolbarItemActivated(QModelIndex index)
{
    QStandardItem *item = static_cast<QStandardItemModel*>( _menuView->model() )->itemFromIndex( index );
    MenuItem *mi = dynamic_cast<MenuItem*>(item);
    if(!mi) return;
    
    switch(mi->type()) 
    {
      case MenuCustomRadioType :
         _radioWidget->_radioFile =  index.data(FileRole).toString();
         qDebug() << "--- CentralWidget --> toolbarItemActivated radio file " << _radioWidget->_radioFile;
      case MenuRadioType :   
         _centraltoolbar->updateMode(Main::RadioMode);
         this->updateMode(Main::RadioMode);
         break; 
      case MenuPlaylistType :   
         _centraltoolbar->updateMode(Main::PlaylistMode);
         this->updateMode(Main::PlaylistMode);
         break; 
      case MenuCollectionType : 
         _centraltoolbar->updateMode(Main::CollectionMode);
         this->updateMode(Main::CollectionMode);
         break; 
      case MenuFilerType: 
         _centraltoolbar->updateMode(Main::FilerMode);
         this->updateMode(Main::FilerMode);
         break; 
     
     default: break;
   }
   
   QAction *action = 0;
   action = index.data(ActionRole).value<QAction*>();
   if(action) action->trigger();
}


//! --------- Mode Update (Collection, Filer, ...) -----------------------------  
void CentralWidget::updateMode(Main::MainMode mode)
{
   //qDebug() << "CentralWidget --> updateMode---" << mode;
  _mainMode = mode;
  int widgetIdx = int(_mainMode) == 0 ? 0 :  int(_mainMode) -1;
  _stackedLeftWidget->setCurrentIndex(widgetIdx);
}
    
//! --------- Hide/Show Playlist Widget (_rightWidget) -------------------------  
void CentralWidget::showPlaylist( ) 
{
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  if(!  actions->value("showPlaylist")->isChecked()) {
    actions->value("showPlaylist")->setToolTip("Show playlist");
    _rightWidget->hide();  
  }
  else {
    actions->value("showPlaylist")->setToolTip("Hide playlist");
    _rightWidget->show(); 
  }
}

//! --------- Hide/Show Menu Widget (_leftWidget) ------------------------------  
void CentralWidget::showMenu( ) 
{
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  if(!  actions->value("showMenu")->isChecked()) {
    actions->value("showMenu")->setToolTip("Show menu");
    _menuView->hide();  
  }
  else {
    actions->value("showMenu")->setToolTip("Hide menu");
    _menuView->show(); 
  }
}
