/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionModel/collectionmodel.h"

#include <QDebug>
#include <QRegExp>


//! ----- global instance ------------------------------------------------------
CollectionModel* CollectionModel::globalCollectionModel = 0;

namespace GLOBAL
{
    CollectionModel* collectionModel()
    {
        return CollectionModel::instance();
    }
}

//! ----- CollectionModel ------------------------------------------------------
//CollectionModel::CollectionModel(const QStringList &headers, QObject *parent) : QAbstractItemModel(parent)
CollectionModel::CollectionModel(QObject *parent) : QAbstractItemModel(parent)
{
  globalCollectionModel = this;
       
  //! root item
  QVector<QVariant> rootData;
  //foreach (QString header, headers)
  rootData << "id" << "name" << "data1" << "data2" << "data3";
  _rootItem = new ModelItem(rootData);

  setSupportedDragActions(Qt::CopyAction);
  
  _filterPattern   = "";
  _playingItemIdx  = QModelIndex();
}

CollectionModel::~CollectionModel()
{
    delete _rootItem;
}



int CollectionModel::columnCount(const QModelIndex & /* parent */) const
{
    return _rootItem->columnCount();
}

QVariant CollectionModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    if (role != Qt::DisplayRole && role != Qt::EditRole)
        return QVariant();

    ModelItem *item = getItem(index);
    return item->getData(index.column());
}

Qt::ItemFlags CollectionModel::flags(const QModelIndex &index) const
{
    if (!index.isValid())
        return 0;

    return Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled;
}

ModelItem *CollectionModel::getItem(const QModelIndex &index) const
{
    if (index.isValid()) {
        ModelItem *item = static_cast<ModelItem*>(index.internalPointer());
        if (item) return item;
    }
    return _rootItem;
}

QVariant CollectionModel::headerData(int section, Qt::Orientation orientation,
                                     int role) const
{
    if (orientation == Qt::Horizontal && role == Qt::DisplayRole)
        return _rootItem->data(section);

    return QVariant();
}

QModelIndex CollectionModel::index(int row, int column, const QModelIndex &parent) const
{
    if (parent.isValid() && parent.column() != 0)
        return QModelIndex();

    ModelItem *parentItem = getItem(parent);

    ModelItem *childItem = parentItem->child(row);
    if (childItem)
        return createIndex(row, column, childItem);
    else
        return QModelIndex();
}

bool CollectionModel::insertRows(int position, int rows, const QModelIndex &parent)
{
  //qDebug() << "insertRows:";
  ModelItem *parentItem = getItem(parent);
  bool success;
  
  beginInsertRows(parent, position, position + rows - 1);
  success = parentItem->insertChildren(position, rows, _rootItem->columnCount());
  endInsertRows();

  return success;
}

bool CollectionModel::removeRows(int position, int rows, const QModelIndex &parent)
{
    ModelItem *parentItem = getItem(parent);
    bool success = true;

    beginRemoveRows(parent, position, position + rows - 1);
    success = parentItem->removeChildren(position, rows);
    endRemoveRows();

    return success;
}

QModelIndex CollectionModel::parent(const QModelIndex &index) const
{
  if (!index.isValid())
    return QModelIndex();
  
  ModelItem *childItem = getItem(index);
  ModelItem *parentItem = childItem->parent();
  
  if (parentItem == _rootItem)
    return QModelIndex();
  
  return createIndex(parentItem->childNumber(), 0, parentItem);
}


int CollectionModel::rowCount(const QModelIndex &parent) const
{
  ModelItem *parentItem = getItem(parent);
  
  return parentItem->childCount();
}

bool CollectionModel::setData(const QModelIndex &index, const QVariant &value,int role)
{
  if (role != Qt::EditRole)
    return false;
  
  ModelItem *item = getItem(index);
  bool result = item->setData(index.column(), value);
  
  if (result)
    emit dataChanged(index, index);
  
  return result;
}



bool CollectionModel::setHeaderData(int section, Qt::Orientation orientation, const QVariant &value, int role)
{
  if (role != Qt::EditRole || orientation != Qt::Horizontal)
    return false;
  
  bool result = _rootItem->setData(section, value);

  if (result)
    emit headerDataChanged(orientation, section, section);
  
  return result;
}


QModelIndex CollectionModel::addChild(const QModelIndex &parent, const QVector<QVariant> &data)
{
  //qDebug() << "addChild:";
  if (!insertRow(rowCount(parent), parent)) {
    return QModelIndex();
  }

  ModelItem *item = getItem(parent);
  int childCount = item->childCount()-1;

  // WARNING QVector can have different size
  for(int i=0; i < data.size() ; i++) {
    item->child(childCount)->setData(i, data.value(i));
  }
  
  item->child(childCount)->setIsPlaying(false);
  item->child(childCount)->setIsFavorite(false);
  
  return index(rowCount(parent)-1,0,parent);
}


void CollectionModel::clear()
{
    removeRows(0,rowCount(QModelIndex()),QModelIndex());
    _listGenre.clear();
    emit modelCleared();
}

//! ----------------------- Get tracks method ----------------------------------
QStringList CollectionModel::getItemChildrenTracks(const QModelIndex &itemIdx)
{
    QStringList result;

    ModelItem *item = getItem(itemIdx);
    if (item->childCount() == 0) { 
      if(isTrackFiltered(itemIdx))
        result.append( item->getData(2).toString() );    
    } 
    else {
      for (int i = 0; i < item->childCount(); i++) {
        result.append(getItemChildrenTracks(index(i,0,itemIdx)));
      }
    }
    return result;
}

QStringList CollectionModel::getAlbumChildrenTracksGenre(const QModelIndex &albumIdx, QString genre) 
{
    QStringList result;

    ModelItem *albumItem = getItem(albumIdx);
    for (int i = 0; i < albumItem->childCount(); i++) {
      if(!isTrackFiltered(index(i,0,albumIdx))) continue;
      ModelItem *trackItem = getItem( index(i,0,albumIdx) );  
      if (!QRegExp(genre, Qt::CaseSensitive).exactMatch(trackItem->getData(4).toString())) continue;
      result.append( trackItem->getData(2).toString() );
    }
    return result;
}

     
//! ----------------------- Favorite status method -----------------------------
bool CollectionModel::itemIsFavorite(const QModelIndex &index) 
{
  ModelItem *item = getItem(index);
  return item->isFavorite();
}


void CollectionModel::setItemIsFavorite(const QModelIndex &index, bool isfavorite) 
{
  ModelItem *item = getItem(index);
  item->setIsFavorite(isfavorite);
  emit signalFavoriteStatusChanged();
}


//! ----------------------- playing status method ------------------------------

bool CollectionModel::itemIsPlaying(const QModelIndex &index) 
{
  ModelItem *item = getItem(index);
  return item->isPlaying();
}


void CollectionModel::setItemIsPlaying(const QModelIndex &index, bool isplaying) 
{
  ModelItem *item = getItem(index);
  item->setIsPlaying(isplaying);
  emit signalPlayingStatusChanged();
}


QModelIndex CollectionModel::getPlayingTrackItem() 
{    
  if(_playingItemIdx.isValid())
    return _playingItemIdx;

  return QModelIndex();
}






QString CollectionModel::getPlayingTrackItemGenre() 
{
  QModelIndex trackIdx = getPlayingTrackItem();
  if(trackIdx.isValid())
    return getItem(trackIdx)->getData(4).toString();
  
  return QString(); 
}

void CollectionModel::updatePlayingItem(const QString filename)
{
  for ( int i = 0; i < rowCount ( QModelIndex() ); i++ )
   {
     QModelIndex artistItemIdx = index(i, 0, QModelIndex());
     for (int j = 0; j < (getItem(artistItemIdx))->childCount(); j++)
     {
       QModelIndex albumItemIdx = index ( j, 0, artistItemIdx);
       for (int k = 0; k < (getItem(albumItemIdx))->childCount(); k++)
       {
         QModelIndex trackItemIdx = index ( k, 0, albumItemIdx);
         ModelItem *trackItem = getItem(trackItemIdx);
	 
         if (filename == trackItem->getData(2).toString()) 
         //if (QRegExp(filename, Qt::CaseSensitive).exactMatch(trackItem->getData(2).toString())) 
         {
           _playingItemIdx = trackItemIdx;
           trackItem->setIsPlaying(true);
           //! update all track parents
           ModelItem *albumItem = trackItem->parent();
           albumItem->setIsPlaying(true);
           
           ModelItem *artistItem = albumItem->parent();
           artistItem->setIsPlaying(true);
           qDebug() << " ---- CollectionModel::updatePlayingItem emit signalPlayingStatusChanged";

           emit signalPlayingStatusChanged();
           return;
          }
      } // end for track
    } // end for album
  }  // end for artist
}

void CollectionModel::stopPlayingItem()
{
 QModelIndex trackIdx = getPlayingTrackItem();
 if(!trackIdx.isValid()) return;

 ModelItem *trackItem = getItem(trackIdx);
 trackItem->setIsPlaying(false);

 //! update all track parents
 ModelItem *albumItem = trackItem->parent();
 albumItem->setIsPlaying(false);
  
 ModelItem *artistItem = albumItem->parent();
 artistItem->setIsPlaying(false);
 
 _playingItemIdx = QModelIndex();
 emit signalPlayingStatusChanged();
}


//! --------------------------- genre method -----------------------------------
bool CollectionModel::isArtistGenre(const QModelIndex &artistIdx, QString genre)
{
  if (!artistIdx.isValid()) return false;
    
  ModelItem *artistItem = getItem(artistIdx);

  for (int i=0; i < artistItem->childCount() ; i++) 
  {
    QModelIndex albumIdx = index ( i, 0, artistIdx);
    ModelItem *albumItem = getItem(albumIdx);

    for (int j=0; j < albumItem->childCount() ; j++) 
    {
      QModelIndex trackIdx = index ( j, 0, albumIdx);
      ModelItem *trackItem = getItem(trackIdx);

      //! check item
      if (QRegExp(genre, Qt::CaseSensitive).exactMatch(trackItem->getData(4).toString())) 
        return true;
    }
  }  
  return false;    
}
  
bool CollectionModel::isAlbumGenre(const QModelIndex &albumIdx, QString genre)
{
  if (!albumIdx.isValid()) return false;
    
  for (int i=0; i < rowCount(albumIdx) ; i++) 
  {
    QModelIndex trackIdx = index ( i, 0, albumIdx);
    ModelItem *trackItem = getItem(trackIdx);
      
    //! check item
    if (QRegExp(genre, Qt::CaseSensitive).exactMatch(trackItem->getData(4).toString())) 
      return true;
  }
  return false;    
}


//! ------------------------- filtering method ---------------------------------
bool CollectionModel::isArtistFiltered(const QModelIndex &artistIdx) 
{
  if (!artistIdx.isValid()) return false;
    
  ModelItem *artistItem = getItem(artistIdx);
  
  //! check artist item 
  if (artistItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive))
    return true;

  //! check child album
  for (int i = 0; i < rowCount(artistIdx); i++) 
  {
    QModelIndex albumIdx = index ( i, 0, artistIdx);
    ModelItem *albumItem = getItem(albumIdx);
   
    if (albumItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive))
      return true;

    //! check child track
    for (int j = 0; j < rowCount(albumIdx); j++)
    {
      QModelIndex trackIdx = index ( j, 0, albumIdx);
      ModelItem *trackItem = getItem(trackIdx);
      if ( trackItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive) ||
           trackItem->getData(4).toString().contains(_filterPattern, Qt::CaseInsensitive) )
        return true;
     }
  }

  return false;
}

bool CollectionModel::isAlbumFiltered(const QModelIndex &albumIdx) 
{
  if (!albumIdx.isValid()) return false;
    
  ModelItem *albumItem = getItem(albumIdx);
  
  //! check album item 
  if (albumItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive))
    return true;
    
  //! check artist parent
  ModelItem *artistItem = albumItem->parent();
  if (artistItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive))
      return true;
  
  //! check child track
  for (int i = 0; i < rowCount(albumIdx); i++)
  {
    QModelIndex trackIdx = index ( i, 0, albumIdx);
    ModelItem *trackItem = getItem(trackIdx);
    if ( trackItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive) ||
         trackItem->getData(4).toString().contains(_filterPattern, Qt::CaseInsensitive) )
      return true;
  }

  return false;
}


bool CollectionModel::isTrackFiltered(const QModelIndex &trackIdx) 
{
  if (!trackIdx.isValid()) return false;
    
  ModelItem *trackItem = getItem(trackIdx);
  
  //! check track item 
  if ( trackItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive) ||
       trackItem->getData(4).toString().contains(_filterPattern, Qt::CaseInsensitive) )
    return true;
    
  //! check album parent
  ModelItem *albumItem = trackItem->parent();
  if (albumItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive))
      return true;
  
  //! check artist parent
  ModelItem *artistItem = albumItem->parent();
  if (artistItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive))
      return true;

  return false;
}




bool CollectionModel::isArtistGenreFiltered(const QModelIndex &artistIdx, QString genre) 
{
  if (!artistIdx.isValid()) return false;
    
  ModelItem *artistItem = getItem(artistIdx);
  
  //! check artist item 
  if (artistItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive))
    return true;

  //! check child album
  for (int i = 0; i < rowCount(artistIdx); i++) 
  {
    QModelIndex albumIdx = index ( i, 0, artistIdx);
    ModelItem *albumItem = getItem(albumIdx);
   
    if (albumItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive))
      return true;

    //! check child track
    for (int j = 0; j < rowCount(albumIdx); j++)
    {
      QModelIndex trackIdx = index ( j, 0, albumIdx);
      ModelItem *trackItem = getItem(trackIdx);
      if ( (trackItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive) ||
           trackItem->getData(4).toString().contains(_filterPattern, Qt::CaseInsensitive) ) &&
           (QRegExp(genre, Qt::CaseInsensitive).exactMatch(trackItem->getData(4).toString())))
        return true;
     }
  }

  return false;
}

bool CollectionModel::isAlbumGenreFiltered(const QModelIndex &albumIdx, QString genre) 
{
  if (!albumIdx.isValid()) return false;
    
  ModelItem *albumItem = getItem(albumIdx);
  
  //! check album item 
  if (albumItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive))
    return true;
    
  //! check artist parent
  ModelItem *artistItem = albumItem->parent();
  if (artistItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive))
      return true;
  
  //! check child track
  for (int i = 0; i < rowCount(albumIdx); i++)
  {
    QModelIndex trackIdx = index ( i, 0, albumIdx);
    ModelItem *trackItem = getItem(trackIdx);
    if ( (trackItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive) ||
          trackItem->getData(4).toString().contains(_filterPattern, Qt::CaseInsensitive) ) &&
         (QRegExp(genre, Qt::CaseInsensitive).exactMatch(trackItem->getData(4).toString())))
      return true;
  }

  return false;
}
