/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionModel/plsfilemodel.h"

#include <QDebug>
#include <QRegExp>

//! ----- global instance ------------------------------------------------------
PlsFileModel* PlsFileModel::globalPlsFileModel = 0;

namespace GLOBAL
{
    PlsFileModel* plsFileModel()
    {
        return PlsFileModel::instance();
    }
}


//! ----- PlsFileModel ---------------------------------------------------------
//PlsFileModel::PlsFileModel(const QStringList &headers, QObject *parent) : QAbstractItemModel(parent)
PlsFileModel::PlsFileModel(QObject *parent) : QAbstractItemModel(parent)
{
  globalPlsFileModel = this;

  //! root item       
  QVector<QVariant> rootData;
  //foreach (QString header, headers)
  rootData << "id" << "name" << "data1" << "data2" << "data3";
  _rootItem = new ModelItem(rootData);
  setSupportedDragActions(Qt::CopyAction);
  
  _filterPattern = "";
}

PlsFileModel::~PlsFileModel()
{
    delete _rootItem;
}



int PlsFileModel::columnCount(const QModelIndex & /* parent */) const
{
    return _rootItem->columnCount();
}

QVariant PlsFileModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    if (role != Qt::DisplayRole && role != Qt::EditRole)
        return QVariant();

    ModelItem *item = getItem(index);
    return item->getData(index.column());
}

Qt::ItemFlags PlsFileModel::flags(const QModelIndex &index) const
{
    if (!index.isValid())
        return 0;

    return Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled;
}

ModelItem *PlsFileModel::getItem(const QModelIndex &index) const
{
    if (index.isValid()) {
        ModelItem *item = static_cast<ModelItem*>(index.internalPointer());
        if (item) return item;
    }
    return _rootItem;
}

QVariant PlsFileModel::headerData(int section, Qt::Orientation orientation,
                                     int role) const
{
    if (orientation == Qt::Horizontal && role == Qt::DisplayRole)
        return _rootItem->data(section);

    return QVariant();
}

QModelIndex PlsFileModel::index(int row, int column, const QModelIndex &parent) const
{
    if (parent.isValid() && parent.column() != 0)
        return QModelIndex();

    ModelItem *parentItem = getItem(parent);

    ModelItem *childItem = parentItem->child(row);
    if (childItem)
        return createIndex(row, column, childItem);
    else
        return QModelIndex();
}

bool PlsFileModel::insertRows(int position, int rows, const QModelIndex &parent)
{
  //qDebug() << "insertRows:";
  ModelItem *parentItem = getItem(parent);
  bool success;
  
  beginInsertRows(parent, position, position + rows - 1);
  success = parentItem->insertChildren(position, rows, _rootItem->columnCount());
  endInsertRows();

  return success;
}

bool PlsFileModel::removeRows(int position, int rows, const QModelIndex &parent)
{
    ModelItem *parentItem = getItem(parent);
    bool success = true;

    beginRemoveRows(parent, position, position + rows - 1);
    success = parentItem->removeChildren(position, rows);
    endRemoveRows();

    return success;
}

QModelIndex PlsFileModel::parent(const QModelIndex &index) const
{
  if (!index.isValid())
    return QModelIndex();
  
  ModelItem *childItem = getItem(index);
  ModelItem *parentItem = childItem->parent();
  
  if (parentItem == _rootItem)
    return QModelIndex();
  
  return createIndex(parentItem->childNumber(), 0, parentItem);
}


int PlsFileModel::rowCount(const QModelIndex &parent) const
{
  ModelItem *parentItem = getItem(parent);
  
  return parentItem->childCount();
}

bool PlsFileModel::setData(const QModelIndex &index, const QVariant &value,int role)
{
  if (role != Qt::EditRole)
    return false;
  
  ModelItem *item = getItem(index);
  bool result = item->setData(index.column(), value);
  
  if (result)
    emit dataChanged(index, index);
  
  return result;
}



bool PlsFileModel::setHeaderData(int section, Qt::Orientation orientation, const QVariant &value, int role)
{
  if (role != Qt::EditRole || orientation != Qt::Horizontal)
    return false;
  
  bool result = _rootItem->setData(section, value);

  if (result)
    emit headerDataChanged(orientation, section, section);
  
  return result;
}


QModelIndex PlsFileModel::addChild(const QModelIndex &parent, const QVector<QVariant> &data)
{
  //qDebug() << "addChild:";
  if (!insertRow(rowCount(parent), parent)) {
    return QModelIndex();
  }

  ModelItem *item = getItem(parent);
  int childCount = item->childCount()-1;

  // WARNING QVector can have different size
  for(int i=0; i < data.size() ; i++) {
    item->child(childCount)->setData(i, data.value(i));
  }
  
  item->child(childCount)->setIsPlaying(false);
  item->child(childCount)->setIsFavorite(false);
  
  return index(rowCount(parent)-1,0,parent);
}


void PlsFileModel::clear()
{
    removeRows(0,rowCount(QModelIndex()),QModelIndex());
    emit modelCleared();
}

//! ----------------------- Get urls method ----------------------------------
QStringList PlsFileModel::getItemChildrenUrls(const QModelIndex &itemIdx)
{
    QStringList result;

    ModelItem *item = getItem(itemIdx);
    if (item->childCount() == 0) { 
      if(isUrlFiltered(itemIdx))
	result.append( item->getData(2).toString() );    
    } 
    else {
      for (int i = 0; i < item->childCount(); i++) {
        result.append(getItemChildrenUrls(index(i,0,itemIdx)));
      }
    }
    return result;
}

     
//! ----------------------- Favorite status method -----------------------------
bool PlsFileModel::itemIsFavorite(const QModelIndex &index) 
{
  ModelItem *item = getItem(index);
  return item->isFavorite();
}


void PlsFileModel::setItemIsFavorite(const QModelIndex &index, bool isfavorite) 
{
  ModelItem *item = getItem(index);
  item->setIsFavorite(isfavorite);
  emit signalFavoriteStatusChanged();
}


//! ----------------------- playing status method ------------------------------

bool PlsFileModel::itemIsPlaying(const QModelIndex &index) 
{
  ModelItem *item = getItem(index);
  return item->isPlaying();
}


void PlsFileModel::setItemIsPlaying(const QModelIndex &index, bool isplaying) 
{
  ModelItem *item = getItem(index);
  item->setIsPlaying(isplaying);
  emit signalPlayingStatusChanged();
}


QModelIndex PlsFileModel::getPlayingUrlItem() 
{
   for ( int i = 0; i < rowCount ( QModelIndex() ); i++ )
   {
     QModelIndex playlistItemIdx = index(i, 0, QModelIndex());
     for (int j = 0; j < (getItem(playlistItemIdx))->childCount(); j++)
     {
       QModelIndex ItemIdx = index ( j, 0, playlistItemIdx);
       if(itemIsPlaying(ItemIdx))
	 return ItemIdx;
     } // end for playlist item
  } // end for playlist

  return QModelIndex();
}

void PlsFileModel::updatePlayingItem(const QString url)
{
  for ( int i = 0; i < rowCount ( QModelIndex() ); i++ )
   {
     QModelIndex playlistItemIdx = index(i, 0, QModelIndex());
     for (int j = 0; j < (getItem(playlistItemIdx))->childCount(); j++)
     {
       QModelIndex trackItemIdx = index ( j, 0, playlistItemIdx);
       ModelItem *trackItem = getItem(trackItemIdx);
       if (url == trackItem->getData(2).toString()) 
       {
         trackItem->setIsPlaying(true);
         //! update parent
         (getItem(playlistItemIdx))->setIsPlaying(true);
         emit signalPlayingStatusChanged();
         return;         
      }
    } // end for track
  }  // end for playlist
}

void PlsFileModel::stopPlayingItem()
{
 QModelIndex trackIdx = getPlayingUrlItem();
 if(!trackIdx.isValid()) return;

 ModelItem *trackItem = getItem(trackIdx);
 trackItem->setIsPlaying(false);

 //! update playlist parents
 ModelItem *playlistItem = trackItem->parent();
 playlistItem->setIsPlaying(false);
 
 emit signalPlayingStatusChanged();
}


 

//! ------------------------- filtering method ---------------------------------
bool PlsFileModel::isPlaylistFiltered(const QModelIndex &playlistIdx) 
{
  if (!playlistIdx.isValid()) return false;
    
  ModelItem *playlistItem = getItem(playlistIdx);
  
  //! check playlist name
  if (playlistItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive))
    return true;

  //! check playlist child (url name)
  for (int i = 0; i < rowCount(playlistIdx); i++) 
  {
      QModelIndex trackIdx = index ( i, 0, playlistIdx);
      ModelItem *trackItem = getItem(trackIdx);
      if ( trackItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive) )
        return true;
   }

  return false;
}



bool PlsFileModel::isUrlFiltered(const QModelIndex &trackIdx) 
{
  if (!trackIdx.isValid()) return false;
    
  ModelItem *trackItem = getItem(trackIdx);
  
  //! check track item 
  if ( trackItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive) )
    return true;
    
  //! check playlist parent name
  ModelItem *playlistItem = trackItem->parent();
  if (playlistItem->getData(1).toString().contains(_filterPattern, Qt::CaseInsensitive))
      return true;
 
  return false;
}
