/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionView/collectionscene.h"
#include "CollectionView/userchangedialog.h"
#include "CollectionModel/collectionmodel.h"
#include "CollectionModel/plsfilemodel.h"
#include "MediaItem/mediaitem.h"
#include "MediaItem/mediaplaylist.h"
#include "Database/databasemanager.h"
#include "Web/coversearch.h"
#include "constants.h"

#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>
#include <QVariant>

namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}


//! --------------------- Collection Scene constructor -------------------------
CollectionScene::CollectionScene(QWidget *parent)
{
    _parentWidget = parent;
    this->setParent(_parentWidget);
    this->setItemIndexMethod(QGraphicsScene::NoIndex);
 
    //! set model  
    _collectionModel = CollectionModel::instance();  
    _plsFileModel    = PlsFileModel::instance();
    connect(_collectionModel, SIGNAL(signalPlayingStatusChanged()), this, SLOT(update()));
    
    //! set player  
    _player = Player::instance();
    QObject::connect(_player, SIGNAL(trackChanged(QString)), this, SLOT(updateTrackPlaying(QString))); 

    _infoSize            = 0;
    _mouseGrabbedItem    = 0;
  
    //! Actions 
    QAction *albumPlayAct = new QAction(QIcon(":/images/media-play.png"), tr("Play Album"), this); 
    QAction *changeCoverAct = new QAction(QIcon(":/images/covericon.png"), tr("Load Cover Art from File"), this); 
    QAction *loadCoverLastFm = new QAction(QIcon(":/images/lastfm.png"), tr("Load Cover Art from LastFm"), this); 
    QAction *removeCoverAct = new QAction(QIcon::fromTheme("edit-delete"), tr("Remove Cover Art"), this); 
    QAction *modifyAlbumAct = new QAction(QIcon(":/images/info-modify.png"), tr("Modify Database Album Info"), this); 
    albumFavoriteAct = new QAction(QIcon(":/images/media-favorite.png"), tr("Add Album to favorites"), this); 

    QAction *albumGenrePlayAct = new QAction(QIcon(":/images/media-play.png"), tr("Play Album"), this); 
    QAction *modifyGenreAct = new QAction(QIcon(":/images/info-modify.png"), tr("Modify Album Genre Info"), this);

    QAction *artistPlayAct = new QAction(QIcon(":/images/media-play.png"), tr("Play Artist"), this); 
    QAction *modifyArtistAct = new QAction(QIcon(":/images/info-modify.png"), tr("Modify Database Artist Info"), this); 
    artistFavoriteAct = new QAction(QIcon(":/images/media-favorite.png"), tr("Add Artist to favorites"), this); 

    QAction *playlistPlayAct = new QAction(QIcon(":/images/media-play.png"), tr("Play Playlist"), this);
    QAction *removePlaylistAct = new QAction(QIcon::fromTheme("edit-delete"),tr("&Remove playlist from disk"), this);
    QAction *modifyPlaylistAct = new QAction(QIcon(":/images/info-modify.png"), tr("Modify Database Playlist Info"), this);
    playlistFavoriteAct = new QAction(QIcon(":/images/media-favorite.png"), tr("Add Playlist to favorites"), this); 

    //! gestion du double click par action globale et data
    QAction *itemDoubleClick = new QAction(this); 
    QObject::connect(itemDoubleClick, SIGNAL(triggered()), this, SLOT(itemDoubleClick()), Qt::DirectConnection);
    QMap<QString, QAction*> *actions = GLOBAL::globalActions();  
    actions->insert("itemDoubleClick", itemDoubleClick);  
  
    //! Connections
    QObject::connect(albumPlayAct, SIGNAL(triggered()), this, SLOT(playAlbum()), Qt::DirectConnection);
    QObject::connect(changeCoverAct, SIGNAL(triggered()), this, SLOT(loadCoverArtFromFile()), Qt::DirectConnection);
    QObject::connect(loadCoverLastFm, SIGNAL(triggered()), this, SLOT(loadLastFmCover()), Qt::DirectConnection);   
    QObject::connect(removeCoverAct, SIGNAL(triggered()), this, SLOT(removeCoverArt()), Qt::DirectConnection);
    QObject::connect(modifyAlbumAct, SIGNAL(triggered()), this, SLOT(modifyAlbumInfo()), Qt::DirectConnection);
    QObject::connect(albumFavoriteAct, SIGNAL(triggered()), this, SLOT(updateAlbumFavorite()), Qt::DirectConnection);

    QObject::connect(albumGenrePlayAct, SIGNAL(triggered()), this, SLOT(playAlbumGenre()), Qt::DirectConnection);
    QObject::connect(modifyGenreAct, SIGNAL(triggered()), this, SLOT(modifyGenreInfo()), Qt::DirectConnection);

    QObject::connect(artistPlayAct, SIGNAL(triggered()), this, SLOT(playArtist()));
    QObject::connect(modifyArtistAct, SIGNAL(triggered()), this, SLOT(modifyArtistInfo()));
    QObject::connect(artistFavoriteAct, SIGNAL(triggered()), this, SLOT(updateArtistFavorite()));

    QObject::connect(playlistPlayAct, SIGNAL(triggered()), this, SLOT(playPlaylist()));
    QObject::connect(removePlaylistAct, SIGNAL(triggered()), this, SLOT(removePlaylistFromDisk()));
    QObject::connect(modifyPlaylistAct, SIGNAL(triggered()), this, SLOT(modifyPlaylistInfo()));
    QObject::connect(playlistFavoriteAct, SIGNAL(triggered()), this, SLOT(updatePlaylistFavorite()));
       
    //! menu for album graphic item
    _menuAlbum      = new QMenu();
    _menuAlbum->addAction(albumPlayAct);
    _menuAlbum->addSeparator();
    _menuAlbum->addAction(changeCoverAct);
    _menuAlbum->addAction(loadCoverLastFm);
    _menuAlbum->addAction(removeCoverAct);
    _menuAlbum->addAction(modifyAlbumAct);
    _menuAlbum->addSeparator(); 
    _menuAlbum->addAction(albumFavoriteAct);
   
    //! menu for album genre graphic item
    _menuAlbumGenre = new QMenu();
    _menuAlbumGenre->addAction(albumGenrePlayAct);
    _menuAlbumGenre->addSeparator();
    _menuAlbumGenre->addAction(changeCoverAct);
    _menuAlbumGenre->addAction(loadCoverLastFm);
    _menuAlbumGenre->addAction(removeCoverAct);
    _menuAlbumGenre->addAction(modifyGenreAct);
  
    //! menu for artist graphic item  
    _menuArtist     = new QMenu();
    _menuArtist->addAction(artistPlayAct);
    _menuArtist->addSeparator();
    _menuArtist->addAction(modifyArtistAct);    
    _menuArtist->addSeparator();
    _menuArtist->addAction(artistFavoriteAct);      

    //! menu for artist graphic item  
    _menuArtist     = new QMenu();
    _menuArtist->addAction(artistPlayAct);
    _menuArtist->addSeparator();
    _menuArtist->addAction(modifyArtistAct);    
    _menuArtist->addSeparator();
    _menuArtist->addAction(artistFavoriteAct);        
  
    //! menu for playlist graphic item  
    _menuPlaylist = new QMenu();
    _menuPlaylist->addAction(playlistPlayAct);
    _menuPlaylist->addSeparator();
    _menuPlaylist->addAction(removePlaylistAct);
    _menuPlaylist->addAction(modifyPlaylistAct);
    _menuPlaylist->addAction(playlistFavoriteAct);
}   

CollectionScene::~CollectionScene()
{
  clear(); 
}   

void CollectionScene::setVirtualPlaylist(VirtualPlaylist  *playlist)
{
  _virtualPlaylist = playlist;
  _virtualPlaylist->StopPlaying();
}

//! --------------------- playing status method -------------------------------- 
void CollectionScene::updateTrackPlaying(QString filename)
{  
  qDebug() << " ---- CollectionScene::updateTrackPlaying";
  if(!filename.isEmpty()) {
      if(MEDIA::isUrlLocal(filename)) {
        _collectionModel->stopPlayingItem();
        _collectionModel->updatePlayingItem(QFileInfo(filename).absoluteFilePath());
      }
      
      // pour le cas de la playlist on autorise la lecture de track locale ou de stream
      _plsFileModel->stopPlayingItem();
      _plsFileModel->updatePlayingItem(filename);
  }
  else {  
     _collectionModel->stopPlayingItem();
     _plsFileModel->stopPlayingItem();
    _virtualPlaylist->StopPlaying();
  }
}


//! --------------------- playing method ---------------------------------------
void CollectionScene::playAlbum() 
{
  qDebug() << " ---- playAlbum";
  if(!_mouseGrabbedItem) return;
  AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);
  
  QStringList _list = _collectionModel->getItemChildrenTracks(item->_modelIdx);

  if(_list.isEmpty()) return;
  _virtualPlaylist->AppendFilesInPlaylist(_list);
}

void CollectionScene::playAlbumGenre()
{
  qDebug() << " ---- playAlbumGenre";
  if(!_mouseGrabbedItem) return;
  AlbumGenreGraphicItem *item = static_cast<AlbumGenreGraphicItem*>(_mouseGrabbedItem);
  
  QStringList _list = _collectionModel->getAlbumChildrenTracksGenre(item->_modelIdx,item->_genre);

  if(_list.isEmpty()) return;
  _virtualPlaylist->AppendFilesInPlaylist(_list);
}

void CollectionScene::playArtist() 
{
  qDebug() << " ---- playArtist";
  if(!_mouseGrabbedItem) return;
  ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(_mouseGrabbedItem);
  
  QStringList _list = _collectionModel->getItemChildrenTracks(item->_modelIdx);

  if(_list.isEmpty()) return;
  _virtualPlaylist->AppendFilesInPlaylist(_list);
}


void CollectionScene::playTrack() 
{
  qDebug() << " ---- playTrack";
  if(!_mouseGrabbedItem) return;
  TrackGraphicItem *item = static_cast<TrackGraphicItem*>(_mouseGrabbedItem);
  
  //! recup de la liste des pistes à partir de celle selectionnée
  QStringList _urlList;
  for (int i = 0; i<_listTrack.size();i++)
    _urlList.append(_collectionModel->getItemChildrenTracks(_listTrack[i]->_modelIdx));

  int trackIndex = _listTrack.indexOf(item);

  if( trackIndex >=0 && trackIndex < _urlList.size() ) {
    _virtualPlaylist->AppendFilesInPlaylist(_urlList, trackIndex);
  } 
}    

void CollectionScene::playPlaylist() 
{
 if(!_mouseGrabbedItem) return;
 PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);
  
 QStringList _list = _plsFileModel->getItemChildrenUrls(item->_modelIdx);
   
  if(_list.isEmpty()) return;
  _virtualPlaylist->AppendFilesInPlaylist(_list);
}


//! --------------------- Collection Scene Populator ---------------------------
void CollectionScene::populateScene() 
{
  //! clear scene and delete all items
  clear();
  switch(_collectionMode) {
    case (Collection::ViewAlbum) : populateAlbumScene(); break;
    case (Collection::ViewArtist) : populateArtistScene(); break;
    case (Collection::ViewTrack) : populateTrackScene(); break;
    case (Collection::ViewGenre) : populateGenreScene(); break;
    case (Collection::ViewFavorite) : populateFavoriteScene(); break;
    case (Collection::ViewPlaylist) : populatePlaylistScene(); break;
    case (Collection::ViewPlaylistDetail) : populatePlaylistDetailScene(); break;
    default: break;
  } 
  
  //! we need to ajust SceneRect 
  int diff = _parentWidget->height() - itemsBoundingRect().height() + 40;
  int ajust  = ( diff > 0) ? diff : 0; 
  setSceneRect ( itemsBoundingRect().adjusted(-10, -10, +10, +10 + ajust) );
  
  qDebug() << " ---- CollectionScene -> End PopulateScene ----";
}


void CollectionScene::populateAlbumScene() 
{
  //qDebug() << " ----Collection = album view ----";

  int albumRow  = 0;
  int artistRow = 0;
  int Column    = 0;
  int idx       = 0;
  _infoSize     = 0;
  
  //! categories list (for up/down navigation)
  _listCategorie.clear();

  //! artist loop
  for ( int i = 0; i < _collectionModel->rowCount ( QModelIndex() ); i++ ) 
  {
    QModelIndex artistIdx = _collectionModel->index ( i, 0, QModelIndex() );
    
    if(!_collectionModel->isArtistFiltered(artistIdx) ) continue;
    
    ModelItem *artistItem = _collectionModel->getItem(artistIdx);
    
    CategorieGraphicItem *_artist = new CategorieGraphicItem();
    _artist->_catName = artistItem->getData(1).toString();
    _artist->setPos( 0 ,10 + artistRow*50 + albumRow*170);
    addItem(_artist);
    _listCategorie.append(_artist);
    
    Column = 0;
    artistRow++;
    idx = 0;

    //! album loop
    for (int j = 0; j < artistItem->childCount(); j++) 
    {
      QModelIndex albumIdx = _collectionModel->index ( j, 0, artistIdx);
      
      if(!_collectionModel->isAlbumFiltered(albumIdx) )	continue;
      
      idx++;
      _infoSize++;
      
      AlbumGraphicItem *_album = new AlbumGraphicItem();
      _album->_modelIdx = albumIdx;
      _album->setPos(4+160*Column, artistRow*50 + albumRow*170);
      addItem(_album);
      
      if(Column <3) {
        Column++;
      } 
      else {
        Column = 0;
        albumRow++;
        idx=0;
      }
    }
    if(idx>0) albumRow++;
  }
}


void CollectionScene::populateArtistScene()
{
  //qDebug() << " ----CollectionScene -> Artist view ----";

  QStringList alphaList = QStringList() 
  << "a" << "b" << "c" << "d" << "e" << "f" << "g" << "h" << "i" 
  << "j" << "k" << "l" << "m" << "n" << "o" << "p" << "q" << "r"
  << "s" << "t" << "u" << "v" << "w" << "x" << "y" << "z" ;
     
  int artistRow    = 0;
  int categorieRow = 0;
  int Column       = 0;
  _infoSize        = 0;  
  QStringList coverPathList = QStringList();

  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  //! alpha loop
  int i=0;
  foreach (const QString &alpha, alphaList)
  {
      bool first = true;
      int idx    = 0;
    
      //! artist loop
      for ( ; i < _collectionModel->rowCount ( QModelIndex() ); i++ ) {
      
        QModelIndex artistIdx = _collectionModel->index ( i, 0, QModelIndex() );
        ModelItem *artistItem = _collectionModel->getItem(artistIdx);	  
        
        if(! (artistItem->getData(1).toString()).startsWith(alpha, Qt::CaseInsensitive)) break;
        if(! _collectionModel->isArtistFiltered(artistIdx) ) continue;
      
        idx++;
        if(first) {
          CategorieGraphicItem *_cat = new CategorieGraphicItem();
          _cat->_catName = alpha[0];
          _cat->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
          addItem(_cat);
          _listCategorie.append(_cat);
          
          Column = 0;
          categorieRow++;
          first = false;
        }
        _infoSize++;
	
        //! New Artist Item
        ArtistGraphicItem *_artist = new ArtistGraphicItem();
        _artist->_modelIdx = artistIdx;
        _artist->setPos( 4 + 160*Column , categorieRow*50 + artistRow*165);
        addItem(_artist);
      
        if(Column <3) {
          Column++;
        }
        else {
          Column = 0;
          artistRow++;
          idx=0;
        }
        //! ALBUM LOOP -> set Album cover path list
        for (int j = 0; j < artistItem->childCount(); j++) {
          QModelIndex albumIdx = _collectionModel->index ( j, 0, artistIdx);
          if(!_collectionModel->isAlbumFiltered(albumIdx) ) continue;
          
          ModelItem *albumItem = _collectionModel->getItem(albumIdx);
          coverPathList << albumItem->getData(3).toString();
        }
   	
        artistItem->setData(2, coverPathList);
        coverPathList.clear();

      } // end artist loop
      if(idx>0) artistRow++;
  }  // end alpha loop
}


void CollectionScene::populateTrackScene() 
{
  //qDebug() << " ----CollectionScene -> Track view ----";
  
  int artistRow     = 0;
  int albumRow      = 0;
  int trackRow      = 0;
  int trackPerAlbum = 0;
  int offset        = 0;
  _infoSize         = 0;
  
  //! track list for virtual playlist
  _listTrack.clear();

  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  //! artist loop
  for ( int i = 0; i < _collectionModel->rowCount ( QModelIndex() ); i++ ) 
  {
    QModelIndex artistIdx = _collectionModel->index ( i, 0, QModelIndex() );
    
    if(!_collectionModel->isArtistFiltered(artistIdx) ) continue;
    
    ModelItem *artistItem = _collectionModel->getItem(artistIdx);
    
    CategorieGraphicItem *_artist = new CategorieGraphicItem();
    _artist->_catName = artistItem->getData(1).toString();
    _artist->setPos( 10 ,artistRow*50 + trackRow*20 + albumRow*30 + offset );
    addItem(_artist);
    _listCategorie.append(_artist);
    
    artistRow++;
    
    //! album loop
    for (int j = 0; j < artistItem->childCount(); j++) 
    {
      QModelIndex albumIdx = _collectionModel->index ( j, 0, artistIdx);
      
      if(!_collectionModel->isAlbumFiltered(albumIdx)) continue;
      
      ModelItem *albumItem = _collectionModel->getItem(albumIdx);
      
      AlbumGraphicItem *_album = new AlbumGraphicItem();
      _album->_modelIdx = albumIdx;
      _album->setPos(10, artistRow*50 + trackRow*20 + albumRow*30 + offset );
      addItem(_album);
      
      trackPerAlbum = 0;
      
      //! track loop
      for (int k = 0; k < albumItem->childCount(); k++) 
      {
        QModelIndex trackIdx = _collectionModel->index ( k, 0, albumIdx);
        if(!_collectionModel->isTrackFiltered(trackIdx)) continue;
	
        TrackGraphicItem *_track = new TrackGraphicItem();
        _track->_modelIdx = trackIdx;
        _track->setPos(180, artistRow*50 + trackRow*20 + albumRow*30 + offset);
        addItem(_track); 
        _listTrack.append(_track);

        trackRow++;
        trackPerAlbum++;
        _infoSize++;
      }
      albumRow++;
      
      if( trackPerAlbum < 8 )	offset = offset + (8 - trackPerAlbum)*20;
    }
  }  
}


void CollectionScene::populateGenreScene()
{
  //qDebug() << " ----CollectionScene -> Genre view ----";

  int categorieRow = 0;
  int albumRow     = 0;
  int Column       = 0;
  _infoSize        = 0;
  
  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  //! GENRE loop
  foreach (const QString &genre, _collectionModel->_listGenre) 
  {
    bool first = true;
    Column     = 0;
    int idx    = 0;

    //! ARTIST loop
    for ( int i = 0; i < _collectionModel->rowCount ( QModelIndex() ); i++ )
    {
      QModelIndex artistIdx = _collectionModel->index ( i, 0, QModelIndex() );
      
      if(!_collectionModel->isArtistGenre(artistIdx,genre) ) continue;
      if(!_collectionModel->isArtistGenreFiltered(artistIdx,genre) ) continue;
      
      //! ALBUM loop
      for ( int j = 0; j < _collectionModel->rowCount ( artistIdx ); j++ )
      {
        QModelIndex albumIdx = _collectionModel->index ( j, 0, artistIdx );
        
        if(!_collectionModel->isAlbumGenre(albumIdx,genre) ) continue;
        if(!_collectionModel->isAlbumGenreFiltered(albumIdx,genre) ) continue;
        
        if(first) {
          CategorieGraphicItem *_cat = new CategorieGraphicItem();
          _cat->_catName = genre;
          _cat->setPos( 0 , 10 + categorieRow*50 + albumRow*170);
          addItem(_cat);
          
          _listCategorie.append(_cat);
	  
          categorieRow++;
          first = false;
          _infoSize++; // on compte les categorie = genre
        }
	
        AlbumGenreGraphicItem *_album = new AlbumGenreGraphicItem();
        _album->_modelIdx = albumIdx;
        _album->_genre = genre;
        _album->setPos(4+160*Column, categorieRow*50 + albumRow*170);
        addItem(_album);
              
        if(Column <3) {
          Column++;
          idx++; 
        }
        else {
          Column = 0;
          albumRow++;
          idx=0;
        }
     } // end album loop
    } // end artist loop
    if(idx>0) albumRow++;    
  }  // end genre loop
}


void CollectionScene::populateFavoriteScene()
{
  //qDebug() << " ----CollectionScene -> Favorite view ----";

  int artistRow    = 0;
  int albumRow     = 0;
  int categorieRow = 0;
  int Column       = 0;
  _infoSize        = 0;  
  
  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  QStringList coverPathList = QStringList();
  bool first = true;
    
  //! ------ Favorite Artists ------
  for ( int i = 0; i < _collectionModel->rowCount ( QModelIndex() ); i++ ) {
      
    QModelIndex artistIdx = _collectionModel->index ( i, 0, QModelIndex() );
    ModelItem *artistItem = _collectionModel->getItem(artistIdx);	  

    if(! _collectionModel->itemIsFavorite(artistIdx) ) continue;
    if(! _collectionModel->isArtistFiltered(artistIdx) ) continue;
      
    if(first) {
      CategorieGraphicItem *_cat = new CategorieGraphicItem();
      _cat->_catName = "Favorite Artists";
      _cat->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
      addItem(_cat);
      
      _listCategorie.append(_cat);
          
      Column = 0;
      categorieRow++;
      first = false;
    }
    _infoSize++; // we count artist favorite item 
	
    //! New Artist Item
    ArtistGraphicItem *_artist = new ArtistGraphicItem();
    _artist->_modelIdx = artistIdx;
    _artist->setPos( 4 + 180*Column , categorieRow*50 + artistRow*165);
    addItem(_artist);
      
    if(Column <3) {
      Column++;
    }
    else {
      Column = 0;
      artistRow++;
    }
    
    //! ALBUM LOOP -> set Album cover path list
    for (int j = 0; j < artistItem->childCount(); j++) {
      QModelIndex albumIdx = _collectionModel->index ( j, 0, artistIdx);
      if(!_collectionModel->isAlbumFiltered(albumIdx) ) continue;
          
      ModelItem *albumItem = _collectionModel->getItem(albumIdx);
      coverPathList << albumItem->getData(3).toString();
    }

    artistItem->setData(2, coverPathList);
    coverPathList.clear();
  } // end artist loop
  
  

  //! ------ Favorite Albums ------
  if (Column > 0) artistRow++;
  first    = true;

  for ( int i = 0; i < _collectionModel->rowCount ( QModelIndex() ); i++ ) 
  {
    QModelIndex artistIdx = _collectionModel->index ( i, 0, QModelIndex() );
    
    //! album loop
    for (int j = 0; j < _collectionModel->rowCount (artistIdx); j++) 
    {
      QModelIndex albumIdx = _collectionModel->index ( j, 0, artistIdx);
      
      if(!_collectionModel->itemIsFavorite(albumIdx) ) continue;
      if(!_collectionModel->isAlbumFiltered(albumIdx) )	continue;
      
      if(first) {
        CategorieGraphicItem *_cat = new CategorieGraphicItem();
        _cat->_catName = "Favorite Albums";
        _cat->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
        addItem(_cat);
        
        _listCategorie.append(_cat);
                
        Column = 0;
        categorieRow++;
        first = false;
      }
      _infoSize++; // we count album favorite item 
      
      AlbumGraphicItem *_album = new AlbumGraphicItem();
      _album->_modelIdx = albumIdx;
      _album->setPos(4+180*Column, categorieRow*50 + artistRow*165 + albumRow*170);
      addItem(_album);
      
      if(Column <3) {
        Column++;
      } 
      else {
        Column = 0;
        albumRow++;
      }
    } // end album loop
  } // end artist loop
}



void CollectionScene::populatePlaylistScene() 
{
  qDebug() << " ---- CollectionScene --> populatePlaylistScene" ;

  int categorieRow = 0;
  int playlistRow  = 0;
  int Column       = 0;
  _infoSize        = 0;
  bool firstinternal = true;
  bool firstuser     = true;
  int idx = 0;
  
  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  //! playlist loop
  for ( int i = 0; i < _plsFileModel->rowCount ( QModelIndex() ); i++ ) 
  {
    QModelIndex playlistIdx = _plsFileModel->index ( i, 0, QModelIndex() );
    
    if(!_plsFileModel->isPlaylistFiltered(playlistIdx) ) continue;
    
    if(firstinternal && (_plsFileModel->getItem(playlistIdx))->getData(4).toInt() == 0) 
    {
      CategorieGraphicItem *_cat = new CategorieGraphicItem();
      _cat->_catName = QString("Playlist files from internal database");
      _cat->setPos( 0 ,10 + categorieRow*50 + playlistRow*100);
      addItem(_cat);
      _listCategorie.append(_cat);
      categorieRow++;

      firstinternal = false;
    }
    else if(firstuser && (_plsFileModel->getItem(playlistIdx))->getData(4).toInt() == 1) 
    {
      if(idx>0) playlistRow++;

      CategorieGraphicItem *_cat2 = new CategorieGraphicItem();
      _cat2->_catName = QString("Playlist files from user collection");
      _cat2->setPos( 0 ,10 + categorieRow*50 + playlistRow*100);
      addItem(_cat2);
      _listCategorie.append(_cat2);
      categorieRow++;

      firstuser = false;  
      Column = 0;
    }
    
    _infoSize++;
    idx++; // nouvelle ligne
      
    PlaylistGraphicItem *_playlist = new PlaylistGraphicItem();
    _playlist->_modelIdx = playlistIdx;
    _playlist->setPos(4+180*Column, categorieRow*50 + playlistRow*100);
    addItem(_playlist);
    
    if(Column <3) {
        Column++;
    } 
    else {
      Column = 0;
      playlistRow++;
      idx = 0;
    }    
 }
}

void CollectionScene::populatePlaylistDetailScene() 
{
  //qDebug() << " ----CollectionScene -> populatePlaylistDetailScene ----";
  
  int playlistRow   = 0;
  int trackRow      = 0;
  int trackPerAlbum = 0;
  int offset        = 0;
  _infoSize         = 0;
  
  //! track list for virtual playlist
  _listTrack.clear();

  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  //! playlist loop
  for ( int i = 0; i < _plsFileModel->rowCount ( QModelIndex() ); i++ ) 
  {
    QModelIndex playlistIdx = _plsFileModel->index ( i, 0, QModelIndex() );
    
    if(!_plsFileModel->isPlaylistFiltered(playlistIdx) ) continue;
    
    ModelItem *playlistItem = _plsFileModel->getItem(playlistIdx);
    
    CategorieGraphicItem *_playlistCat = new CategorieGraphicItem();
    _playlistCat->_catName = playlistItem->getData(2).toString();
    _playlistCat->setPos( 10 ,playlistRow*50 + trackRow*20 + offset );
    addItem(_playlistCat);
    _listCategorie.append(_playlistCat);
    
    playlistRow++;
    
    PlaylistGraphicItem *_playlist = new PlaylistGraphicItem();
    _playlist->_modelIdx = playlistIdx;
    _playlist->setPos(10, playlistRow*50 + trackRow*20 + offset );
    addItem(_playlist);
    trackPerAlbum = 0;

    //! track loop
    for (int k = 0; k < playlistItem->childCount(); k++) 
    {
        QModelIndex trackIdx = _plsFileModel->index ( k, 0, playlistIdx);
        if(!_plsFileModel->isUrlFiltered(trackIdx)) continue;
	
        TrackGraphicItem *_track = new TrackGraphicItem();
        _track->_modelIdx = trackIdx;
        _track->setPos(180, playlistRow*50 + trackRow*20 + offset -20);
        addItem(_track); 
        _listTrack.append(_track);

        trackRow++;
        trackPerAlbum++;
        _infoSize++;
    }
    if( trackPerAlbum < 6 ) offset = offset + (6 - trackPerAlbum)*25;
  }  
}


//! --------------------- contextMenuEvent ------------------------------------- 
void CollectionScene::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) 
{
  _mouseGrabbedItem = this->mouseGrabberItem();
   qDebug() << " ---- CollectionScene::contextMenuEvent _mouseGrabbedItem" << _mouseGrabbedItem;
  
  if(!_mouseGrabbedItem) return;
  
  //! -- Album Graphic Item 
  if(_mouseGrabbedItem->type()== GraphicsItem::AlbumType)
  {
    AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);
    if(_collectionModel->itemIsFavorite(item->_modelIdx))
      albumFavoriteAct->setText ("Remove Album from Favorites");
    else
      albumFavoriteAct->setText ("Add Album to Favorites");

    _menuAlbum->exec(event->screenPos());
  } 
  else if(_mouseGrabbedItem->type()== GraphicsItem::AlbumGenreType) 
  {
    _menuAlbumGenre->exec(event->screenPos());  
  }
  else if(_mouseGrabbedItem->type()== GraphicsItem::ArtistType) 
  {
     ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(_mouseGrabbedItem);

     if(_collectionModel->itemIsFavorite(item->_modelIdx))
      artistFavoriteAct->setText ("Remove Artist from Favorites");
    else
      artistFavoriteAct->setText ("Add Artist to Favorites");
      
    _menuArtist->exec(event->screenPos());
  }
  else if(_mouseGrabbedItem->type()== GraphicsItem::PlaylistType)
  {   
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);
    
    if(_plsFileModel->itemIsFavorite(item->_modelIdx))
      playlistFavoriteAct->setText ("Remove Playlist from Favorites");
    else
      playlistFavoriteAct->setText ("Add Playlist to Favorites");
  
    _menuPlaylist->exec(event->screenPos()); 
  }
  else {
    _mouseGrabbedItem = 0;
  }
}

//! TEST récupération du double click sur un item par une action globale
void CollectionScene::itemDoubleClick()
{
   qDebug() << " ---- CollectionScene::itemDoubleClick";
   QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  
   qDebug() << " ---- CollectionScene::itemDoubleClick" << (actions->value("itemDoubleClick"))->data();
   
   QGraphicsItem *gItem = qvariant_cast<QGraphicsItem *>( (actions->value("itemDoubleClick"))->data() );
   _mouseGrabbedItem = gItem;
   if(!_mouseGrabbedItem) return;
   
   switch(gItem->type()) {
     case GraphicsItem::AlbumType: playAlbum();break;
     case GraphicsItem::AlbumGenreType: playAlbumGenre();break;
     case GraphicsItem::ArtistType: playArtist();break;
     case GraphicsItem::TrackType: playTrack();break;
     case GraphicsItem::PlaylistType: playPlaylist();break;
     default:break;    
   }
}

//! --------------------- Album --> loadCoverArtFromFile ----------------------- 
void CollectionScene::loadCoverArtFromFile() 
{
  if(!_mouseGrabbedItem) return;
  AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

  //! open file dialog  
  QString filename = QFileDialog::getOpenFileName(new QWidget(), tr("Select cover file..."),tr("Files (*.png *.jpg)"));
  if( filename.isEmpty() ) return;

  ModelItem *mi   = _collectionModel->getItem(item->_modelIdx);
  QString coverPath = _CONFIGDIR + "/albums/" + mi->getData(3).toString();

  //! get new cover file
  QImage image = QImage(filename);
  image = image.scaled(QSize(110, 110), Qt::KeepAspectRatio, Qt::SmoothTransformation);
  //! remove existing cover file
  QFile coverfile(coverPath);
  if(coverfile.exists()) coverfile.remove();
  //! save new cover file
  image.save(coverPath, "png", -1);
  //! scene update
  update(_mouseGrabbedItem->boundingRect());
}

void CollectionScene::loadLastFmCover() 
{
  if(!_mouseGrabbedItem) return;
  AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

  //! Get album Name and coverPath
  ModelItem *mi    =  _collectionModel->getItem(item->_modelIdx);
  QString _album     =  mi->getData(1).toString();
  QString _coverPath =  mi->getData(3).toString();

  //! Get Parent = Artist Name
  ModelItem *parentItem = mi->parent();
  QString _artist =  parentItem->getData(1).toString();
  
  //! start search cover process
  CoverSearch *coverSearch = new CoverSearch(_artist, _album, _coverPath);
  QObject::connect(coverSearch, SIGNAL(coverUpdated()), this, SLOT(updateMouseGrabbedItem())); 
}

void CollectionScene::removeCoverArt()
{
  qDebug() << " ----RemoveCoverArtForAlbum ---- ";  
  if(!_mouseGrabbedItem) return;
  AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

  ModelItem *mi    =  _collectionModel->getItem(item->_modelIdx);
  QString _coverPath =  mi->getData(3).toString();

  QFile coverfile(_CONFIGDIR + "/albums/" + _coverPath);
  if(coverfile.exists()) {
    coverfile.remove();
    //! item update
    item->update();
  }
} 

void CollectionScene::modifyAlbumInfo() 
{
  if(!_mouseGrabbedItem) return;
  AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

  //! get Model data
  ModelItem *mi    =  _collectionModel->getItem(item->_modelIdx);
  int _albumBdId     = mi->getData(0).toInt();
   
  AlbumChangeDialog *modifDialog = new AlbumChangeDialog(_albumBdId);
  connect(modifDialog,SIGNAL(signal_AlbumInfoChanged()),this,SIGNAL(signal_RepopulateCollection()));
  modifDialog->exec();    
}

void CollectionScene::modifyGenreInfo()
{
  if(!_mouseGrabbedItem) return;
  AlbumGenreGraphicItem *item = static_cast<AlbumGenreGraphicItem*>(_mouseGrabbedItem);

  //! get Model data 
  ModelItem *mi    =  _collectionModel->getItem(item->_modelIdx);
  int _albumBdId     = mi->getData(0).toInt();
   
  GenreChangeDialog *modifDialog = new GenreChangeDialog(_albumBdId, item->_genre);
  connect(modifDialog,SIGNAL(signal_GenreInfoChanged()),this,SIGNAL(signal_RepopulateCollection()));
  modifDialog->exec(); 
} 

void CollectionScene::modifyArtistInfo()
{
  if(!_mouseGrabbedItem) return;
  ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(_mouseGrabbedItem);

  //! get Model data 
  ModelItem *mi = _collectionModel->getItem(item->_modelIdx);
  int _artistBdId = mi->getData(0).toInt();
   
  ArtistChangeDialog *modifDialog = new ArtistChangeDialog(_artistBdId);
  connect(modifDialog,SIGNAL(signal_ArtistInfoChanged()),this,SIGNAL(signal_RepopulateCollection()));
  modifDialog->exec();
}


void CollectionScene::updateAlbumFavorite()
{
  if(!_mouseGrabbedItem) return;
  AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

  int _albumBdId;
  int newFavorite;
  
   //! Change model item favorites status
   if( _collectionModel->itemIsFavorite(item->_modelIdx) )
     _collectionModel->setItemIsFavorite(item->_modelIdx, false);
   else
     _collectionModel->setItemIsFavorite(item->_modelIdx, true);
 
   newFavorite = _collectionModel->itemIsFavorite(item->_modelIdx);
   
   //! Apply changes into Db
   ModelItem *mi = _collectionModel->getItem(item->_modelIdx);
   _albumBdId = mi->getData(0).toInt();

   DatabaseManager dbManager("AlbumItem");
   if (!dbManager.connectToDB()) return;
   
   QSqlQuery("BEGIN TRANSACTION;",*dbManager.sqlDb());
   
   QSqlQuery query("UPDATE `albums` "       \
                   "SET `favorite`="+QString::number(newFavorite)+" " \
                   "WHERE `id`="+QString::number(_albumBdId)+";",*dbManager.sqlDb());
   QSqlQuery("COMMIT TRANSACTION;",*dbManager.sqlDb());
   
   //! item update
   item->update();
}


void CollectionScene::updateArtistFavorite()
{
  if(!_mouseGrabbedItem) return;
  ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(_mouseGrabbedItem);

  int _artistBdId;
  int newFavorite;
  
   //! Change model item favorites status
   if(_collectionModel->itemIsFavorite(item->_modelIdx))
     _collectionModel->setItemIsFavorite(item->_modelIdx, false);
   else
     _collectionModel->setItemIsFavorite(item->_modelIdx, true);
 
   newFavorite = _collectionModel->itemIsFavorite(item->_modelIdx);
   
   //! Apply changes into Db
   ModelItem *mi = _collectionModel->getItem(item->_modelIdx);
   _artistBdId     = mi->getData(0).toInt();

   DatabaseManager dbManager("ArtistItem");
   if (!dbManager.connectToDB()) return;
   
   QSqlQuery("BEGIN TRANSACTION;",*dbManager.sqlDb());
   
   QSqlQuery query("UPDATE `interprets` "       \
                   "SET `favorite`="+QString::number(newFavorite)+" " \
                   "WHERE `id`="+QString::number(_artistBdId)+";",*dbManager.sqlDb());
   QSqlQuery("COMMIT TRANSACTION;",*dbManager.sqlDb());
   
   //! item update
   item->update();
}


void CollectionScene::updateMouseGrabbedItem()
{
  if(!_mouseGrabbedItem) return;
  QList<QGraphicsItem *> list = this->items();
  if (list.contains(_mouseGrabbedItem))
    _mouseGrabbedItem->update();
}


void CollectionScene::removePlaylistFromDisk()
{
  if(!_mouseGrabbedItem) return;
  PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);

  //! get Model data 
  ModelItem *mi = _plsFileModel->getItem(item->_modelIdx);
  QString filename = mi->getData(1).toString();
  qDebug() << "--- PlaylistGraphicItem --> remove item " << filename;

  QFile playlistFile(filename);
  qDebug() << " PlaylistGraphicItem --> file exist" << playlistFile.exists();

  if(playlistFile.exists()) {
     playlistFile.remove();
     
     DatabaseManager dbManager("PlaylistItem");
     if (!dbManager.connectToDB()) return;
   
     QFileInfo fileInfo(filename);
     QString fname = fileInfo.filePath().toUtf8();
     QSqlField data("col",QVariant::String);
     data.setValue(fname);
     fname = dbManager.sqlDb()->driver()->formatValue(data,false);
     QSqlQuery query("DELETE FROM `playlists` WHERE `filename`="+fname+";",*dbManager.sqlDb());
     qDebug() << query.exec();
     //QSqlQuery query2("DELETE FROM `playlistitem` WHERE `playlists` NOT IN (SELECT `id` FROM `playlists`);", *dbManager.sqlDb());
     //qDebug() << query2.exec();
     emit signal_RepopulatePlsFile(); 
  }
}

void CollectionScene::modifyPlaylistInfo()
{
  if(!_mouseGrabbedItem) return;
  PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);

  //! get Model data 
  ModelItem *mi = _plsFileModel->getItem(item->_modelIdx);
  int _playlistBdId = mi->getData(0).toInt();
   
  PlaylistChangeDialog *modifDialog = new PlaylistChangeDialog(_playlistBdId);
  connect(modifDialog,SIGNAL(signal_PlaylistInfoChanged()),this,SIGNAL(signal_RepopulatePlsFile()));
  modifDialog->exec();
}


void CollectionScene::updatePlaylistFavorite()
{
  if(!_mouseGrabbedItem) return;
  PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);

  int _playlistBdId;
  int newFavorite;
  
   //! Change model item favorites status
   if(_plsFileModel->itemIsFavorite(item->_modelIdx))
     _plsFileModel->setItemIsFavorite(item->_modelIdx, false);
   else
     _plsFileModel->setItemIsFavorite(item->_modelIdx, true);
 
   newFavorite = _plsFileModel->itemIsFavorite(item->_modelIdx);
   
   //! Apply changes into Db
   ModelItem *mi = _plsFileModel->getItem(item->_modelIdx);
   _playlistBdId = mi->getData(0).toInt();

   DatabaseManager dbManager("PlaylistItem");
   if (!dbManager.connectToDB()) return;
   
   QSqlQuery("BEGIN TRANSACTION;",*dbManager.sqlDb());
   
   QSqlQuery query("UPDATE `playlists` "       \
                   "SET `favorite`="+QString::number(newFavorite)+" " \
                   "WHERE `id`="+QString::number(_playlistBdId)+";",*dbManager.sqlDb());
   QSqlQuery("COMMIT TRANSACTION;",*dbManager.sqlDb());

   //! item update
   item->update();
}
