/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionView/collectionwidget.h"
#include "Widget/statusbar.h"
#include "constants.h"

#include <QSettings>

namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}


CollectionWidget::CollectionWidget(QWidget *parent)
{
  Q_UNUSED(parent)
   
  //! get StatusBar global instance
  _statusBar         = StatusBar::instance();
  _centralToolBar    = CentralToolBar::instance();

  _graphicView = new QGraphicsView(this) ;
  
  //! GraphicWidget Setup  
  setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
  
  //! GraphicView Setup  
  _graphicView->setCacheMode(QGraphicsView::CacheNone);
  _graphicView->setDragMode(QGraphicsView::NoDrag); 
  
  _graphicView->setRenderHint(QPainter::Antialiasing);
  _graphicView->setOptimizationFlags(QGraphicsView::DontAdjustForAntialiasing
                                   | QGraphicsView::DontClipPainter
                                   | QGraphicsView::DontSavePainterState);
  _graphicView->setResizeAnchor(QGraphicsView::NoAnchor);
  _graphicView->setViewportUpdateMode(QGraphicsView::MinimalViewportUpdate); 
  _graphicView->setAlignment(Qt::AlignLeft | Qt::AlignTop); 
      
  //! Gui Creation
  QVBoxLayout *verticalLayout_0 = new QVBoxLayout(this);
  verticalLayout_0->setSpacing(0);
  verticalLayout_0->setContentsMargins(0, 0, 0, 0);
  verticalLayout_0->addWidget(_graphicView);
    
  //! default _scrollPosition
  _scrollPosition[Collection::ViewAlbum]         = 1;
  _scrollPosition[Collection::ViewArtist]        = 1;
  _scrollPosition[Collection::ViewTrack]         = 1;
  _scrollPosition[Collection::ViewGenre]         = 1;
  _scrollPosition[Collection::ViewFavorite]      = 1;
  _scrollPosition[Collection::ViewPlaylist]      = 1;
  _scrollPosition[Collection::ViewPlaylistDetail]= 1;
  
  //! connect toolbar signals
  connect(_centralToolBar, SIGNAL(collectionSearch(QString)),this, SLOT(updateCollectionSearch(QString)));
  connect(_centralToolBar, SIGNAL(playlistSearch(QString)),this, SLOT(updatePlaylistSearch(QString)));
  
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  connect(actions->value("albumview"), SIGNAL(triggered()),this,  SLOT(switchAlbumView()));
  connect(actions->value("artistview"), SIGNAL(triggered()),this,  SLOT(switchArtistView()));
  connect(actions->value("trackview"), SIGNAL(triggered()),this,  SLOT(switchTrackView()));
  connect(actions->value("genreview"), SIGNAL(triggered()),this,  SLOT(switchGenreView()));
  connect(actions->value("favoriteview"), SIGNAL(triggered()),this,  SLOT(switchFavoriteView()));
  connect(actions->value("playlistview"), SIGNAL(triggered()),this,  SLOT(switchPlaylistView()));
  connect(actions->value("playlistdetailview"), SIGNAL(triggered()),this,  SLOT(switchPlaylistDetailView()));
  connect(actions->value("goUp"), SIGNAL(triggered()),this,  SLOT(navigateGoUp()));
  connect(actions->value("goDown"), SIGNAL(triggered()),this,  SLOT(navigateGoDown()));
}


void CollectionWidget::setScene(CollectionScene *scene)
{
  _collectionScene  = scene;
  _graphicView->setScene(_collectionScene);
}


//!----------------------------------------------------------------------------
//! save and restore view mode
//!----------------------------------------------------------------------------

void CollectionWidget::saveLastCollectionView()
{
    QSettings *_settings = new QSettings(_CONFIGDIR + QDir::separator() + "main.conf",QSettings::IniFormat,this);
    _settings->setValue("Collection/view",_collectionMode );
}

void CollectionWidget::restoreLastCollectionView()
{
  QSettings *_settings = new QSettings(_CONFIGDIR + QDir::separator() + "main.conf",QSettings::IniFormat,this);
  
  if(_settings->contains("Collection/view"))
    _collectionMode = Collection::CollectionMode(_settings->value("Collection/view").toInt());
  else 
    _collectionMode = Collection::ViewAlbum;
  
  //! initialization des previous views
  _prevCollectionMode = int(_collectionMode) > 5 ?  Collection::ViewAlbum : _collectionMode;
  _prevPlaylistMode   = int(_collectionMode) < 6 ?  Collection::ViewPlaylist : _collectionMode;
}



//!----------------------------------------------------------------------------
//! Scene population
//!----------------------------------------------------------------------------
//! only use after database rebuild
void CollectionWidget::updateCollectionScene() 
{
    qDebug() << "CollectionWidget -  updateCollectionScene " << _collectionMode;
    switchView(_collectionMode);
}

void CollectionWidget::updatePlsFileScene()
{
   qDebug() << "CollectionWidget -  updatePlsFileScene " << _collectionMode;
   switchView(_collectionMode);
}

//!----------------------------------------------------------------------------
//! View Switch
//!----------------------------------------------------------------------------
void CollectionWidget::switchView(Collection::CollectionMode mode) 
{
  qDebug() << "CollectionWidget -  switchView " << mode;
  
  //! Save previous scroll bar position
  _scrollPosition[_collectionMode]  = _graphicView->verticalScrollBar()->sliderPosition();

  //! Change collection view mode
  _collectionMode = mode;

  //! save previous views
  _prevCollectionMode = int(_collectionMode) > 5 ?  _prevCollectionMode : _collectionMode;
  _prevPlaylistMode   = int(_collectionMode) < 6 ?  _prevPlaylistMode : _collectionMode;
  
  _collectionScene->_collectionMode = _collectionMode;
  _collectionScene->populateScene();

  updateCollectionSummary();
    
  //!restore scroll position
  _graphicView->verticalScrollBar()->setSliderPosition(_scrollPosition[mode]);
}


void CollectionWidget::switchAlbumView() {
  switchView(Collection::ViewAlbum);
}

void CollectionWidget::switchArtistView() {
  switchView(Collection::ViewArtist);
}

void CollectionWidget::switchTrackView() {
  switchView(Collection::ViewTrack);
}

void CollectionWidget::switchGenreView() {
  switchView(Collection::ViewGenre);
}

void CollectionWidget::switchFavoriteView() {
  switchView(Collection::ViewFavorite);
}

void CollectionWidget::switchPlaylistView() {
  switchView(Collection::ViewPlaylist);
}

void CollectionWidget::switchPlaylistDetailView() {
  switchView(Collection::ViewPlaylistDetail);
}


void CollectionWidget::restorePrevCollectionView() 
{
  switchView(_prevCollectionMode);
}

void CollectionWidget::restorePrevPlaylistView()
{
  switchView(_prevPlaylistMode);
}
  
//!----------------------------------------------------------------------------
//! Search management
//!----------------------------------------------------------------------------
void CollectionWidget::updateCollectionSearch(QString filter) 
{
  //qDebug() << "updateSearch  " << filter;
  _collectionScene->_collectionModel->_filterPattern = filter;
  switchView(_collectionMode);
}

void CollectionWidget::updatePlaylistSearch(QString filter) 
{
  //qDebug() << "updateSearch  " << filter;
  _collectionScene->_plsFileModel->_filterPattern = filter;
  switchView(_collectionMode);
}
//!----------------------------------------------------------------------------
//! Collection summary
//!----------------------------------------------------------------------------

void CollectionWidget::updateCollectionSummary() 
{
  int size = _collectionScene->getSceneInfoSize();
  QString     collectionInfo;
  
  
  switch(_collectionMode)
  {
    case (Collection::ViewAlbum) : 
      collectionInfo = QString(tr("Collection : <b>%1</b> albums")).arg(QString::number(size));
      break;
    
    case (Collection::ViewArtist) :
      collectionInfo = QString(tr("Collection : <b>%1</b> artist")).arg(QString::number(size));
      break;
    
    case (Collection::ViewTrack) :
      collectionInfo = QString(tr("Collection : <b>%1</b> tracks")).arg(QString::number(size));
      break;

    case (Collection::ViewGenre) :
      collectionInfo = QString(tr("Collection : <b>%1</b> styles")).arg(QString::number(size));
      break;
    
    case (Collection::ViewFavorite) :
      collectionInfo = QString(tr("Collection : <b>%1</b> favorite item")).arg(QString::number(size));
      break;

    case (Collection::ViewPlaylist) :
      collectionInfo = QString(tr("Playlist : <b>%1</b> playlist files")).arg(QString::number(size));
      break;

    case (Collection::ViewPlaylistDetail) :
      collectionInfo = QString(tr("Playlist : <b>%1</b> tracks")).arg(QString::number(size));
      break;
      
    default: collectionInfo = "";break;
  }
  
  _statusBar->updateCollectionInfo(collectionInfo);

}

//!----------------------------------------------------------------------------
//! Collection Navigation
//!----------------------------------------------------------------------------
void CollectionWidget::navigateGoUp() 
{
  int top     = _graphicView->verticalScrollBar()->sliderPosition();

  for(int i=_collectionScene->_listCategorie.size()-1; i>=0;i--){

    QPoint itemPoint = (_collectionScene->_listCategorie.at(i)->scenePos()).toPoint();

    if(top > itemPoint.y()) {
      _graphicView->verticalScrollBar()->setSliderPosition(itemPoint.y());
      break;
    }
  }
}

void CollectionWidget::navigateGoDown() 
{
  int top     = _graphicView->verticalScrollBar()->sliderPosition();
  
  foreach (QGraphicsItem *item, _collectionScene->_listCategorie)
  {
    QPoint itemPoint = (item->scenePos()).toPoint();

    if(top < itemPoint.y()) {
      _graphicView->verticalScrollBar()->setSliderPosition(itemPoint.y());
      break;
    }
  }
}
