/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Database/databasebuilder.h"
#include "Database/databasemanager.h"
#include "MediaItem/mediaitem.h"
#include "constants.h"

#include <taglib/fileref.h>
#include <taglib/tag.h>
#include <taglib/tstring.h>
#include <taglib/mpegfile.h>

#include <QtCore>
#include <QImage>
#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlQuery>
#include <QtSql/QSqlDriver>
#include <QtSql/QSqlField>
#include <QtSql/QSqlError>
#include <QtSql/QSqlResult>

const QStringList audioFilters = QStringList() << "*.mp3" << "*.ogg" << "*.wav" << "*.flac";
		
//! --------- DataBaseBuilder -------------------------------------------------- 
DataBaseBuilder::DataBaseBuilder()
{
   _isRunning = false;
   
   //! Settings reading (option -> recup cover art from files directory)
   QSettings _settings(QString(_CONFIGDIR).append("/main.conf"),QSettings::IniFormat,this);
   _settings.beginGroup("Collections");
   if(_settings.contains("SearchCoverFromDirectory")) {
     _coverFromDir = _settings.value("SearchCoverFromDirectory",true).toBool();
   }
   else {
     _coverFromDir = false;
   }
   _settings.endGroup();
}

//! --------- DataBaseBuilder::rebuildFolder -----------------------------------
// User entry point -> add folder to parse
void DataBaseBuilder::rebuildFolder(QStringList folder)
{
    _folders.append(folder);
}

//! --------- DataBaseBuilder::run --------------------------------------------- 
void DataBaseBuilder::run()
{
    _isRunning = true;
    if (_folders.isEmpty()) return;

    DatabaseManager dbManager("collectionsUpdateConnection");
    if (!dbManager.connectToDB()) return;
      
    qDebug() << " --- DataBaseBuilder -> starting Database update";

    { // Populate dbFiles map by _ALL_ tracks from db
        QSqlQuery query("SELECT filename, mtime FROM tracks;",*dbManager.sqlDb());
        while (query.next()) {
          dbFiles.insert(query.value(0).toString(),query.value(1).toUInt());
        }
    }
    QFileInfo fileInfo;

    //On SQLite --> it's MUCH faster to have everything in one transaction 
    //with only one disk write than to commit every each insert individually
    QSqlQuery("BEGIN TRANSACTION;",*dbManager.sqlDb());

    do 
    {
        QDir dirlist(_folders.takeFirst());
        dirlist.setNameFilters(audioFilters);

        QDirIterator dirIterator(dirlist,QDirIterator::Subdirectories);
        while (dirIterator.hasNext()) {
            dirIterator.next();
            fileInfo = dirIterator.fileInfo();
                    
            if (fileInfo.isFile()) {
                //! If the file is not in database then insert
                if (!dbFiles.contains(fileInfo.filePath().toUtf8())) {
                    insertTrack(fileInfo.absoluteFilePath(), dbManager.sqlDb());
                    dbFiles.remove(fileInfo.filePath().toUtf8());
                }

                //! If the file is in database but has another mtime then update it
                if ((dbFiles.contains(fileInfo.filePath().toUtf8())) &&
                    (dbFiles[fileInfo.filePath().toUtf8()]!=fileInfo.lastModified().toTime_t())) {
                        updateTrack(fileInfo.absoluteFilePath(), dbManager.sqlDb());
                        dbFiles.remove(fileInfo.filePath().toUtf8());
                }

                //! If the file is in database and was not changed then do nothing and just
                // remove it from file to proceed
                if ((dbFiles.contains(fileInfo.filePath().toUtf8())) &&
                    (dbFiles[fileInfo.filePath().toUtf8()]==fileInfo.lastModified().toTime_t())) {
                     dbFiles.remove(fileInfo.filePath().toUtf8());
                }
            } //IF fileInfo.isFile()
        } // WHILE dirIterator.hasNext
    } while (_folders.size()>0);

    //! Get files that are in DB but not on harddrive
    QHashIterator<QString, uint> i(dbFiles);
    while (i.hasNext()) {
        i.next();
        removeTrack(i.key(), dbManager.sqlDb());
    }
       
    dbFiles.clear();

    // Check for interprets/albums/genres... that are not used anymore
    cleanUpDatabase(dbManager.sqlDb());
		
    // Now write all data to the disk
    QSqlQuery("COMMIT TRANSACTION;",*dbManager.sqlDb());

    _isRunning = false;
    emit buildingFinished();
}



//! --------- DataBaseBuilder::insertTrack -------------------------------------
//   -     MEDIA::FromLocalFile(fname) to get track metadata
//   -     MEDIA::getCoverName(_mediaItem) to get hash of covername
void DataBaseBuilder::insertTrack(QString filename, QSqlDatabase *sqlDB)
{
    QFileInfo fileInfo(filename);

    QString fname = fileInfo.filePath().toUtf8();

    uint mtime = QFileInfo(filename).lastModified().toTime_t();

    qDebug() << "--- DataBaseBuilder -> insert track :" << filename;

    QSqlField data("col",QVariant::String);

    //! check album storage localtion
    QString storageLocation = _CONFIGDIR + "/albums/";
    QDir dir;
    dir.mkpath(storageLocation);

    //! Read tag from URL file (with taglib) 
    MediaItem *_mediaItem = MEDIA::FromLocalFile(fname);
    QString  _coverName   = MEDIA::getCoverName(_mediaItem);

    data.setValue(_coverName);
    QString coverName_Db = sqlDB->driver()->formatValue(data,false);

    //! GENRE part in database
    {
        data.setValue(_mediaItem->genre);
        _mediaItem->genre = sqlDB->driver()->formatValue(data,false);
        QSqlQuery query("SELECT COUNT(*) FROM `genres` WHERE `genre`="+_mediaItem->genre+";", *sqlDB);
        query.next();
        if (query.value(0).toInt() == 0) {
            QSqlQuery query("INSERT INTO `genres`(`genre`) VALUES("+_mediaItem->genre+");", *sqlDB);
        }
    }
    
    //! YEAR part in database
    QString s_year;
    {
        data.setValue(_mediaItem->year);
        s_year = sqlDB->driver()->formatValue(data,false);
        QSqlQuery query("SELECT COUNT(*) FROM `years` WHERE `year`="+s_year+";", *sqlDB);
        query.next();
        if (query.value(0).toInt() == 0) {
            QSqlQuery query("INSERT INTO `years`(`year`) VALUES("+s_year+");", *sqlDB);
        }
    }
    
    
    //! ARTIST part in database
    {
        data.setValue(_mediaItem->artist);
        _mediaItem->artist = sqlDB->driver()->formatValue(data,false);
        QSqlQuery query("SELECT `id` FROM `interprets` WHERE `interpret`="+_mediaItem->artist+";", *sqlDB);
        query.next();
        if (!query.isValid()) {
            QSqlQuery query("INSERT INTO `interprets`(`interpret`, `favorite`) " \
                            "VALUES("+_mediaItem->artist+",0);", *sqlDB);
        } 
    }
    
    //! ALBUM part in database
    QString interpretID;
    {
	QSqlQuery artistquery("SELECT `id` FROM `interprets` WHERE `interpret`="+_mediaItem->artist+";", *sqlDB);
        artistquery.next();
        data.setValue(artistquery.value(0).toString());
        interpretID = sqlDB->driver()->formatValue(data,false);
        
	data.setValue(_mediaItem->album);
        _mediaItem->album = sqlDB->driver()->formatValue(data,false);
	
        QSqlQuery query("SELECT `id`,`album`,`interpretID` FROM `albums` WHERE `album`="+_mediaItem->album+" AND `interpretID`="+interpretID+";", *sqlDB);
        query.next();
        if (!query.isValid()) {
          QSqlQuery query("INSERT INTO `albums`(`album`,`interpretID`, `cover`, `year`, `favorite`) " \
                          "VALUES("+_mediaItem->album+","+interpretID+", "+coverName_Db+", "+s_year+",0);", *sqlDB);
        } 
        //! mise à jour du cover
        storeCoverArt(storageLocation + _coverName, _mediaItem->url);
	if(_coverFromDir)
	  recupCoverArtFromDir(storageLocation + _coverName, _mediaItem->url);
	  
    }

    //! TRACK part in database
    {
        QSqlQuery query(*sqlDB);
        query.prepare("INSERT INTO `tracks`(`filename`,`trackname`,`track`,`length`,`interpret`,`album`,`year`,`genre`,`mtime`)" \
                      "VALUES(?," \
                      "       ?," \
                      "       ?," \
                      "       ?," \
                      "       (SELECT `id` FROM `interprets` WHERE `interpret`="+_mediaItem->artist+")," \
                      "       (SELECT `id` FROM `albums` WHERE `album`="+_mediaItem->album+" AND `interpretID`="+interpretID+")," \
                      "       (SELECT `id` FROM `years` WHERE `year`="+s_year+")," \
                      "       (SELECT `id` FROM `genres` WHERE `genre`="+_mediaItem->genre+")," \
                      "       ?);");
        query.addBindValue(fname);
        query.addBindValue(_mediaItem->title);
        query.addBindValue(_mediaItem->num);
        query.addBindValue(_mediaItem->duration);
        query.addBindValue(mtime);
        query.exec();
    }

}

//! --------- DataBaseBuilder::storeCoverArt -----------------------------------
void DataBaseBuilder::storeCoverArt(QString coverFilePath, QString trackFilename)
{
  qDebug() << " ---- DataBaseBuilder -> storeCoverArt " << coverFilePath;

  //! check if coverArt already exist
  QFile file(coverFilePath);
  if(file.exists()) return;
  
  //! get cover from file
  TagLib::MPEG::File mpegFile(QFileInfo(trackFilename).filePath().toLocal8Bit());
  TagLib::ID3v2::Tag *id3tag = mpegFile.ID3v2Tag(false);
  if (id3tag) {
    TagLib::ID3v2::AttachedPictureFrame *selectedFrame = MEDIA::attachedPictureFrame(id3tag);
    
    if (selectedFrame)
    {
      QByteArray pictureData = QByteArray(selectedFrame->picture().data(), selectedFrame->picture().size());
      if(!pictureData.isNull()) 
      {
	if (!file.open(QIODevice::WriteOnly)) 
	{
	  qDebug() << "Error opening file for writing" << file.fileName();
	} 
	else
	{
	  qDebug() << " ---- DataBaseBuilder -> new cover file stored " << coverFilePath;
	  //! WARNING la taille est fixée à 100 par 100 (taille utile à la vue album)
	  QImage image = QImage::fromData(pictureData);
	  image = image.scaled(QSize(110, 110), Qt::KeepAspectRatio, Qt::SmoothTransformation);
	  image.save(coverFilePath, "png", -1);
	}
      }
    }
  }
}

//! --------- DataBaseBuilder::recupCoverArtFromDir ----------------------------
void DataBaseBuilder::recupCoverArtFromDir(QString coverFilePath, QString trackFilename)
{
  qDebug() << " ---- DataBaseBuilder -> recupCoverArtFromDir " << coverFilePath;

  //! check if coverArt already exist
  QFile file(coverFilePath);
  if(file.exists()) return ;
  
  //! search album art into file source directory
  const QStringList imageFilters = QStringList() << "*.jpg" << "*.png";
  QDir sourceDir(QFileInfo(trackFilename).absolutePath());
  
  sourceDir.setNameFilters(imageFilters);
  
  QStringList entryList = sourceDir.entryList(imageFilters, QDir::Files, QDir::Size);
 
  while(!entryList.isEmpty()) {
    //! I take the first one (the biggest one)
    //!WARNING simplification WARNING
    QString file = QFileInfo(trackFilename).absolutePath() + "/" + entryList.takeFirst();
    QImage image = QImage(file);
    //! check if not null image (occur when file is KO)
    if(!image.isNull()) {
      image = image.scaled(QSize(110, 110), Qt::KeepAspectRatio, Qt::SmoothTransformation);
      //! check if save is OK
      if(image.save(coverFilePath, "png", -1))
	break;
    }
  }
}


//! --------- DataBaseBuilder::updateTrack -------------------------------------
void DataBaseBuilder::updateTrack(QString filename, QSqlDatabase *sqlDb)
{
    removeTrack(filename,sqlDb);
    insertTrack(filename,sqlDb);
}


//! --------- DataBaseBuilder::removeTrack -------------------------------------
void DataBaseBuilder::removeTrack(QString filename, QSqlDatabase *sqlDb)
{
    qDebug() << " --- DataBaseBuilder -> Deleting track :" << filename;
    QFileInfo fileInfo(filename);
    QString fname = fileInfo.filePath().toUtf8();

    QSqlField data("col",QVariant::String);
    data.setValue(fname);
    fname = sqlDb->driver()->formatValue(data,false);
    QSqlQuery query("DELETE FROM `tracks` WHERE `filename`="+fname+";", *sqlDb);
}

//! --------- DataBaseBuilder::cleanUpDatabase ---------------------------------
void DataBaseBuilder::cleanUpDatabase(QSqlDatabase *sqlDb)
{
  {
    QSqlQuery query("DELETE FROM `albums` WHERE `id` NOT IN (SELECT `album` FROM `tracks` GROUP BY `album`);", *sqlDb);
  }
  {
    QSqlQuery query("DELETE FROM `genres` WHERE `id` NOT IN (SELECT `genre` FROM `tracks` GROUP BY `genre`);", *sqlDb);
  }
  {      
    QSqlQuery query("DELETE FROM `interprets` WHERE `id` NOT IN (SELECT `interpret` FROM `tracks` GROUP BY `interpret`);", *sqlDb);
  }
  {
    QSqlQuery query("DELETE FROM `years` WHERE `id` NOT IN (SELECT `year` FROM `tracks` GROUP BY `year`);", *sqlDb);
  }
}
