/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "databasemanager.h"
#include "constants.h"

#include <QDebug>
#include <QDir>

#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlError>
#include <QtSql/QSqlQuery>

//! --------- DatabaseManager -------------------------------------------------- 
DatabaseManager::DatabaseManager(QString connectionName)
{
    _connectionName = connectionName;
    _sqlDb          = NULL;
}

//! --------- ~DatabaseManager -------------------------------------------------
DatabaseManager::~DatabaseManager()
{
    if (_sqlDb != NULL)
        delete _sqlDb;
    QSqlDatabase::removeDatabase(_connectionName);
}


//! --------- connectToDB ------------------------------------------------------
bool DatabaseManager::connectToDB()
{
    if (QSqlDatabase::contains(_connectionName)) {
        //qDebug() << "Connection with name " << _connectionName << " exists.";
        return true;
    }

    // Sql Database creation
    _sqlDb = new QSqlDatabase(QSqlDatabase::addDatabase("QSQLITE",_connectionName));
    _sqlDb->setDatabaseName(QString(_CONFIGDIR).append("/collection.db"));

    if (!_sqlDb->open()) {
        qDebug() << "Failed to establish '" << _sqlDb->connectionName() <<"' connection to database!";
        qDebug() << "Reason: " << _sqlDb->lastError().text();
        _sqlDb = NULL;
        return false;
    }
    
    //! check if all tables are presents
    QStringList tables = _sqlDb->tables(QSql::AllTables);
    if (!(tables.contains("albums") &&
          tables.contains("genres") &&
          tables.contains("interprets") &&
          tables.contains("tracks") &&
          tables.contains("years") &&
          tables.contains("db_rev") &&
          tables.contains("playlists") &&
          tables.contains("playlistitem") &&
          tables.contains("view_tracks"))) {
        initDb();
    }

    //! Now check if DB revision match
    {
      QSqlQuery query("SELECT `revision` FROM `db_rev` LIMIT 1;",*_sqlDb);
      query.next();
      QString rev = _DBREVISION;
      QString currentRev =  query.value(0).toString();
      if (currentRev != rev) {
         qDebug() << "Database revisions don't match: Found revision" << query.value(0).toString() << ", expected revision " << rev;
         qDebug() << "Collections will be rebuilt";
         
         initDb();
      }
    }
    return true;
}


//! --------- DatabaseManager::initDb ------------------------------------------ 
void DatabaseManager::initDb()
{
    qDebug() << "DatabaseManager --> Initializing database structure";

    QSqlQuery query(*_sqlDb);

    query.exec("DROP TABLE IF EXISTS `albums`;");
    query.exec("DROP TABLE IF EXISTS `genres`;");
    query.exec("DROP TABLE IF EXISTS `interprets`;");
    query.exec("DROP TABLE IF EXISTS `tracks`;");
    query.exec("DROP TABLE IF EXISTS `years`;");
    query.exec("DROP TABLE IF EXISTS `db_rev`;");
    query.exec("DROP TABLE IF EXISTS `playlists`;");
    query.exec("DROP TABLE IF EXISTS `playlistitem`;");
    query.exec("DROP VIEW IF EXISTS `view_tracks`;");

    query.exec("CREATE TABLE `genres` (" \
                 "    `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL," \
                 "    `genre` TEXT NOT NULL);");

    query.exec("CREATE TABLE `interprets` (" \
                 "    `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL," \
                 "    `interpret` TEXT NOT NULL,"   \
                 "    `favorite` INTEGER);");

    query.exec("CREATE TABLE `albums` (" \
                 "    `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL," \
                 "    `album` TEXT NOT NULL," \
                 "    `interpretID` INTEGER," \
                 "    `cover` TEXT,"          \
                 "    `year` INTEGER NOT NULL,"   \
                 "    `favorite` INTEGER);");

    query.exec("CREATE TABLE `tracks` (" \
                 "    `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL," \
                 "    `filename` TEXT NOT NULL," \
                 "    `trackname` TEXT NOT NULL," \
                 "    `track` INTEGER," \
                 "    `length` INTEGER NULL," \
                 "    `interpret` INTEGER," \
                 "    `album` INTEGER," \
                 "    `genre` INTEGER," \
                 "    `year` INTEGER, "\
                 "    `mtime` INTEGER);");

    query.exec("CREATE TABLE `years` (" \
                 "    `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL," \
                 "    `year` INTEGER NOT NULL);");

    QString rev = _DBREVISION;
    QString sql = "CREATE TABLE `db_rev` (" \
                    "    `revision` INTEGER NOT NULL DEFAULT("+rev+"));";

    qDebug() << query.exec(sql);
    qDebug() << query.exec("INSERT INTO `db_rev` VALUES('"+rev+"');");

    query.exec("CREATE VIEW `view_tracks` AS" \
                 "    SELECT `tracks`.`id`," \
                 "           `tracks`.`filename`," \
                 "           `tracks`.`trackname`," \
                 "           `tracks`.`track`," \
                 "           `tracks`.`length`," \
                 "           `tracks`.`album` AS `albumID`," \
                 "           `tracks`.`interpret` AS `interpretID`," \
                 "           `tracks`.`genre` AS `genreID`," \
                 "           `interprets`.`interpret`," \
                 "           `genres`.`genre`," \
                 "           `albums`.`album`," \
                 "           `years`.`year`" \
                 "    FROM `tracks`" \
                 "    LEFT JOIN `interprets` ON `tracks`.`interpret` = `interprets`.`id`" \
                 "    LEFT JOIN `genres` ON `tracks`.`genre` = `genres`.`id`" \
                 "    LEFT JOIN `albums` ON `tracks`.`album` = `albums`.`id`" \
                 "    LEFT JOIN `years` ON `tracks`.`year` = `years`.`id`;");
    
    //! playlist part
    query.exec("CREATE TABLE `playlists` (" \
                 "    `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL," \
                 "    `filename` TEXT NOT NULL," \
                 "    `name` TEXT NOT NULL," \
                 "    `type` INTEGER," \
                 "    `favorite` INTEGER," \
                 "    `mtime` INTEGER);");
    
    query.exec("CREATE TABLE `playlistitem` (" \
                 "    `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL," \
                 "    `urlname` TEXT NOT NULL," \
                 "    `name` TEXT NOT NULL,"
                 "    `playlist` INTEGER);");
    
}