/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Database/databaseplsbuilder.h"
#include "Database/databasemanager.h"
#include "MediaItem/mediaplaylist.h"

#include "constants.h"

#include <QtCore>
#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlQuery>
#include <QtSql/QSqlDriver>
#include <QtSql/QSqlField>
#include <QtSql/QSqlError>
#include <QtSql/QSqlResult>

const QStringList playlistFilters = QStringList() 
      << "*.m3u" << "*.m3u8"  << "*.pls" << "*.xspf";
		
//! --------- DataBasePlsBuilder ----------------------------------------------- 
DataBasePlsBuilder::DataBasePlsBuilder()
{
  _isRunning = false;
}


//! --------- DataBasePlsBuilder::rebuildFolder --------------------------------
// User entry point -> add folder to parse
void DataBasePlsBuilder::rebuildFolder(QStringList folder)
{
    //! check internal playlist storage localtion
    QString storageLocation = _CONFIGDIR + "/playlists/";
    QDir dir;
    dir.mkpath(storageLocation);
    
    _folders.append(folder);
    _folders.append(storageLocation);
}

void DataBasePlsBuilder::rebuildOnlyInternalFolder()
{
    //! check internal playlist storage localtion
    QString storageLocation = _CONFIGDIR + "/playlists/";
    QDir dir;
    dir.mkpath(storageLocation);
    
    _folders.append(storageLocation);
}


//! --------- DataBasePlsBuilder::run ------------------------------------------ 
void DataBasePlsBuilder::run()
{
    _isRunning = true;
    if (_folders.isEmpty()) return;
    
    DatabaseManager dbManager("PlaylistUpdateConnection");
    if (!dbManager.connectToDB()) return;
    qDebug() << " --- DataBasePlsBuilder -> starting Database update";

    { // Populate dbFiles map with All Playlist from db
        QSqlQuery query("SELECT filename, mtime FROM playlists;",*dbManager.sqlDb());
        while (query.next()) {
            dbFiles.insert(query.value(0).toString(),query.value(1).toUInt());
        }
     }

     QFileInfo fileInfo;

      //On SQLite --> it's MUCH faster to have everything in one transaction 
      //with only one disk write than to commit every each insert individually
      QSqlQuery("BEGIN TRANSACTION;",*dbManager.sqlDb());

      do 
      {
          QDir dirlist(_folders.takeFirst());
          dirlist.setNameFilters(playlistFilters);

          QDirIterator dirIterator(dirlist,QDirIterator::Subdirectories);
          while (dirIterator.hasNext()) {
              dirIterator.next();
              fileInfo = dirIterator.fileInfo();
                    
              if (fileInfo.isFile()) {
                  //! If the file is not in database then insert
                  if (!dbFiles.contains(fileInfo.filePath().toUtf8())) {
                       insertPlaylist(fileInfo.absoluteFilePath(), dbManager.sqlDb());
                       dbFiles.remove(fileInfo.filePath().toUtf8());
                  }

                  //! If the file is in database but has another mtime then update it
                  if ((dbFiles.contains(fileInfo.filePath().toUtf8())) &&
                      (dbFiles[fileInfo.filePath().toUtf8()]!=fileInfo.lastModified().toTime_t())) {
                          
                        updatePlaylist(fileInfo.absoluteFilePath(), dbManager.sqlDb());
                        dbFiles.remove(fileInfo.filePath().toUtf8());
                  }

                  //! If the file is in database and was not changed then do nothing and just
                  // remove it from file to proceed
                  if ((dbFiles.contains(fileInfo.filePath().toUtf8())) &&
                      (dbFiles[fileInfo.filePath().toUtf8()]==fileInfo.lastModified().toTime_t())) {
                        dbFiles.remove(fileInfo.filePath().toUtf8());
                  }
              } //IF fileInfo.isFile()
          } // WHILE dirIterator.hasNext
      } while (_folders.size()>0);

      //! Get files that are in DB but not on harddrive
      QHashIterator<QString, uint> i(dbFiles);
      while (i.hasNext()) {
          i.next();
          removePlaylist(i.key(), dbManager.sqlDb());
      }
      dbFiles.clear();

      // Check for playlist item not used anymore
      //!cleanUpDatabase(dbManager.sqlDb());
      // Now write all data to the disk
      QSqlQuery("COMMIT TRANSACTION;",*dbManager.sqlDb());

      emit buildingFinished();
      _isRunning = false;
       qDebug() << " --- DataBasePlsBuilder -> end Database update";

}



//! --------- DataBasePlsBuilder::insertPlaylist -------------------------------
void DataBasePlsBuilder::insertPlaylist(QString filename, QSqlDatabase *sqlDB)
{
    QSqlField data("col",QVariant::String);

    QFileInfo fileInfo(filename);

    QString fname = fileInfo.filePath().toUtf8();
    QString pname = fileInfo.baseName();

    uint mtime = QFileInfo(filename).lastModified().toTime_t();
    qDebug() << "--- DataBasePlsBuilder -> insert playlist :" << filename;

    int type = filename.contains(_CONFIGDIR + "/playlists/") ? 0 : 1;
    
    //! PLAYLIST part in database
    {
        QSqlQuery query(*sqlDB);
        query.prepare("INSERT INTO `playlists`(`filename`,`name`,`type`,`mtime`)" \
                      "VALUES(?," \
                      "       ?," \
                      "       ?," \
                      "       ?);");
        query.addBindValue(fname);
        query.addBindValue(pname);
        query.addBindValue(type);
        query.addBindValue(mtime);
        query.exec();
    }
    
    data.setValue(fname);
    QString fnameDB= sqlDB->driver()->formatValue(data,false);

	
    QStringList list = MEDIA::readPlaylistFile(filename);
    
    foreach (const QString &url, list) {
      if(url.isEmpty()) continue;
      QString itemUrlName   = url; //QFileInfo(url).filePath().toUtf8();
      QString itemName      = QFileInfo(url).baseName();

      //! Playlist Item part in database
      {
        QSqlQuery itemQuery(*sqlDB);
        qDebug() << "--- DataBasePlsBuilder -> insert playlistitem itemUrlName:" << itemUrlName;

        itemQuery.prepare("INSERT INTO `playlistitem`(`urlname`,`name`,`playlist`)" \
                      "VALUES(?," \
                      "       ?," \
	              "       (SELECT `id` FROM `playlists` WHERE `filename`="+fnameDB+"));");
        itemQuery.addBindValue(itemUrlName);
        itemQuery.addBindValue(itemName);
        itemQuery.exec();
      }  

    } // end foreach url into playlist
}


//! --------- DataBasePlsBuilder::updatePlaylist -------------------------------
void DataBasePlsBuilder::updatePlaylist(QString filename, QSqlDatabase *sqlDb)
{
    removePlaylist(filename,sqlDb);
    insertPlaylist(filename,sqlDb);
}


//! --------- DataBasePlsBuilder::removePlaylist -------------------------------
void DataBasePlsBuilder::removePlaylist(QString filename, QSqlDatabase *sqlDb)
{
    qDebug() << " --- DataBasePlsBuilder -> Deleting playlist :" << filename;
    QFileInfo fileInfo(filename);
    QString fname = fileInfo.filePath().toUtf8();

    QSqlField data("col",QVariant::String);
    data.setValue(fname);
    fname = sqlDb->driver()->formatValue(data,false);
    QSqlQuery query("DELETE FROM `playlists` WHERE `filename`="+fname+";", *sqlDb);
}

//! --------- DataBasePlsBuilder::cleanUpDatabase ------------------------------
void DataBasePlsBuilder::cleanUpDatabase(QSqlDatabase *sqlDb)
{
  //! suppression des playlist item qui ne sont pas rattachs  une playlist existante
  {
    QSqlQuery query("DELETE FROM `playlistitem` WHERE `playlists` NOT IN (SELECT `id` FROM `playlists`);", *sqlDb);
  }
}
