/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "mediaplaylist.h"
#include "mediaitem.h"

#include <QtCore>


const QStringList m3uFilter = QStringList() << "m3u" << "m3u8";
const QStringList plsFilter = QStringList() << "pls";
const QStringList xspfFilter = QStringList() << "xspf";

//! -------- MEDIA::readM3uPlaylist --------------------------------------------
QStringList MEDIA::readPlaylistFile(QString filename)
{
  qDebug() << "---- MEDIA::readPlaylistFile" << QUrl(filename).toLocalFile();

  QStringList mediaList;
  if((m3uFilter).contains(QFileInfo(filename).suffix().toLower())) {
    mediaList = MEDIA::readM3uPlaylist(QUrl(filename).toLocalFile());
  }
  else if((plsFilter).contains(QFileInfo(filename).suffix().toLower())) {
    mediaList = MEDIA::readPlsPlaylist(QUrl(filename).toLocalFile());
  }
  else if((xspfFilter).contains(QFileInfo(filename).suffix().toLower())) {
    mediaList = MEDIA::readXspfPlaylist(QUrl(filename).toLocalFile());
  }
  return mediaList;
}



//! -------- MEDIA::readM3uPlaylist --------------------------------------------
QStringList MEDIA::readM3uPlaylist(QString filename)
{
  QStringList mediaFiles;
  
  QFile file(filename);
  if (!file.open(QFile::ReadOnly)) {
    qDebug() << "Failed to open read only file " << filename;
    return mediaFiles;
  }

  QString data = QString::fromUtf8(file.readAll());
  if(data.isEmpty()) return mediaFiles;
  data.replace('\r', '\n');
  QByteArray bytes = data.toUtf8();
  QBuffer buffer(&bytes);
  buffer.open(QIODevice::ReadOnly);
  
  QString line;
  //! main reading loop
  while(!buffer.atEnd()) {
    line = QString::fromUtf8(buffer.readLine()).trimmed();
    if (line.startsWith('#') || line.isEmpty()) {
          continue;
    }
    else {
          qDebug() << "---- readM3uPlaylist -> line " << line << "\n";

        //! Find the Track location
         if (line.contains(QRegExp("^[a-z]+://"))) {
	   //! remote URL
	   QUrl url(line);
	   if (url.isValid()) {
	     qDebug() << "---- readM3uPlaylist -> url.isValid()" << url;
	     mediaFiles << url.toString();
	   }
	 }
         else if (QDir::isAbsolutePath(line)) {
	   //! Absolute path  --> It's OK
	   //qDebug() << "---- readM3uPlaylist -> isAbsolutePath\n";
           mediaFiles << QDir::fromNativeSeparators(line); // handles \\ under windows
         }
         else {
	   //! Relative path  --> make it absolute
	   //qDebug() << "---- readM3uPlaylist -> Relative\n";    
           QString relativePath = QDir::fromNativeSeparators(line);
           QDir playlistDir(QFileInfo(filename).absoluteDir());
           QString absolutePath = playlistDir.absoluteFilePath(relativePath);
           mediaFiles << absolutePath;
         } 
    }
   // if (buffer.atEnd()) break;
  } // end forever
    
  return mediaFiles;
}
    
//! -------- MEDIA::readPlsPlaylist --------------------------------------------
QStringList MEDIA::readPlsPlaylist(QString filename) 
{
  QStringList mediaFiles;
  
  QFile file(filename);
  if (!file.open(QFile::ReadOnly)) {
    qDebug() << "Failed to open read only file " << filename;
    return mediaFiles;
  }

  while (!file.atEnd()) {
    QString line = QString::fromUtf8(file.readLine()).trimmed();
    int equals = line.indexOf('=');
    QString key = line.left(equals).toLower();
    QString value = line.mid(equals + 1);

    if (key.startsWith("file")) 
    {
        //! Find the Track location
         if (value.contains(QRegExp("^[a-z]+://"))) {
	   //! remote URL
	   QUrl url(value);
	   if (url.isValid()) {
	     qDebug() << "---- readPlsPlaylist -> url.isValid()\n";
	     mediaFiles << value;
	   }
	 }
         else if (QDir::isAbsolutePath(value)) {
	   //! Absolute path  --> It's OK
	   qDebug() << "---- readPlsPlaylist -> isAbsolutePath\n";
           mediaFiles << QDir::fromNativeSeparators(value);
         }
         else {
	   //! Relative path  --> make it absolute
	   qDebug() << "---- readPlsPlaylist -> Relative\n";           
           QString relativePath = QDir::fromNativeSeparators(value);
           QDir playlistDir(QFileInfo(filename).absolutePath());
           QString absolutePath = playlistDir.absoluteFilePath(relativePath);
	   mediaFiles << absolutePath;        
         }       
    }
  }

  return mediaFiles;
}  
//! -------- MEDIA::readXspfPlaylist -------------------------------------------
QStringList MEDIA::readXspfPlaylist(QString filename) 
{
  QStringList mediaFiles;  

  //! open file to read
  QFile file(filename);  
  if(!file.open(QIODevice::ReadOnly))
    return mediaFiles;

  
  QXmlStreamReader xml(&file);  
  while (!xml.atEnd()) {
    QXmlStreamReader::TokenType type = xml.readNext();
    if ((type == QXmlStreamReader::StartElement) && (xml.name() == "track")) {

      //! track found
       while (!xml.atEnd()) {
	   QXmlStreamReader::TokenType type = xml.readNext();
	   if ((type == QXmlStreamReader::StartElement) && (xml.name() == "location")) {
	       
	      QString value = QString(xml.readElementText());
	      qDebug() << "---- readXspfPlaylist -> location found\n" << value;           

	      //! Find the Track location
	      if (value.contains(QRegExp("^[a-z]+://"))) {
		//! remote URL
		QUrl url(value);
		if (url.isValid()) {
		  qDebug() << "---- readXspfPlaylist -> url.isValid()\n";
		  mediaFiles << value;
		}
	      }
	      else if (QDir::isAbsolutePath(value)) {
		//! Absolute path  --> It's OK
		qDebug() << "---- readXspfPlaylist -> isAbsolutePath\n";
		mediaFiles << QDir::fromNativeSeparators(value);
	      }
	      else {
		    //! Relative path  --> make it absolute
		    qDebug() << "---- readXspfPlaylist -> Relative\n";           
		    QString relativePath = QDir::fromNativeSeparators(value);
		    QDir playlistDir(QFileInfo(filename).absolutePath());
		    QString absolutePath = playlistDir.absoluteFilePath(relativePath);
		    mediaFiles << absolutePath;
	      }
	  }
       }
    } // end StartElement
  }  // End while xml end

  return mediaFiles;
}  

//! ----------------------- MEDIA::savePlaylistFile ----------------------------
void MEDIA::savePlaylistFile(QString filename, QList<MediaItem*> _listMediaItem) 
{
  if((m3uFilter).contains(QFileInfo(filename).suffix().toLower())) {
    MEDIA::saveM3uPlaylist(filename, _listMediaItem);
  }
  else if((plsFilter).contains(QFileInfo(filename).suffix().toLower())) {
    MEDIA::savePlsPlaylist(filename, _listMediaItem);
  }
  else if((xspfFilter).contains(QFileInfo(filename).suffix().toLower())) {
    MEDIA::saveXspfPlaylist(filename, _listMediaItem);
  }
}


//! ----------------------- MEDIA::saveM3UPlaylist -----------------------------
void MEDIA::saveM3uPlaylist(QString filename, QList<MediaItem*> _listMediaItem) 
{
  //! get the playlist file directory
  QDir playlistDir(QFileInfo(filename).absolutePath());

  //! open file to write
  QFile file(filename);  
  if(!file.open(QIODevice::WriteOnly)) return;  
  
  file.write("#EXTM3U\n");

  foreach (MediaItem* mi, _listMediaItem) 
  {
    if (!mi) continue;

    QString info = QString("#EXTINF:%1,%2 - %3\n").arg(mi->duration).arg(mi->artist).arg(mi->album);
    file.write(info.toUtf8());
    
    //! Get the path to MediaItem relative to the Playlist File directory
    QString mediaFilePath = mi->url;
    if (!mediaFilePath.contains(QRegExp("^[a-z]+://"))) {
       mediaFilePath = playlistDir.relativeFilePath(mediaFilePath);
    }
     
    file.write(mediaFilePath.toUtf8());
    file.write("\n");
  }
  
  file.close();
}


//! ----------------------- MEDIA::savePlsPlaylist --------------------
void MEDIA::savePlsPlaylist(QString filename, QList<MediaItem*> _listMediaItem) 
{
  //! get the playlist file directory
  QDir playlistDir(QFileInfo(filename).absolutePath());
  
  //! open file to write
  QFile file(filename);  
  if(!file.open(QIODevice::WriteOnly)) return;  
  
  QTextStream textStream(&file);
  textStream << "[playlist]" << endl;
  textStream << "Version=2" << endl;
  textStream << "NumberOfEntries=" << _listMediaItem.size() << endl;

  int n = 1;
  foreach (MediaItem* mi, _listMediaItem) 
  {
    if (!mi) continue;
    //! Get the path to MediaItem relative to the Playlist File directory
    QString mediaFilePath = mi->url;
    if (!mediaFilePath.contains(QRegExp("^[a-z]+://"))) {
       mediaFilePath = playlistDir.relativeFilePath(mediaFilePath);
    }
    
    textStream << "File" << n << "=" << mediaFilePath << endl;
    textStream << "Title" << n << "=" << mi->title << endl;
    textStream << "Length" << n << "=" << mi->duration << endl;
    ++n;
  }
}

//! ----------------------- MEDIA::saveXspfPlaylist --------------------
void MEDIA::saveXspfPlaylist(QString filename, QList<MediaItem*> _listMediaItem) 
{
  //! get the playlist file directory
  QDir playlistDir(QFileInfo(filename).absolutePath());
  
  //! open file to write
  QFile file(filename);  
  if(!file.open(QIODevice::WriteOnly)) return;  
  
  QXmlStreamWriter xml(&file);
  xml.writeStartDocument();
  xml.writeStartElement("playlist");
  xml.writeAttribute("version", "1");
  xml.writeDefaultNamespace("http://xspf.org/ns/0/");

  xml.writeStartElement("trackList");
  foreach (MediaItem* mi, _listMediaItem) 
  {
    if (!mi) continue;
      
    //! Get the path to MediaItem relative to the Playlist File directory
    QString mediaFilePath = mi->url;
    if (!mediaFilePath.contains(QRegExp("^[a-z]+://"))) {
       mediaFilePath = playlistDir.relativeFilePath(mediaFilePath);
    }
    
    xml.writeStartElement("track");
    xml.writeTextElement("location", mediaFilePath);
    xml.writeTextElement("title", mi->title);
    if (!mi->artist.isEmpty()) {
      xml.writeTextElement("creator", mi->artist);
    }
    if (!mi->album.isEmpty()) {
      xml.writeTextElement("album", mi->album);
    }
    if (mi->duration > 0) {
      xml.writeTextElement("duration", QString::number(mi->duration * 1000));
    }
    xml.writeEndElement(); //track
  }
  xml.writeEndElement(); //trackList
  xml.writeEndElement(); //playlist
  xml.writeEndDocument(); 
}
