/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "NowPlaying/nowplayingview.h" 
#include "MediaItem/mediaitem.h"
#include "constants.h"

#include <QMap>

//! -------- NowPlayingView class ------------------------------------------------
NowPlayingView::NowPlayingView(QWidget *parent) : QFrame(parent)
{
  setObjectName(QString::fromUtf8("nowplayingwidget"));
  
  QPalette palette;
  palette.setColor(QPalette::Background, Qt::white);
  this->setPalette(palette);
  
  setFrameShape(QFrame::NoFrame);
  
  setFixedHeight (130);
  setFocusPolicy( Qt::NoFocus );
  setAutoFillBackground(true);  
  
  //! set player
  _Player = Player::instance();
  connect(_Player, SIGNAL(trackChanged(QString)), this, SLOT(updateSource(QString)));  
  connect(_Player, SIGNAL(metaDataChanged()), this, SLOT(updateMetaData()));  

  //! init and update
  _mediaItem    = new MediaItem();
  _state        = Stopped;
  _streamName   = "";
  _pix          = QPixmap();
  emit update();
}


//! -------- updateSource ------------------------------------------------------
void NowPlayingView::updateSource(QString filename)
{
  qDebug() << "--- UpdateNowPlaying --> updateSource" << filename;
  if(filename.isEmpty()) {
    _state = Stopped;
    emit update();
    return;
  }

  _state = Playing; 
  _pix   = QPixmap();
  
  //! ----- Local File Case ----------------------------------------------------
  if(MEDIA::isUrlLocal(filename)) 
  {
    _sourceType = LocalType;
    
    _mediaItem = MEDIA::FromDataBase(filename);
    if(!_mediaItem)
      _mediaItem     = MEDIA::FromLocalFile(QFileInfo(filename).absoluteFilePath());

    QString coverpath = _CONFIGDIR + "/albums/" + MEDIA::getCoverName(_mediaItem);
    _pix = QPixmap::fromImage(QImage(coverpath));
    
    if (_pix.isNull()) _pix = MEDIA::getPixmapFromMediaUrl(filename, QSize(110,110));
    if (_pix.isNull()) _pix = QPixmap(":/images/emptycoverblue110x110.png");
  }
  //! ----- Remote Url  --------------------------------------------------------
  else  
  {
     _sourceType = UrlType;

    _streamName          = QFileInfo(filename).baseName(); //! A VOIR
    _mediaItem->title    = "";
    _mediaItem->album    = "";
    _mediaItem->artist   = "";
    _pix           = QPixmap(":/images/podcast110x110.png");
  }
        
  // update NowPlaying Widget --> repaintevent called
  emit update();
}



//! -------- UpdateMetaData ----------------------------------------------------
void NowPlayingView::updateMetaData()
{
  qDebug() << "--- UpdateNowPlaying --> updateMetaData";

  if(_state == Stopped || _sourceType == LocalType)
    return;

  QMap<QString, QString> metaData = _Player->currentMetaData();

  _mediaItem->title    = metaData.value("TITLE");
  _mediaItem->album    = metaData.value("ALBUM");
  _mediaItem->artist   = metaData.value("ARTIST");
        
  //! update NowPlaying Widget --> repaintevent called
  emit update();
}




//! -------- NowPlayingView::paintEvent ----------------------------------------
void NowPlayingView::paintEvent(QPaintEvent *)
{
  QPainter painter(this);
  const int width = this->width();
  QRectF rect(130, 15, width-135, 20);
	  
  //! -- Playing State
  if(_state == Playing) {
    if(_sourceType == LocalType) {
        //! paint title
        painter.setFont(QFont("Arial", 10, QFont::Bold));
        QFontMetrics fm( QFont("Arial", 10, QFont::Bold));   
        QString titleTruncated = fm.elidedText ( _mediaItem->title, Qt::ElideRight, width-135 );
        painter.drawText(rect, Qt::AlignVCenter | Qt::AlignLeft,titleTruncated );

        //! paint album    
        painter.setFont(QFont("Arial", 10, QFont::Normal));
        QFontMetrics fm2( QFont("Arial", 10, QFont::Normal));   
        QString albumTruncated = fm2.elidedText ( _mediaItem->album, Qt::ElideRight, width-135 );
        painter.drawText(rect.adjusted(0,25,0,25), Qt::AlignVCenter | Qt::AlignLeft,albumTruncated );

        //! paint artist    
        QString artistTruncated = fm2.elidedText ( _mediaItem->artist, Qt::ElideRight, width-135 );
        painter.drawText(rect.adjusted(0,50,0,50), Qt::AlignVCenter | Qt::AlignLeft,artistTruncated );
    
        //! paint duration    
        if(_mediaItem->duration > 0) {
          QString durationText = MEDIA::getDurationText(_mediaItem->duration);
          QString dureeTruncated = fm2.elidedText ( durationText, Qt::ElideRight, width-135 );
          painter.drawText(rect.adjusted(0,75,0,75), Qt::AlignVCenter | Qt::AlignLeft,dureeTruncated );
        }

        //! paint pixmap (coverart)
        painter.drawPixmap(10,10,110,110, _pix);
    }
    else {
        //! paint stream name
        painter.setFont(QFont("Arial", 10, QFont::Bold));
        QFontMetrics fm( QFont("Arial", 10, QFont::Bold));   
        QString streamTruncated = fm.elidedText ( _streamName, Qt::ElideRight, width-135 );
        painter.drawText(rect, Qt::AlignVCenter | Qt::AlignLeft,streamTruncated );

        //! paint title    
        painter.setFont(QFont("Arial", 10, QFont::Normal));
        QFontMetrics fm2( QFont("Arial", 10, QFont::Normal));   
        QString titleTruncated = fm2.elidedText ( _mediaItem->title, Qt::ElideRight, width-135 );
        painter.drawText(rect.adjusted(0,25,0,25), Qt::AlignVCenter | Qt::AlignLeft,titleTruncated );

        //! paint album    
        QString albumTruncated = fm2.elidedText ( _mediaItem->album, Qt::ElideRight, width-135 );
        painter.drawText(rect.adjusted(0,50,0,50), Qt::AlignVCenter | Qt::AlignLeft,albumTruncated );

        //! paint artist    
        QString artistTruncated = fm2.elidedText ( _mediaItem->artist, Qt::ElideRight, width-135 );
        painter.drawText(rect.adjusted(0,75,0,75), Qt::AlignVCenter | Qt::AlignLeft,artistTruncated );

        //! paint pixmap (coverart)
        painter.drawPixmap(8,8,110,110, _pix); 
    }
  }
  //! -- Stopped State
  else 
  {
    painter.setFont(QFont("Arial", 10, QFont::Bold));
    QFontMetrics fm( QFont("Arial", 10, QFont::Bold));   
    QString text = fm.elidedText ( tr("Playlist"), Qt::ElideRight, width-135 );
    painter.drawText(rect, Qt::AlignVCenter | Qt::AlignLeft,text );
    
    painter.setFont(QFont("Arial", 10, QFont::Normal));
    QFontMetrics fm2( QFont("Arial", 10, QFont::Normal));   
    QString text2 = fm2.elidedText ( tr("Playing Stopped"), Qt::ElideRight, width-135 );
    painter.drawText(rect.adjusted(0,25,0,25), Qt::AlignVCenter | Qt::AlignLeft,text2 );
    
    painter.drawPixmap(10,10,64,64, QPixmap(":/icon/64x64/yarock.png"));
   }
   painter.end();
}
