/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "playlistmodel.h"
#include "MediaItem/mediaitem.h"
#include "MediaItem/mediamimedata.h"


//! ------------------ Global PlaylistModel instance -------------------------------
PlaylistModel* PlaylistModel::globalPlaylistModel = 0;

namespace GLOBAL
{
    PlaylistModel* playlistModel()
    {
        return PlaylistModel::instance();
    }
}

//!------------PlaylistModel ---------------------------------------------------
PlaylistModel::PlaylistModel(QWidget *parent) : QAbstractListModel(parent) 
{
  globalPlaylistModel = this;
    _activeMediaItem =  0;
    _activeRow       = -1;
}


//!------------PlaylistModel::rowCount -----------------------------------------
// QAbstractListModel implementation
int PlaylistModel::rowCount(const QModelIndex & /* parent */) const 
{
    return _listMediaItem.size();
}

//!------------PlaylistModel::data ---------------------------------------------
// QAbstractListModel implementation
QVariant PlaylistModel::data(const QModelIndex &index, int role) const
{
    const int row = index.row();
    if (row < 0 || row >= _listMediaItem.size())
        return QVariant();

    MediaItem *mediaItem = _listMediaItem.at(row);
    if (!mediaItem) return QVariant();

    switch (role) {

    case Playlist::ExistItemRole:
        return QVariant::fromValue(mediaItem->isExists);
    case Playlist::PlayingItemRole:
        return QVariant::fromValue(_activeRow);
    }

    return QVariant();
}


//!------------PlaylistModel::setInactiveMediaItem -----------------------------
// Specific method --> stop graphical activated item
void PlaylistModel::setInactiveMediaItem()
{
  if (_activeMediaItem) {
    _activeMediaItem->isPlaying = false;
	 	_activeMediaItem = 0;
	
    QModelIndex newIndex = index(_activeRow, 0, QModelIndex());
	  emit dataChanged(newIndex, newIndex);
	  _activeRow = -1;
	  emit activeRowChanged();
    }
}

//!------------PlaylistModel::setInactiveMediaItem -----------------------------
// Specific method --> stop graphical activated item
void PlaylistModel::setActiveRow(int row)
{
  if (!rowExists(row)) return;

  const int oldActiveRow = _activeRow;

  if (rowExists(oldActiveRow)) {
    _activeMediaItem = MediaItemAt(oldActiveRow);
    _activeMediaItem->isPlaying = false;   
    QModelIndex oldIndex = index(oldActiveRow, 0, QModelIndex());
    emit dataChanged(oldIndex, oldIndex);
  }

  _activeRow = row;
  _activeMediaItem = MediaItemAt(_activeRow);
  _activeMediaItem->isPlaying = true;

  QModelIndex newIndex = index(_activeRow, 0, QModelIndex());
  emit dataChanged(newIndex, newIndex);
  emit activeRowChanged();
}

//!------------PlaylistModel::skipBackward -------------------------------------
// Specific method
void PlaylistModel::skipBackward()
{
   MediaItem *previousMediaItem = 0;

   int prevRow = _activeRow - 1;
   if (rowExists(prevRow)) {
      previousMediaItem = _listMediaItem.at(prevRow);
      previousMediaItem->isPlaying = false;

      if (previousMediaItem) {
	  int prevRow = _listMediaItem.indexOf(previousMediaItem);
	  setActiveRow(prevRow);
      }
   }
}

//!------------PlaylistModel::skipForward --------------------------------------
// Specific method
void PlaylistModel::skipForward() {

   MediaItem *nextMediaItem = 0;

   int nextRow = _activeRow + 1;
   if (rowExists(nextRow)) {
     nextMediaItem = _listMediaItem.at(nextRow);
     nextMediaItem->isPlaying = false;

     if (nextMediaItem) {
       int nextRow = _listMediaItem.indexOf(nextMediaItem);
       setActiveRow(nextRow);
     }
   }
}

//!------------PlaylistModel::MediaItemAt --------------------------------------
// Specific method
MediaItem* PlaylistModel::MediaItemAt(int row) const 
{
    if (rowExists(row)) {
      return _listMediaItem.at(row);
    }
    return 0;
}

//!------------PlaylistModel::getActiveMediaItem -------------------------------
// Specific method
MediaItem* PlaylistModel::getActiveMediaItem() const
{
    if(_activeMediaItem)
      return _activeMediaItem;
    return 0;
}

//!------------PlaylistModel::getActiveUrl -------------------------------------
// Specific method
QString PlaylistModel::getActiveUrl() const 
{
    QString url;
    if(_activeMediaItem)
      url = _activeMediaItem->url;
    return url;
}


//!------------PlaylistModel::addMediaItem -------------------------------------
// Specific method
void PlaylistModel::addMediaItem(MediaItem* mediaitem) 
{
    // no duplicates
    if (!_listMediaItem.contains(mediaitem)) {
        beginInsertRows(QModelIndex(), _listMediaItem.size(), _listMediaItem.size());
        _listMediaItem << mediaitem;
        endInsertRows();
        emit layoutChanged();
    }
}

//!------------PlaylistModel::addMediaItemList ---------------------------------
void PlaylistModel::addMediaItemList(QList<MediaItem*> list)
{
    if (!list.empty()) {
        // remove duplicates
        foreach(MediaItem* mi, this->_listMediaItem) {
            list.removeAll(mi);
        }
        if (!_listMediaItem.empty()) {
            beginInsertRows(QModelIndex(), this->_listMediaItem.size(), this->_listMediaItem.size() + list.size() - 1);
            this->_listMediaItem.append(list);
            endInsertRows();
        }
    emit layoutChanged();
    }
}

//!------------PlaylistModel::getListDuration ----------------------------------
QString PlaylistModel::getListDuration()
{ 
  return MEDIA::getMediaListDurationText(_listMediaItem); 
}

//!------------PlaylistModel::clear --------------------------------------------
void PlaylistModel::clear()
{
    qDeleteAll(_listMediaItem.constBegin(), _listMediaItem.constEnd());
      
    _listMediaItem.clear();
    _activeMediaItem  = 0;
    _activeRow        = -1;
    emit layoutChanged();
    emit reset();
    emit activeRowChanged();
}


//!------------PlaylistModel::emit_layoutChanged -------------------------------
void PlaylistModel::emit_layoutChanged()
{
    emit layoutChanged();
}


//!------------PlaylistModel::removeRows ---------------------------------------
bool PlaylistModel::removeRows(int position, int rows, const QModelIndex & /*parent*/) 
{
    if( position + rows - 1 >= _listMediaItem.size()) return false;

    beginRemoveRows(QModelIndex(), position, position + rows - 1);
    for (int row = 0; row < rows; ++row) {
        MediaItem *mi = _listMediaItem.takeAt(position);
        if (mi) {
            mi->isPlaying = false;
        }
    }
    endRemoveRows();
    emit layoutChanged();
    return true;
}

//!------------PlaylistModel::removeIndexes ------------------------------------
void PlaylistModel::removeIndexes(QModelIndexList &indexes) 
{
    QList<MediaItem*> originalList(_listMediaItem);
    QList<MediaItem*> delitems;
    foreach (QModelIndex index, indexes) {
        MediaItem* mi = originalList.at(index.row());
        int idx = _listMediaItem.indexOf(mi);
        if (idx != -1) {
            beginRemoveRows(QModelIndex(), idx, idx);
            delitems.append(mi);
            _listMediaItem.removeAll(mi);
            endRemoveRows();
            mi->isPlaying = false;
        }
    }
}


//! DRAG AND DROP
//!------------PlaylistModel::supportedDropActions -----------------------------
Qt::DropActions PlaylistModel::supportedDropActions() const 
{
    return Qt::CopyAction | Qt::MoveAction; // deplace les donn�es de la source vers la cible
}

Qt::ItemFlags PlaylistModel::flags(const QModelIndex &index) const {
    Qt::ItemFlags defaultFlags = QAbstractListModel::flags(index);
    if (index.isValid()) {
        return ( defaultFlags | Qt::ItemIsDropEnabled | Qt::ItemIsDragEnabled );
    } else
        return Qt::ItemIsDropEnabled | defaultFlags;
}

//!------------PlaylistModel::mimeTypes ----------------------------------------
// Encoding Exported Data with mimeTypes
QStringList PlaylistModel::mimeTypes() const
{
    QStringList types;
    types << MEDIA_MIME;
    types << "text/uri-list"; //we do accept urls
    return types;
}

//!------------PlaylistModel::mimeData -----------------------------------------
QMimeData* PlaylistModel::mimeData( const QModelIndexList &indexes ) const 
{
    MediaMimeData* mime = new MediaMimeData();

    foreach( const QModelIndex &index, indexes ) {
        int row = index.row();
        
        if (row >= 0 && row < _listMediaItem.size())
            mime->addMediaItem( _listMediaItem.at(row) );
    }

    return mime;
}


//!------------PlaylistModel::dropMimeData -------------------------------------
// --> Inserting Dropped Data into a Model
bool PlaylistModel::dropMimeData(const QMimeData *data,
                                 Qt::DropAction action, int row, int column,
                                 const QModelIndex &parent) 
{
    qDebug() << "Drop MimeData";
    QAbstractListModel::dropMimeData(data, action, row, column, parent);
    
    if (action == Qt::IgnoreAction)
        return true;

    if (column > 0)
        return false;
    
    
    if (data->hasFormat(MEDIA_MIME)) {
        
      int beginRow;
      if (row != -1)
	  beginRow = row;
      else if (parent.isValid())
	  beginRow = parent.row();
      else
	  beginRow = rowCount();

      const MediaMimeData* mediaMimeData = dynamic_cast<const MediaMimeData*>( data );
      if(!mediaMimeData ) return false;

      int counter = 0;
      QList<MediaItem*> droppedMediaItems = mediaMimeData->getMediaItem();

      qDebug() << "Dropped" << droppedMediaItems << "at" << beginRow;

      foreach(MediaItem *mi, droppedMediaItems) {

	  // remove track
	  const int miRow = _listMediaItem.indexOf(mi);
	  if (miRow != -1)
	      removeRows(miRow, 1, QModelIndex());

	  // and then add it at the new position
	  const int targetRow = beginRow + counter;
	  beginInsertRows(QModelIndex(), targetRow, targetRow);
	  _listMediaItem.insert(targetRow, mi);
	  endInsertRows();
	  counter++;
      }

    // fix activeRow after mediaItem insertion
    _activeRow = _listMediaItem.indexOf(_activeMediaItem);
    
    }
    else if( data->hasUrls() )
    {
      qDebug() << "PlaylistModel::dropMimeData --> data->hasUrls()";
      QList<QUrl> listUrl = data->urls();
      emit signalAddUrls(listUrl);
    }
    else {
     return false; 
    }

    emit layoutChanged();
    return true;
}


//!------------PlaylistModel::rowForMediaItem ----------------------------------
int PlaylistModel::rowForMediaItem(MediaItem* mediaitem)
{
    return _listMediaItem.indexOf(mediaitem);
}


//!------------PlaylistModel::indexForMediaItem --------------------------------
QModelIndex PlaylistModel::indexForMediaItem(MediaItem* mediaitem)
{
    return createIndex(_listMediaItem.indexOf(mediaitem), 0);
}

//!------------PlaylistModel::move ---------------------------------------------
void PlaylistModel::move(QModelIndexList &indexes, bool up) 
{
    QList<MediaItem*> movedMediaItems;

    foreach (QModelIndex index, indexes) {
        int row = index.row();
        // qDebug() << "index row" << row;
        MediaItem *mi = MediaItemAt(row);
        if (mi)
            movedMediaItems << mi;
    }

    int counter = 1;
    foreach (MediaItem *mi, movedMediaItems) {

        int row = rowForMediaItem(mi);
        // qDebug() << "track row" << row;
        removeRows(row, 1, QModelIndex());

        if (up) row--;
        else row++;

        beginInsertRows(QModelIndex(), row, row);
        _listMediaItem.insert(row, mi);
        endInsertRows();

        counter++;
    }
    emit layoutChanged();
}
