/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Playlist/playlistview.h" 
#include "Playlist/playlistmodel.h" 
#include "MediaItem/mediaitem.h" 

#include <QFileInfo>
#include <QFileIconProvider>
#include <QtDebug>
#include <QString>

namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}

//!---------- PlaylistView -----------------------------------------------------
PlaylistView::PlaylistView(QWidget *parent) : 
        QListView(parent),
        _playlistModel(0) 
{
  //! widget
  setEnabled(true);
  setAlternatingRowColors(false);
  setEditTriggers(QAbstractItemView::NoEditTriggers);
  setSelectionMode(QAbstractItemView::SingleSelection); 
  setUniformItemSizes(true); //!optim
  setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
  
  //! dragndrop
  setDragEnabled(true);
  setAcceptDrops(true);
  setDropIndicatorShown(false);
  setDragDropMode(QAbstractItemView::DragDrop);  
  
  //! Delegate
  _playlistdelegate = new PlaylistDelegate(this);
  setItemDelegate(_playlistdelegate);
   
  //! Menu contextuel 
  _contextMenu = new QMenu(this);
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  _contextMenu->addAction(actions->value("addfile"));
  _contextMenu->addAction(actions->value("adddir"));
  _contextMenu->addAction(actions->value("addurl"));    
  _contextMenu->addAction(actions->value("clear"));
  _contextMenu->addAction(actions->value("save"));
  _contextMenu->addAction(actions->value("autosave"));
    
  removeMediaItemAct = new QAction(QIcon::fromTheme("edit-delete"),tr("&Remove media from playlist"), this);
  removeMediaItemAct->setEnabled(false);
  _contextMenu->addAction(removeMediaItemAct);

  _contextMenu->addSeparator();
  QMenu *menuMode = _contextMenu->addMenu(tr("&Choose playlist mode"));
  menuMode->addAction(actions->value("playlistModeTitle"));
  menuMode->addAction(actions->value("playlistModeAlbum"));
  menuMode->addAction(actions->value("playlistModeArtist"));
  _contextMenu->addSeparator();
  _contextMenu->addAction(actions->value("showPlaylist"));
  
  //! Signaux
  connect(removeMediaItemAct, SIGNAL(triggered()), this, SLOT(RemoveTrack()));
  connect(this, SIGNAL(activated(const QModelIndex &)), SLOT(itemActivated(const QModelIndex &)));
}

//!---------- PlaylistView::setPlaylistModel -----------------------------------
void PlaylistView::setPlaylistModel(PlaylistModel *playlistModel) 
{
  this->_playlistModel = playlistModel;
  setModel(playlistModel);
  
  connect(_playlistModel, SIGNAL(layoutChanged()), this, SLOT(updatePlaylistActions()));
  connect(_playlistModel, SIGNAL(rowsInserted(const QModelIndex &, int, int)), this, SLOT(updatePlaylistActions()));
  connect(_playlistModel, SIGNAL(rowsRemoved(const QModelIndex &, int, int)), this, SLOT(updatePlaylistActions()));
  connect(_playlistModel, SIGNAL(modelReset()),this,  SLOT(updatePlaylistActions()));
  
  updatePlaylistActions();
}

//!---------- PlaylistView::updatePlaylistActions ------------------------------
void PlaylistView::updatePlaylistActions()
{
    const int rowCount  = _playlistModel->rowCount(QModelIndex());
    //const int activeRow = _playlistModel->getActiveRow();
    const bool isPlaylistEmpty = rowCount < 1;
    
    GLOBAL::globalActions()->value("clear")->setEnabled(!isPlaylistEmpty);
    GLOBAL::globalActions()->value("save")->setEnabled(!isPlaylistEmpty);
    GLOBAL::globalActions()->value("autosave")->setEnabled(!isPlaylistEmpty);
    removeMediaItemAct->setEnabled((!isPlaylistEmpty) && (_selectedTrack>-1));
}


//!---------- PlaylistView::selectionChanged -----------------------------------
void PlaylistView::selectionChanged(const QItemSelection &selected, const QItemSelection &deselected) 
{
    QModelIndexList indexlist = selected.indexes();
    QModelIndex index = indexlist[0];
    _selectedTrack = index.row();

    QListView::selectionChanged(selected, deselected);
}

//!---------- PlaylistView::itemActivated --------------------------------------
// gestion du doucle click sur un item
void PlaylistView::itemActivated(const QModelIndex &index)
{
    if (_playlistModel->rowExists(index.row())) {
      _playlistModel->setActiveRow(index.row());
      emit signal_playlist_itemDoubleClicked();
      updatePlaylistActions();
    }
}

//!---------- PlaylistView::ClearPlaylist --------------------------------------
void PlaylistView::ClearPlaylist()
{
    _playlistModel->clear();
    _selectedTrack = -1;
}

//!---------- PlaylistView::GetNextTrack ---------------------------------------
QString PlaylistView::GetNextTrack()
{
  QString file;
  
  int activerow = _playlistModel->getActiveRow();
  
  if(activerow < _playlistModel->rowCount(QModelIndex()) - 1) {
    _playlistModel->skipForward();
    file = _playlistModel->getActiveUrl();
    updatePlaylistActions();
  }
  return (file);  
}


//!---------- PlaylistView::GetPrevTrack ---------------------------------------
QString PlaylistView::GetPrevTrack()
{
  QString file;
  
  int activerow = _playlistModel->getActiveRow();

  if(activerow > 0) {
    _playlistModel->skipBackward();
    file = _playlistModel->getActiveUrl();
    updatePlaylistActions();    
  }
  return (file);  
}

//!---------- PlaylistView::GetActiveTrack -------------------------------------
QString PlaylistView::GetActiveTrack()
{
  return (_playlistModel->getActiveUrl()); 
}

//!---------- PlaylistView::GetSelectedTrack -----------------------------------
QString PlaylistView::GetSelectedTrack()
{
  //! TEST modification, pas besoin de crer un nouveau pointeur
  QString file;
   
  if (_playlistModel->rowExists(_selectedTrack)) {
  	MediaItem *mi = _playlistModel->MediaItemAt(_selectedTrack);
  	file = mi->url;
  }
 
  return (file); 
}


//!---------- PlaylistView::ActiveSelectedTrack --------------------------------
void PlaylistView::ActiveSelectedTrack()
{
  // use when play/pause action from mainwindow
  // we get first the selected item and play it (so we have to activate it) 
  if (_playlistModel->rowExists(_selectedTrack)) {
     _playlistModel->setActiveRow(_selectedTrack);
     updatePlaylistActions();
  }
}


//!---------- PlaylistView::StopPlaying ----------------------------------------
void  PlaylistView::StopPlaying() 
{
  //stop all item activation from the model
  _playlistModel->setInactiveMediaItem();
}


//!---------- PlaylistView::RemoveTrack ----------------------------------------
void  PlaylistView::RemoveTrack() 
{
  if( (_selectedTrack > -1) && _playlistModel->rowExists(_selectedTrack)) {
      qDebug() << "PlaylistView --> removing track from playlist : selectedtrack" << _selectedTrack;
      _playlistModel->removeRows(_selectedTrack, 1, QModelIndex());
  }
}


//!---------- PlaylistView::contextMenuEvent -----------------------------------
void PlaylistView::contextMenuEvent ( QContextMenuEvent * event ) 
{
  _contextMenu->popup(mapToGlobal(event->pos()));
}

//!---------- PlaylistView::paintEvent -----------------------------------------
// specific painting for empty playlist (message for the user)
void PlaylistView::paintEvent(QPaintEvent *event)
{ 
  if(_playlistModel->rowCount(QModelIndex()) < 1) 
  {
    QPainter painter(viewport());
    QString message = tr("Add file to play or Drag and Drop File from Collection");
    
    QRect textBox = rect().adjusted(5, 0, 0, -10);
    QFontMetrics fontmetric(painter.font());
    QRect fontrect = fontmetric.boundingRect (textBox,  Qt::AlignCenter | Qt::TextWordWrap, message, 0, 0 );

    painter.drawText(textBox.adjusted(0, fontrect.height(), 0, 0), Qt::AlignCenter | Qt::TextWordWrap, message);
  }
  else 
  {
    QListView::paintEvent(event);
  }
}




//!---------- PlaylistDelegate::PlaylistDelegate -------------------------------
PlaylistDelegate::PlaylistDelegate(QObject *parent):QItemDelegate(parent)
{
  _playlistMode = Playlist::TitleMode;
}



void PlaylistDelegate::paint ( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
    QStyleOptionViewItemV4 opt(option);
    QStyle *style = opt.widget ? opt.widget->style() : QApplication::style();
    style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, painter, opt.widget);  
      
    //! get the data object
    PlaylistModel * model = (PlaylistModel *)index.model();
    MediaItem *mi = model->MediaItemAt(index.row());
    
    const int left = option.rect.left();
    const int top = option.rect.top();
    const int width = option.rect.width();
    const int height = 20;
    int vAlign = Qt::AlignVCenter;
    int hAlign = Qt::AlignLeft;
    const bool isSelected = option.state & QStyle::State_Selected;

    QFontMetrics fm( QFont("Trebuchet MS", 8, QFont::Normal) );    
   
    //! Create base pixmap
    QPixmap pixmap(width, height);
    pixmap.fill(Qt::transparent);
    QPainter p(&pixmap);
    p.translate(-option.rect.topLeft());

    p.setFont(QFont("Trebuchet MS", 8, QFont::Normal));
    if (isSelected) {
        QApplication::style()->drawPrimitive(QStyle::PE_PanelItemViewItem, &option, painter);
        p.setPen(QPen(option.palette.brush(QPalette::HighlightedText), 0));
    } else {    
      p.setPen(QColor(0x315996));
    }

    QIcon icon;    
    if(mi->isExists)  {
      //! Paint Icon
      icon = this->getIcon(mi);
      icon.paint(&p, left + 3, top + 2, 16, 16, Qt::AlignCenter, QIcon::Normal);
      
      //! Paint duration
      QString durationText = MEDIA::getDurationText(mi->duration);
      p.drawText(left + width - 50,top, 49, height,Qt::AlignVCenter | Qt::AlignRight, durationText);	
      QRect rectDuree = fm.boundingRect ( durationText );

      //! Paint track info
      QString trackinfo = this->getTrackInfo(mi);
      trackinfo = fm.elidedText ( trackinfo, Qt::ElideRight, width-51 );
      p.drawText(left + 25,top,width-rectDuree.width()-5, 20,vAlign | hAlign, trackinfo);
    }
    else  {
      //Paint Icon
      icon = QIcon(":/images/file-broken.png");
      icon.paint(&p, left + 3, top + 2, 16, 16, Qt::AlignCenter, QIcon::Normal);
      
      //Paint Url
      QString urlTruncated = fm.elidedText ( mi->url, Qt::ElideRight, width-51 );
      p.drawText(left + 25,top,width -5, 20,vAlign | hAlign, urlTruncated);
    }
    
    p.end();
    //Draw finished pixmap
    painter->drawPixmap(option.rect.topLeft(), pixmap);
}

    
QSize PlaylistDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
{
  Q_UNUSED(index)
  return QSize(option.rect.width(), 20);
}


QString PlaylistDelegate::getTrackInfo(const MediaItem *mi) const
{
  QString trackinfo;
  if( MEDIA::isUrlLocal(mi->url) ) {
    //! local Url --> depend du mode choisi
    switch(_playlistMode) {
      case Playlist::TitleMode  : trackinfo = (mi->title);break;
      case Playlist::AlbumMode  : trackinfo = (mi->album + " - " + mi->title);break;
      case Playlist::ArtistMode  : trackinfo = (mi->artist + " - " + mi->album + " - " + mi->title);break;
      default : trackinfo = (mi->title);
    }
  }
  else {
    //! remote Url --> on affiche seulement le basename
    //TEST ou mi->url si probleme de QFileINfo(stringUrl).basename
    trackinfo = mi->title; 
  }

  return trackinfo;
}


QIcon PlaylistDelegate::getIcon(const MediaItem *mi) const
{
  if(mi->isPlaying) 
    return QIcon(":/images/media-playing.png");
  
  if(MEDIA::isUrlLocal(mi->url)) 
    return QFileIconProvider().icon(QFileInfo(mi->url));
  else 
    return QIcon(":/images/media-url.png");; 
}
