/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "RadioView/radiowidget.h"
#include "Widget/statusbar.h"
#include "constants.h"

#include <QSettings>

namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}


RadioWidget::RadioWidget(QWidget *parent)
{
  Q_UNUSED(parent)
   
  //! get StatusBar global instance
  _statusBar        = StatusBar::instance();
  _centralToolBar   = CentralToolBar::instance();
  
  _graphicView = new QGraphicsView(this) ;
  
  //! GraphicWidget Setup  
  setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
  
  //! GraphicView Setup  
  _graphicView->setCacheMode(QGraphicsView::CacheNone);
  _graphicView->setDragMode(QGraphicsView::NoDrag); 
  
  _graphicView->setRenderHint(QPainter::Antialiasing);
  _graphicView->setOptimizationFlags(QGraphicsView::DontAdjustForAntialiasing
                                   | QGraphicsView::DontClipPainter
                                   | QGraphicsView::DontSavePainterState);
  _graphicView->setResizeAnchor(QGraphicsView::NoAnchor);
  _graphicView->setViewportUpdateMode(QGraphicsView::MinimalViewportUpdate); 
  _graphicView->setAlignment(Qt::AlignLeft | Qt::AlignTop); 
      
  //! Gui Creation
  QVBoxLayout *verticalLayout_0 = new QVBoxLayout(this);
  verticalLayout_0->setSpacing(0);
  verticalLayout_0->setContentsMargins(0, 0, 0, 0);
  verticalLayout_0->addWidget(_graphicView);
    
  //! default _scrollPosition
  _scrollPosition[Radio::ViewIceCast]        = 1;
  _scrollPosition[Radio::ViewShoutCast]      = 1;
  _scrollPosition[Radio::ViewCustomRadio]    = 1;
  _scrollPosition[Radio::ViewFavoriteRadio]  = 1;
  
  
  //! connect toolbar signals
  connect(_centralToolBar, SIGNAL(radioSearch(QString)),this, SLOT(updateRadioSearch(QString)));
  
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  connect(actions->value("icecastview"), SIGNAL(triggered()),this,  SLOT(switchIceCastView()));
  connect(actions->value("shoutcastview"), SIGNAL(triggered()),this,  SLOT(switchShoutCastView()));
  connect(actions->value("customradioview"), SIGNAL(triggered()),this,  SLOT(switchCustomRadioView()));
  connect(actions->value("favoriteradioview"), SIGNAL(triggered()),this,  SLOT(switchFavoriteRadioView()));
  connect(actions->value("goUp"), SIGNAL(triggered()),this,  SLOT(navigateGoUp()));
  connect(actions->value("goDown"), SIGNAL(triggered()),this,  SLOT(navigateGoDown()));
}


void RadioWidget::setScene(RadioScene *scene)
{
  _radioScene  = scene;
  _graphicView->setScene(_radioScene);
  connect(_radioScene, SIGNAL(repopulateRadioScene()),this,  SLOT(updateRadioScene()));
}


//!----------------------------------------------------------------------------
//! save and restore view mode
//!----------------------------------------------------------------------------

void RadioWidget::saveLastRadioView()
{
    QSettings *_settings = new QSettings(_CONFIGDIR + QDir::separator() + "main.conf",QSettings::IniFormat,this);
    _settings->setValue("Radio/view",_radioMode );
    _settings->setValue("Radio/file",_radioScene->currentXspfFilename );
}

void RadioWidget::restoreLastRadioView()
{
  QSettings *_settings = new QSettings(_CONFIGDIR + QDir::separator() + "main.conf",QSettings::IniFormat,this);
  
  if(_settings->contains("Radio/view"))
    _radioMode = Radio::RadioMode(_settings->value("Radio/view").toInt());
  else 
    _radioMode = Radio::ViewIceCast;
  
  if(_settings->contains("Radio/file"))
   _radioFile = _settings->value("Radio/file").toString();
  
  _prevRadioView = _radioMode;
  
  updateRadioScene();
}



//!----------------------------------------------------------------------------
//! Scene population
//!----------------------------------------------------------------------------
//! only use after database rebuild
void RadioWidget::updateRadioScene() 
{
    qDebug() << "RadioWidget -  updateRadioScene " << _radioMode;
    switchView(_radioMode);
}


//!----------------------------------------------------------------------------
//! View Switch
//!----------------------------------------------------------------------------
void RadioWidget::switchView(Radio::RadioMode mode) 
{
  qDebug() << "RadioWidget -  switchView " << mode;
  
  //! Save previous scroll bar position
  _scrollPosition[_radioMode]  = _graphicView->verticalScrollBar()->sliderPosition();

   //! save previous views
  _prevRadioView = _radioMode;

  //! Change collection view mode
  _radioMode = mode;
  
  _radioScene->_radioMode = _radioMode;
  _radioScene->currentXspfFilename = _radioFile;
  _radioScene->populateScene();

  updateRadioSummary();
    
  //!restore scroll position
  _graphicView->verticalScrollBar()->setSliderPosition(_scrollPosition[mode]);
}


void RadioWidget::switchIceCastView() {
  switchView(Radio::ViewIceCast);
}

void RadioWidget::switchShoutCastView() {
  switchView(Radio::ViewShoutCast);
}

void RadioWidget::switchCustomRadioView() {
   switchView(Radio::ViewCustomRadio);
}

void RadioWidget::switchFavoriteRadioView() {
  switchView(Radio::ViewFavoriteRadio);
}
    
    
void RadioWidget::restorePrevRadioView() {
   switchView(_prevRadioView);
}
  
//!----------------------------------------------------------------------------
//! Search management
//!----------------------------------------------------------------------------
void RadioWidget::updateRadioSearch(QString filter) 
{
  qDebug() << "---RadioWidget::updateRadioSearch  " << filter;
  _radioScene->_streamModel->_filterPattern = filter;
  switchView(_radioMode);
}

//!----------------------------------------------------------------------------
//! Collection summary
//!----------------------------------------------------------------------------
void RadioWidget::updateRadioSummary() 
{
  int size = _radioScene->getSceneInfoSize();
  
  QString info = QString(tr("Radio : <b>%1</b> streams")).arg(QString::number(size));
  _statusBar->updateCollectionInfo(info);
}

//!----------------------------------------------------------------------------
//! Collection Navigation
//!----------------------------------------------------------------------------
void RadioWidget::navigateGoUp() 
{
  int top     = _graphicView->verticalScrollBar()->sliderPosition();

  for(int i=_radioScene->_listCategorie.size()-1; i>=0;i--){

    QPoint itemPoint = (_radioScene->_listCategorie.at(i)->scenePos()).toPoint();

    if(top > itemPoint.y()) {
      _graphicView->verticalScrollBar()->setSliderPosition(itemPoint.y());
      break;
    }
  }
}

void RadioWidget::navigateGoDown() 
{
  int top     = _graphicView->verticalScrollBar()->sliderPosition();
  
  foreach (QGraphicsItem *item, _radioScene->_listCategorie)
  {
    QPoint itemPoint = (item->scenePos()).toPoint();

    if(top < itemPoint.y()) {
      _graphicView->verticalScrollBar()->setSliderPosition(itemPoint.y());
      break;
    }
  }
}
