/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "settingsdialog.h"
#include "mainwindow.h"
#include "constants.h"

#include <QDebug>
#include <QListWidget>
#include <QStringList>


SettingsDialog::SettingsDialog(MainWindow *parent): QDialog(parent)
{
  _parent = parent;
  //! ---- create GUI ----
  createGui();

  //! ---- restore Global Yarock settings ----
  QSettings _settings(QString(_CONFIGDIR).append("/main.conf"),QSettings::IniFormat,this);
  _settings.beginGroup("Yarock");
  if(_settings.contains("Systray")) {
    _settingGlobalPage->minimazeToSystray->setChecked(_settings.value("Systray",true).toBool());
  }
  _settings.endGroup();

  
  //! ---- restore Collection settings ----
  _settings.beginGroup("Collections");
  if(_settings.contains("AutoRebuildAfterStart")) {
    _settingCollectionPage->autoupdateCollectionsCheckbox->setChecked(_settings.value("AutoRebuildAfterStart",true).toBool());
  }
  if(_settings.contains("SearchCoverFromDirectory")) {
    _settingCollectionPage->searchCoverFromDir->setChecked(_settings.value("SearchCoverFromDirectory",true).toBool());
  }
  if(_settings.contains("SourcePaths")) {
    _settingCollectionPage->collectionsPathsList->addItems(_settings.value("SourcePaths",QStringList()).toStringList());
  }
  _settings.endGroup();

  
  //! ---- restore Radio settings ----
  const QStringList filters = QStringList() << "*.xspf";
  QDir sourceDir(_CONFIGDIR + "/radio");
  sourceDir.setNameFilters(filters);  

  QStringList entryList = sourceDir.entryList(filters, QDir::Files);
  _settingRadioPage->RadioFileList->addItems(entryList);
  
  //! ---- Connection ----
  QObject::connect(pageSelector, SIGNAL(itemClicked ( QListWidgetItem *)),this, SLOT(changePage(QListWidgetItem*)));
  QObject::connect(buttonBox, SIGNAL(accepted()), this, SLOT(on_buttonBox_accepted()));
  QObject::connect(buttonBox, SIGNAL(rejected()), this, SLOT(on_buttonBox_rejected()));
}


void SettingsDialog::createGui() 
{
  //! QDialog
  this->resize(489, 358);
  this->setModal(false); 
  this->setWindowTitle(tr("YaRock settings"));

  //! ListWidget 
  pageSelector = new QListWidget(); 
  pageSelector->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Expanding);
  pageSelector->setMinimumWidth(112);
  pageSelector->setMaximumWidth(112);
  pageSelector->setDropIndicatorShown(false);
  pageSelector->setDragDropMode(QAbstractItemView::NoDragDrop);
  pageSelector->setIconSize(QSize(26, 26));
  pageSelector->setMovement(QListView::Static);
  pageSelector->setViewMode(QListView::IconMode);
  pageSelector->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    
  //! ListWidget Item
  QListWidgetItem *globalMenuItem     = new QListWidgetItem("Global setting");
  QListWidgetItem *collectionMenuItem = new QListWidgetItem("Collection setting");
  QListWidgetItem *radioMenuItem      = new QListWidgetItem("Radio setting");

  globalMenuItem->setIcon(QIcon(":/icon/64x64/yarock.png"));
  globalMenuItem->setText(tr("Yarock"));
  globalMenuItem->setTextAlignment(Qt::AlignHCenter);
  globalMenuItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
  globalMenuItem->setSizeHint(QSize(105,50));  
  
  collectionMenuItem->setIcon(QIcon(":/images/collection.png"));
  collectionMenuItem->setText(tr("Collection"));
  collectionMenuItem->setTextAlignment(Qt::AlignHCenter);
  collectionMenuItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
  collectionMenuItem->setSizeHint(QSize(105,50));

  radioMenuItem->setIcon(QIcon(":/images/podcast110x110.png"));
  radioMenuItem->setText(tr("Radio"));
  radioMenuItem->setTextAlignment(Qt::AlignHCenter);
  radioMenuItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
  radioMenuItem->setSizeHint(QSize(105,50));
  
  pageSelector->addItem(globalMenuItem);
  pageSelector->addItem(collectionMenuItem);
  pageSelector->addItem(radioMenuItem);

  //! StackedWidget
  _settingGlobalPage      = new SettingGlobalPage;
  _settingCollectionPage  = new SettingCollectionPage;
  _settingRadioPage       = new SettingRadioPage;
  _stackedPages = new QStackedWidget();
  _stackedPages->addWidget(_settingGlobalPage);
  _stackedPages->addWidget(_settingCollectionPage);
  _stackedPages->addWidget(_settingRadioPage);
     
  //! QDialogButtonBox
  buttonBox = new QDialogButtonBox(this);
  buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
  buttonBox->setOrientation(Qt::Horizontal);
  buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
        
  //! Layout
  QVBoxLayout* vl0 = new QVBoxLayout(this);
  vl0->setObjectName(QString::fromUtf8("vl0"));
     
  QHBoxLayout* hl0 = new QHBoxLayout();
  hl0->setObjectName(QString::fromUtf8("hl0"));    
  hl0->addWidget(pageSelector);
  hl0->addWidget(_stackedPages);    
  
  vl0->addLayout(hl0);
  vl0->addWidget(buttonBox);
}

//! --------- setStartPage -----------------------------------------------------
void SettingsDialog::setStartPage(SettingsPage::Page page)
{
  pageSelector->setCurrentRow(int(page));
  _stackedPages->setCurrentIndex(pageSelector->currentRow());
}


//! --------- changePage -------------------------------------------------------
void SettingsDialog::changePage(QListWidgetItem *itemClicked)
{
  _stackedPages->setCurrentIndex(pageSelector->row(itemClicked));
}

//! --------- on_buttonBox_accepted --------------------------------------------
void SettingsDialog::on_buttonBox_accepted()
{
  //! save Global Yarock setting
  QSettings _settings(QString(_CONFIGDIR).append("/main.conf"),QSettings::IniFormat,this);
  _settings.beginGroup("Yarock");
  _settings.setValue("Systray",_settingGlobalPage->minimazeToSystray->isChecked());
  _settings.endGroup();
    
  //! save collection setting
  _settings.beginGroup("Collections");
  _settings.setValue("AutoRebuildAfterStart",_settingCollectionPage->autoupdateCollectionsCheckbox->isChecked());
  _settings.setValue("SearchCoverFromDirectory", _settingCollectionPage->searchCoverFromDir->isChecked());
  
  QStringList items;
  for (int i = 0; i < _settingCollectionPage->collectionsPathsList->count(); i++) {
      items.append(_settingCollectionPage->collectionsPathsList->item(i)->text());
  }
  
  _settings.setValue("SourcePaths",items);
  _settings.endGroup();

  if(_settingCollectionPage->_collectionsSourceChanged == true) {
      //qDebug() << "emition signal rebuild collection : ";
      emit signal_rebuildDatabase();
  }

  //! save collection setting (nothing to save --> signal to emit ?)
  if(_settingRadioPage->_radioSourceChanged == true) {
      //qDebug() << "emition signal rebuild radio : ";
      emit signal_rebuildRadio();
  }  

  this->close();
}

//! --------- on_buttonBox_rejected --------------------------------------------
void SettingsDialog::on_buttonBox_rejected()
{
    this->close();
}
