/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtGui>
#include <QtNetwork>
#include "Web/coversearch.h"
#include "Web/networkaccess.h"
#include "constants.h"

namespace GLOBAL {
    NetworkAccess* http();
}

//! ------------------ class CoverSearch ---------------------------------------  
CoverSearch::CoverSearch(QString artist, QString album, QString coverName) 
{
  //_statusBar = StatusBar::instance();
  qDebug() << " ---- SearchAlbumCover ----";
  //_statusBar->showMessage("Searching cover on LastFm", 4000);

  _album     = album;
  _artist    = artist;
  _coverName = coverName;
  
  //! check artist (shall be present)
  if (_artist.isEmpty()) {
    coverSearchStop("Error artist name empty");
  }
  else {
    //sendAlbumSearch();
    sendAlbumInfo();
  }
}

//! ------------------  sendAlbumSearch ---------------------------------------  
void CoverSearch::sendAlbumSearch() 
{
  qDebug() << " ---- CoverSearch -> sendAlbumSearch";

  QUrl url("http://ws.audioscrobbler.com/2.0/");
  url.addQueryItem("method", "album.search");
  url.addQueryItem("api_key", LASTFM_API_KEY);
  url.addQueryItem("artist", _artist);
  url.addQueryItem("album", _album);

  QObject *reply = GLOBAL::http()->get(url);
  connect(reply, SIGNAL(data(QByteArray)), this, SLOT(parseAlbumSearch(QByteArray)));
  connect(reply, SIGNAL(error(QNetworkReply*)), this, SLOT(coverSearchStop()));
}

//! ------------------  parseAlbumSearch ---------------------------------------  
void CoverSearch::parseAlbumSearch(QByteArray bytes) 
{
  qDebug() << " ---- CoverSearch -> parseAlbumSearch";
  QXmlStreamReader xml(bytes);

  QString lastFmArtist;
  QString lastFmAlbum;

  while(!xml.atEnd() && !xml.hasError()) {
    xml.readNext();
    if (xml.isStartElement()) {
        if(xml.name() == "artist") {
          lastFmArtist = xml.readElementText();
        } else if (xml.name() == "name") {
          lastFmAlbum = xml.readElementText();
        }
    } 
    else if (xml.isEndElement()) {
        if(xml.name() == "album") {
            if (_artist == lastFmArtist) {
                if (_album != lastFmAlbum) {
                  //! album name correction
                  _album = lastFmAlbum;
                }
            break;
            }
        }
    }
  } // end while loop

  sendAlbumInfo();
}



//! ------------------  sendAlbumInfo ------------------------------------------  
void CoverSearch::sendAlbumInfo() 
{
  qDebug() << " ---- CoverSearch -> sendAlbumInfo";

  QUrl url("http://ws.audioscrobbler.com/2.0/");
  url.addQueryItem("method", "album.getinfo");
  url.addQueryItem("api_key", LASTFM_API_KEY);

  url.addQueryItem("artist", _artist);
  url.addQueryItem("album",  _album);
  
  QObject *reply = GLOBAL::http()->get(url);
  connect(reply, SIGNAL(data(QByteArray)), this, SLOT(parseAlbumInfo(QByteArray)));
  connect(reply, SIGNAL(error(QNetworkReply*)), this, SLOT(coverSearchStop()));
}

//! ------------------  parseAlbumInfo -----------------------------------------  
void CoverSearch::parseAlbumInfo(QByteArray bytes) 
{
  qDebug() << " ---- CoverSearch -> parseAlbumInfo";
  QXmlStreamReader xml(bytes);
  bool imageFound = false;

  //! Xml reading loop
  while(!xml.atEnd() && !xml.hasError()) 
  {
    QXmlStreamReader::TokenType token = xml.readNext();
    //! search Xml start element
    if(token == QXmlStreamReader::StartElement) 
    {	  
      //! search Xml Image element (size = medium, large, extralarge)
	    if(xml.name() == "image" && xml.attributes().value("size") == "large")
      {
        //! we have to check if image is already stored
        bool imageAlreadyPresent = false;
        QString imageFilePath = _CONFIGDIR + "/albums/" + _coverName;
        if (QFile::exists(imageFilePath)) {
          qDebug() << " ---- CoverSearch -> parseAlbumInfo Image already exist";
          imageAlreadyPresent = true;
        }
        // image has to be stored
        if (!imageAlreadyPresent) {
          QString imageUrl = xml.readElementText();
          if (!imageUrl.isEmpty()) {
            QUrl url = QUrl::fromEncoded(imageUrl.toUtf8());
            QObject *reply = GLOBAL::http()->get(url);
            connect(reply, SIGNAL(data(QByteArray)), this, SLOT(saveCover(QByteArray)));
            connect(reply, SIGNAL(error(QNetworkReply*)), this, SLOT(coverSearchStop()));
	    imageFound = true;
            break;
          }
        }
      } // end Xml image element
    } // end Xml start element
  } // end Xml reading loop

  // Error handling.
  if(xml.hasError()) coverSearchStop("Network error");  
  if(!imageFound) coverSearchStop("No cover found");
}


//! ------------------  saveCover ----------------------------------------------  
void CoverSearch::saveCover(QByteArray bytes) 
{
  qDebug() << " ---- CoverSearch -> saveCover";
  
  //! filepath for cover file
  QString filePath = _CONFIGDIR + "/albums/" + _coverName;
    
  //! remove existing old cover image
  QFile file(filePath);
  //if(file.exists()) file.remove();

  if(file.exists()) {
    coverSearchStop("Cover Already set");
    return;
  }
  
  
  if (!file.open(QIODevice::WriteOnly)) {
      coverSearchStop("Error writing album image");
  }
  else {
    QImage image = QImage::fromData(bytes);
    //!WARNING fix size
    image = image.scaled(QSize(110, 110), Qt::KeepAspectRatio, Qt::SmoothTransformation);
    image.save(filePath, "png", -1);
    
    emit coverUpdated();
    coverSearchStop("Cover downloaded");
  }
}


//! ------------------  coverSearchStop ----------------------------------------  
void CoverSearch::coverSearchStop(QString message)  
{
  qDebug() << " ---- CoverSearch -> coverSearchStop : message" << message;
  emit coverSearchDone(message);
}


void CoverSearch::coverSearchStop()  
{
  qDebug() << " ---- CoverSearch -> coverSearchStop : ";

  emit coverSearchDone("");
}
