/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "playertoolbar.h"
#include "Widget/spacer.h"

#include <phonon/seekslider.h>
#include <phonon/volumeslider.h>
#include <QTime>
#include <QSpacerItem>

namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}


//!-----------------------------------------------------------------------------------
//! class VolumeToolButton
//!-----------------------------------------------------------------------------------
VolumeToolButton::VolumeToolButton(QWidget *parent, Player* player)  
: QToolButton( parent )
{
  this->setIconSize( QSize( 24, 24 ) );
  this->setAutoRaise(true);
  this->setIcon(QIcon(":/images/volume-icon.png")); 

   _player = player;

  //! Phonon volume slider
  Phonon::VolumeSlider *volumeSlider = new Phonon::VolumeSlider(this);
  volumeSlider->setAudioOutput(_player->audioOutput());
  volumeSlider->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
  volumeSlider->setMuteVisible(false);
  volumeSlider->setOrientation (Qt::Vertical);
  volumeSlider->setFixedHeight( 100 );
  
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  //! Popup volume menu 
  _volumeMenu = new QMenu();
  QToolBar *volumeToolBar = new QToolBar(_volumeMenu);
  volumeToolBar->addAction(actions->value("volmute"));
     
  QVBoxLayout * mainBox = new QVBoxLayout(_volumeMenu);
  mainBox->setSpacing(0);
  mainBox->setContentsMargins(10,10,10,10);
  mainBox->addWidget(volumeToolBar);
  mainBox->addWidget(volumeSlider);
  
  QObject::connect(actions->value("volmute"), SIGNAL(triggered()), this, SIGNAL(signalVolumeMute()));
  QObject::connect(this, SIGNAL(clicked()), this, SLOT(volumeMenu()));
}


void VolumeToolButton::volumeMenu()
{
    QPoint menuLocation = this->mapToGlobal(QPoint(0,+35));
    _volumeMenu->exec(menuLocation);
}






//!-----------------------------------------------------------------------------------
//! class PlayerToolBar
//!-----------------------------------------------------------------------------------
PlayerToolBar::PlayerToolBar(QWidget *parent, Player* player)  
 : QToolBar( "Player Tool Bar", parent )
{
  _player = player;
  

  actions = GLOBAL::globalActions();

  this->setObjectName(QString::fromUtf8("playerToolBar"));
  this->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
  this->setIconSize( QSize( 32, 32 ) ); 
  this->setFloatable(false);
  this->setMovable(false);
  this->setContentsMargins( 0, 0, 0, 0);
  this->setAllowedAreas( Qt::AllToolBarAreas );

  //! phonon slider
  seekSlider = new Phonon::SeekSlider(this);
  seekSlider->setMediaObject(_player->mediaObject());
  seekSlider->setIconVisible(false);
  seekSlider->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

  //! track time position
  QFont font; font.setPointSize(font.pointSize()*.90);
  currentTime = new QLabel(this);
  currentTime->setFont(font);

  totalTime = new QLabel(this);
  totalTime->setFont(font);

  //! ToolButton creation
  QToolButton* _prevButton = new QToolButton(this);
  _prevButton->setDefaultAction(actions->value("prev"));
  _prevButton->setIconSize( QSize( 24, 24 ) );
  
  QToolButton* _nextButton = new QToolButton(this);
  _nextButton->setDefaultAction(actions->value("next"));  
  _nextButton->setIconSize( QSize( 24, 24 ) );

    QToolButton* _playButton = new QToolButton(this);
  _playButton->setDefaultAction(actions->value("play"));    
  _playButton->setIconSize( QSize( 24, 24 ) );
  
  QToolButton* _stopButton = new QToolButton(this);
  _stopButton->setDefaultAction(actions->value("stop"));  
  _stopButton->setIconSize( QSize( 24, 24 ) );

  //! volume tool button and menu
  _volumeButton = new VolumeToolButton(this, _player);
  QObject::connect(_volumeButton, SIGNAL(signalVolumeMute()), this, SLOT(volumeMute()));
  
  this->addWidget( new FixedSpacer(this, QSize(5,0)) );
  this->addWidget( _prevButton);
  this->addWidget( _playButton);
  this->addWidget( _stopButton);
  this->addWidget( _nextButton);
  this->addWidget( new FixedSpacer(this, QSize(40,0)) );
  this->addWidget( currentTime);
  this->addWidget( seekSlider);
  this->addWidget( totalTime);    
  this->addWidget( new FixedSpacer(this, QSize(40,0)) );
  this->addWidget(_volumeButton);
  this->addWidget( new FixedSpacer(this, QSize(5,0)) );
  
  QObject::connect(this->_player, SIGNAL(trackPositionChanged(qint64)), this, SLOT(changeTimePosition(qint64)));
  QObject::connect(this->_player, SIGNAL(trackTotalTimeChanged(qint64)), this, SLOT(changeTotalTimePosition(qint64)));
}

void PlayerToolBar::StopTrackTime() 
{
  currentTime->clear();
  totalTime->clear();
}


void PlayerToolBar::changeTimePosition(qint64 newPos) {
    if (newPos <= 0) {
        currentTime->clear();
        return;
    }

    QTime displayTime = displayTime.addMSecs(newPos);
    QString timeString;

    if (newPos > 3600000)
        timeString = displayTime.toString("h:mm:ss");
    else
        timeString = displayTime.toString("m:ss");
        
    currentTime->setText(timeString);
}


void PlayerToolBar::changeTotalTimePosition(qint64 newTotalTime) {
    currentTime->clear();
    if (newTotalTime <= 0) {
        totalTime->clear();
        return;
    }

    QTime displayTime = displayTime.addMSecs(newTotalTime);
    QString timeString;

    if (newTotalTime > 3600000)
        timeString = displayTime.toString("h:mm:ss");
    else
        timeString = displayTime.toString("m:ss");
        
    currentTime->setText("0:00");
    totalTime->setText(timeString);
}


void PlayerToolBar::volumeMute()
{
  if (_player->audioOutput()->isMuted()) {
    _volumeButton->setIcon(QIcon(":/images/volume-icon.png"));
    (actions->value("volmute"))->setIcon(QIcon(":/images/volume-icon.png"));
    _player->audioOutput()->setMuted(false);
  }
  else {
    _volumeButton->setIcon(QIcon(":/images/volume-muted.png"));
    (actions->value("volmute"))->setIcon(QIcon(":/images/volume-muted.png"));
    _player->audioOutput()->setMuted(true);	    
  }
}
