/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "statusbar.h"
#include "MediaItem/mediaitem.h"

#include <QFileInfo>
#include <QFontMetrics>
#include <QFont>
#include <QFontInfo>


//! ------------------ Global StatusBar instance -------------------------------
StatusBar* StatusBar::s_instance = 0;

namespace GLOBAL
{
    StatusBar* statusBar()
    {
        return StatusBar::instance();
    }
}

//! ------------------ class StatusBar -----------------------------------------
StatusBar::StatusBar(QWidget *parent) : QStatusBar(parent)
{
  //! global statuc instance
  s_instance = this;
  
  //! set player
  _player = Player::instance();
  connect (_player, SIGNAL(trackChanged(QString)), this, SLOT(updateNowPlayingInfo(QString)));
  connect (_player, SIGNAL(metaDataChanged()), this, SLOT(updateMetaData()));
    
  _progressInfoLabel   = 0;
  _progressInfoPixmap  = 0;
  _collectionInfoLabel = 0;
  _playlistInfoLabel   = 0;
  _nowPlayingInfoLabel = 0;

  //! Statusbar setup
  this->setFont(QFont("Trebuchet MS", 8, QFont::Normal));
  this->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
  this->setMinimumHeight (QFontMetrics(QFont("Trebuchet MS",20, QFont::Normal)).height()); 
  this->setContentsMargins( 2, 2, 2, 2 ); 
  this->setSizeGripEnabled( false );

  //! Create Gui
  infoWidget = new QWidget(this);
  infoWidget->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::Expanding);
  
  QHBoxLayout *playlistLayout = new QHBoxLayout(infoWidget);
  playlistLayout->setContentsMargins( 0, 0, 0, 0 );
  playlistLayout->setSpacing( 0 );

  _playlistInfoLabel = new QLabel("Playlist Info",infoWidget);
  _playlistInfoLabel->setMinimumWidth(180);
  _playlistInfoLabel->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::Expanding);
  _playlistInfoLabel->setAlignment( Qt::AlignRight | Qt::AlignVCenter );
    
  _collectionInfoLabel = new QLabel("Collection Info",infoWidget);
  _collectionInfoLabel->setMinimumWidth(180);
  _collectionInfoLabel->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::Expanding);
  _collectionInfoLabel->setAlignment( Qt::AlignRight | Qt::AlignVCenter );
      
  _nowPlayingInfoLabel = new QLabel(tr("No track playing"),infoWidget);
  _nowPlayingInfoLabel->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::Expanding);
  _nowPlayingInfoLabel->setAlignment( Qt::AlignRight | Qt::AlignVCenter );
	  
  QFrame* separator1 = new QFrame( infoWidget );
  separator1->setFrameShape( QFrame::VLine );
  
  QFrame* separator2 = new QFrame( infoWidget );
  separator2->setFrameShape( QFrame::VLine );
  
  QWidget * spacer = new QWidget( infoWidget );
  spacer->setFixedWidth( 5 );
  
  playlistLayout->addWidget(_nowPlayingInfoLabel);
  playlistLayout->addWidget(spacer);   
  playlistLayout->addWidget(separator1);
  playlistLayout->addWidget(_collectionInfoLabel);
  playlistLayout->addWidget(spacer);
  playlistLayout->addWidget(separator2);
  playlistLayout->addWidget(_playlistInfoLabel);
  playlistLayout->addWidget(spacer);
  
  addPermanentWidget( infoWidget );
}



//! --------- updateNowPlayingInfo ---------------------------------------------
void StatusBar::updateNowPlayingInfo(QString url)
{
   //qDebug() << " ---- updateNowPlayingInfo  -> url "<< url;
   QString playingInfo = tr("No track playing");
   if(url.isEmpty()) {
       _nowPlayingInfoLabel->setText(playingInfo);
       infoWidget->update();   
       return;
   } 
   
   if(MEDIA::isUrlLocal(url)) {
       MediaItem *mi = MEDIA::FromDataBase(url);
       if(!mi) mi = MEDIA::FromLocalFile(QFileInfo(url).absoluteFilePath());
        
       playingInfo = QString(tr("<b>%1</b> by <b>%2</b> on <b>%3</b>"))
                            .arg(mi->title, mi->artist, mi->album);
                            
    }
    else {
       playingInfo = QString(tr("<b>%1</b> stream")).arg(QFileInfo(url).baseName());
    }

    _nowPlayingInfoLabel->setText(playingInfo);
    infoWidget->update();
}


//! -------- UpdateMetaData ----------------------------------------------------
void StatusBar::updateMetaData()
{
  QString source = _player->currentMediaSource();
  if(source.isEmpty()) return;
  if(MEDIA::isUrlLocal(source)) return;
    
  QMap<QString, QString> metaData = _player->currentMetaData();

  QString title = metaData.value("TITLE");
  QString album = metaData.value("ALBUM");
  QString artist = metaData.value("ARTIST");
  
  if(!title.isEmpty() && !album.isEmpty() && !artist.isEmpty()) {
      QString playingInfo = QString(tr("<b>%1</b> by <b>%2</b> on <b>%3</b>"))
                            .arg(title, artist, album);
        
      _nowPlayingInfoLabel->setText(playingInfo);
      infoWidget->update();
  }
}
    
//! --------- StatusBar::updateCollectionInfo ----------------------------------
void StatusBar::updateCollectionInfo(QString info)
{
    _collectionInfoLabel->setText(info);
    infoWidget->update();
}


//! --------- StatusBar::updatePlaylistInfo ------------------------------------
void StatusBar::updatePlaylistInfo(QString info)
{
    _playlistInfoLabel->setText(info);
    infoWidget->update();
}


//! --------- Action Management -> startAction ---------------------------------
void StatusBar::startAction(QString action)
{
    _listActions.append(action);
    
    updateProgressAction();
}

//! --------- Action Management -> cancelAction --------------------------------
void StatusBar::stopAction(QString action)
{
    if(_listActions.contains(action))
      _listActions.removeAt (_listActions.indexOf(action));
}

//! --------- Action Management -> updateProgressAction ------------------------
// progress action with visual loading progress icon
static int _loadingProgress = 0;

void StatusBar::updateProgressAction()
{
  bool inProgress = !_listActions.isEmpty();
   
  if(!inProgress) {
    removeWidget(_progressInfoLabel);
    removeWidget(_progressInfoPixmap);
    delete _progressInfoLabel;
    delete _progressInfoPixmap;
    _progressInfoLabel   = 0;
    _progressInfoPixmap  = 0;
    
    return;
  }
  
  //! Les actions sont prioritaires sur les short messages
  //! si actions alors on efface les messages 
  clearMessage();
  QString action = _listActions.first();
  
  if (_progressInfoLabel == 0) {
      _progressInfoLabel = new QLabel(action,this);
      _progressInfoLabel->setAlignment(Qt::AlignLeft | Qt::AlignVCenter); 
      _progressInfoLabel->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::MinimumExpanding);
      insertWidget(0,_progressInfoLabel,1);
  }
  
  _progressInfoLabel->setText(action);

  if ((_loadingProgress > 7) || (_loadingProgress < 0)) {
      _loadingProgress = 0;
  }
  QString pixName = QString("loading-%1").arg(++_loadingProgress);
    
  if (_progressInfoPixmap == 0) {
      _progressInfoPixmap = new QLabel(this);
      _progressInfoPixmap->setMaximumWidth(28);
      _progressInfoPixmap->setMaximumHeight(28);
      _progressInfoPixmap->setPixmap(QPixmap(":/images/" + pixName + ".png"));
      _progressInfoPixmap->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Minimum);
      insertWidget(0,_progressInfoPixmap,1);
  }
  else {
      _progressInfoPixmap->setPixmap(QPixmap(":/images/" + pixName + ".png"));
  }
  
  //! on relance l'update de facon cyclique et automatique
  QTimer::singleShot(300, this, SLOT(updateProgressAction())); 
}
