/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "constants.h"
#include "mainwindow.h"
#include "global.h"
#include "Playlist/playlistview.h"
#include "Playlist/playlistmodel.h"
#include "Playlist/playlistwidget.h"
#include "NowPlaying/nowplayingview.h"

#include "CollectionModel/collectionmodel.h"
#include "CollectionView/collectionscene.h"
#include "CollectionView/collectionwidget.h"
#include "CollectionView/virtualplaylist.h"

#include "CentralWidget/centralwidget.h"
#include "CentralWidget/menumodel.h"
#include "CentralWidget/menuview.h"

#include "UserSettings/settingsdialog.h"

#include "RadioModel/streammodel.h"
#include "RadioView/radioscene.h"
#include "RadioView/radiowidget.h"


MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
{
    //qDebug() << "--- Entering main Window ---";
    setObjectName(QString::fromUtf8("YaRock Player"));
    setWindowTitle(QString::fromUtf8("YaRock Player"));
    resize(1100, 840);
    
    _appIcon = new QIcon(":/icon/64x64/yarock.png");
    QApplication::setWindowIcon(*_appIcon);
    
    // Size Policy
    QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(this->sizePolicy().hasHeightForWidth());
    setSizePolicy(sizePolicy);
    
    _state = Running;
    
    // SysTray can close
    _canClose = false;
    
    // Settings
    _settings = new QSettings(_CONFIGDIR + QDir::separator() + "main.conf",QSettings::IniFormat,this);
    
    //! Create Actions
    actions = GLOBAL::globalActions();
    createActions();
    
    //! Player Phonon
    _player = new Player();
    
    //! status Bar
    _statusBar = new StatusBar(this); 
    this->setStatusBar(_statusBar);
    
    //! Playlist view/model
    _playlistModel = new PlaylistModel(this);
    _playlistView = new PlaylistView(this);
    _playlistView->setPlaylistModel(_playlistModel);
    
    //! Now Playing Widget
    qDebug() << "creation now playing view :";
    _nowplayingview = new NowPlayingView(this);
    
    //! Playlist widget
    qDebug() << "creation playlist widget:";
    _playlistwidget = new PlaylistWidget(this,_playlistView);
    _playlistwidget->setPlaylistModel(_playlistModel);

    //! Collection widget
    qDebug() << "creation collection model:";
    //QStringList header = QStringList() << "id" << "name" << "data1" << "data2" << "data3"; 
    _collectionModel = new CollectionModel(this);

    //! Pls File widget
    qDebug() << "creation plsfile model:";
    //QStringList plsmodelHeader = QStringList() << "id" << "name" << "data1" << "data2" << "data3"; 
    _plsFileModel = new PlsFileModel(this);
    
    //! WARNING menumodel defined browsing views actions
    //! WARNING centraltoolbar defined search and up/down actions
    _menuModel = new MenuModel();
    _menuView = new MenuView(_menuModel, new QWidget());         
    _centralToolBar = new CentralToolBar(this);
        

    qDebug() << "creation CollectionWidget:";
    _collectionWidget     = new CollectionWidget(this);
    _collectionScene      = new CollectionScene(_collectionWidget);
    _collectionWidget->setScene(_collectionScene);

    qDebug() << "creation VirtualPlaylist:";
    _virtualPlaylist   = new VirtualPlaylist(this); 
    _collectionScene->setVirtualPlaylist(_virtualPlaylist);

    qDebug() << "creation FilerViewer:";
    _fileViewerWidget     = new FileViewer(this);
    
    qDebug() << "creation radio widget:";
    _streamModel = new StreamModel(this);
    _radioWidget     = new RadioWidget(this);
    _radioScene      = new RadioScene(_radioWidget);
    _radioScene->setVirtualPlaylist(_virtualPlaylist);
    _radioWidget->setScene(_radioScene);
    
    //! thread to perform full cover search for all the collection
    _coverThread = 0;
    
    //! create  GUI
    createGui();
    _playerToolBar = new PlayerToolBar(this, _player);
    this->addToolBar(Qt::TopToolBarArea, _playerToolBar);
    createMenuBar();

    //! Thread management
    _threadPool = new ThreadPool();

    //! connect Slots 
    connectSlots();

    //! init music collection database
    QFile database(_CONFIGDIR + "/collection.db");
    if(!database.exists()) {
        firstStartDialog();
    }
    else if (_settings->value("Collections/AutoRebuildAfterStart",false).toBool()==true) {
        rebuildDatabase();
    }
    else {
        populateCollectionModel();
        populatePlsFileModel();
    }

    //! Restore windows geometry
    if(_settings->contains("Window/Geometry"))
      restoreGeometry(_settings->value("Window/Geometry", saveGeometry()).toByteArray());
    
    if(_settings->contains("Window/State"))
      restoreState(_settings->value("Window/State", saveState()).toByteArray());
    
    //! restore Playlist Mode/ Collection view
    _playlistwidget->restoreLastPlaylistMode();    
    _collectionWidget->restoreLastCollectionView();
    _radioWidget->restoreLastRadioView();

    //! restore Playlist last played 
    QFile playlistSave(_CONFIGDIR + "/last.m3u");
    if(playlistSave.exists()) 
      _threadPool->PlaylistAddFile(QString(_CONFIGDIR).append("/last.m3u"));
      
    createSystray();
    
    //! Check repertoire pour les download d'url
    if(!QDir().exists(_CONFIGDIR + "/download/")) {
      QDir().mkpath(_CONFIGDIR + "/download/");
    }
}

//! --------- ~MainWindow ------------------------------------------------------  
MainWindow::~MainWindow()
{
   _player->stop();

  _state = Closing;
  //! Save windows setting
  _settings->setValue("Window/Geometry", saveGeometry());
  _settings->setValue("Window/State", saveState());

  //! Save current playlist to file
  _threadPool->PlaylistSaveToFile(QString(_CONFIGDIR).append("/last.m3u"));
    
  //! save collection view / playlist mode
  _collectionWidget->saveLastCollectionView();
  _playlistwidget->saveLastPlaylistMode();   
  _radioWidget->saveLastRadioView();
  
  
  //! delete repertoire temporaire de telechargement
  QString storageLocation = _CONFIGDIR + "/download/";
  QDir dir(storageLocation);

  Q_FOREACH(QFileInfo info, dir.entryInfoList(QDir::NoDotAndDotDot | QDir::System | QDir::Hidden  | QDir::AllDirs | QDir::Files, QDir::DirsFirst)) 
  {
    QFile::remove(info.absoluteFilePath());
  }
  
  //! delete object
  qDebug() << " ---- Waiting threadPool to finish...";
  delete _threadPool;
  //!WARNING needed in order to active save method
  delete _centralWidget;
  delete _menuModel;
  delete _collectionModel;
  delete _plsFileModel;
  delete _playlistModel;
}

//! --------- createGui --------------------------------------------------------  
void MainWindow::createGui() 
{
  //! central widget 
  _centralWidget = new CentralWidget(this,
                                _centralToolBar,
                                _menuView,
                                _nowplayingview,
                                _playlistwidget,
                                _collectionWidget,
                                _fileViewerWidget,
                                _radioWidget
                                );
  this->setCentralWidget(_centralWidget);
}

//! --------- createSystray ----------------------------------------------------
void MainWindow::createSystray() 
{
  //! restore Systray Option
  if(_settings->contains("Yarock/Systray"))
    _systrayActivated = _settings->value("Yarock/Systray",true).toBool();
  else
    _systrayActivated = false;
      
  //! systray
  if(!_systrayActivated) {
    _trayIcon = 0;
    return;
  }
   
  QMenu *_trayIconMenu = new QMenu(this);
	_trayIconMenu->addAction(actions->value("prev"));
	_trayIconMenu->addAction(actions->value("play"));
	_trayIconMenu->addAction(actions->value("stop"));
	_trayIconMenu->addAction(actions->value("next"));
	_trayIconMenu->addSeparator();
	_trayIconMenu->addAction(actions->value("quit"));
	
  _trayIcon = new QSystemTrayIcon(this);
	_trayIcon->setIcon(*_appIcon);
  _trayIcon->setToolTip("Yarock");
	_trayIcon->setContextMenu(_trayIconMenu);
  _trayIcon->show();
	
	connect(_trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(trayClicked(QSystemTrayIcon::ActivationReason)));
	connect(_trayIcon, SIGNAL(messageClicked()), this, SLOT(setFocus()));
}

     
//! --------- createActions ----------------------------------------------------
void MainWindow::createActions()
{
  //! yarock global actions
  QAction *quitAction          = new QAction(QIcon::fromTheme("application-exit"),tr("&Quit Yarock"), this);
  //! playlist action
  QAction *addfileAct          = new QAction(QIcon(":/images/playlist-addfile.png"),tr("&Add media to playlist"), this);
  QAction *adddirAct           = new QAction(QIcon(":/images/playlist-adddir.png"),tr("&Add directory to playlist"), this);
  QAction *addUrlAct           = new QAction(QIcon(":/images/podcast110x110.png"),tr("&Add Url..."), this);
  QAction *clearplaylistAct    = new QAction(QIcon::fromTheme("edit-clear"), tr("&Clear playlist"), this);
  QAction *savePlaylistAct     = new QAction(QIcon(":/images/playlist-save.png"), tr("&Save playlist to file"), this);
  QAction *saveAutoPlaylistAct = new QAction(QIcon(":/images/playlist-auto-save.png"), tr("&Auto save playlist to YaRock database"), this);
  //! player action
  QAction *playAct             = new QAction(QIcon(":/images/media-play.png"), tr("Play or Pause media"), this);
  QAction *stopAct             = new QAction(QIcon(":/images/media-stop.png"), tr("Stop playing media"), this);
  QAction *nextAct             = new QAction(QIcon(":/images/media-next.png"), tr("Play next media"), this);
  QAction *prevAct             = new QAction(QIcon(":/images/media-prev.png"), tr("Play previous media"), this); 
  QAction *volumeMuteAct       = new QAction(QIcon(":/images/volume-icon.png"),"", this); 
  //! database action
  QAction *rebuildAct          = new QAction(QIcon(":/images/rebuild.png"),tr("Force Rebuild Collection"), this); 
  QAction *coverThread         = new QAction(QIcon(":/images/lastfm.png"),tr("Find missing cover art"), this);
  //! help action
  QAction *helpAboutAct        = new QAction(QIcon::fromTheme("help-about"),tr("About YaRock"), this); 
  QAction *helpQtAct           = new QAction(QIcon::fromTheme("help-about"),tr("About Qt"), this); 
  //! user settings actions
  QAction *collectionSettings = new QAction(QIcon(":/images/settings.png"),tr("Collection settings"), this); 
  QAction *allSettings = new QAction(QIcon(":/images/settings.png"),tr("Yarock settings"), this); 
  //! radio actions
  QAction *radioSettings = new QAction(QIcon(":/images/settings.png"),tr("Add/Remove Radio File"), this); 
  
  actions->insert("quit", quitAction);
  actions->insert("addfile", addfileAct);               
  actions->insert("adddir", adddirAct);               
  actions->insert("addurl", addUrlAct);               
  actions->insert("clear", clearplaylistAct);     
  actions->insert("save", savePlaylistAct);  
  actions->insert("autosave", saveAutoPlaylistAct);  
  actions->insert("play", playAct);     
  actions->insert("stop", stopAct);      
  actions->insert("next", nextAct);     
  actions->insert("prev", prevAct);     
  actions->insert("volmute", volumeMuteAct);    
  actions->insert("helpYaRock", helpAboutAct);
  actions->insert("helpQt", helpQtAct);
  actions->insert("forceRebuild", rebuildAct);
  actions->insert("coverThread", coverThread);
  actions->insert("collectionSettings", collectionSettings);  
  actions->insert("allSettings", allSettings);  
  actions->insert("radioSettings", radioSettings);  
        
  (actions->value("clear"))->setEnabled(false);
  (actions->value("save"))->setEnabled(false);
  
  //! playlist Mode Affichage
  QAction *modeTitleAct  = new QAction(tr("&Title"), this);
  QAction *modeAlbumAct  = new QAction(tr("Albu&m - Title"), this);
  QAction *modeArtistAct = new QAction(tr("&Artist - Album - Title"), this);
  modeTitleAct->setCheckable(true);
  modeAlbumAct->setCheckable(true);
  modeArtistAct->setCheckable(true);
  actions->insert("playlistModeTitle", modeTitleAct);               
  actions->insert("playlistModeAlbum", modeAlbumAct);               
  actions->insert("playlistModeArtist", modeArtistAct);  

  //! Main widget global actions
  QAction *showPlaylist = new QAction(QIcon(":/images/playlist.png"),tr("Show/hide playlist"), this);
  QAction *showMenu     = new QAction(QIcon(":/images/menu.png"),tr("Show/hide menu"), this);
  showMenu->setCheckable(true);
  showPlaylist->setCheckable(true);

  actions->insert("showPlaylist", showPlaylist);     
  actions->insert("showMenu", showMenu);
}

//! --------- createMenuBar ----------------------------------------------------
void MainWindow::createMenuBar() 
{
  //! Set QMenuBar
  QMenuBar   *menuBar        = new QMenuBar(this);
  QMenu      *menuYarock     = menuBar->addMenu(tr("&Yarock"));
  QMenu      *menuPlayer     = menuBar->addMenu(tr("&Player"));
  QMenu      *menuPlaylist   = menuBar->addMenu(tr("P&laylist"));
  QMenu      *menuCollection = menuBar->addMenu(tr("&Collection"));
  QMenu      *menuRadio      = menuBar->addMenu(tr("&Radio"));
  QMenu      *menuHelp       = menuBar->addMenu(tr("&Help"));

  //! App Yarock Menu
  menuYarock->addAction(actions->value("showMenu"));
  menuYarock->addAction(actions->value("showPlaylist"));
  menuYarock->addSeparator();  
  menuYarock->addAction(actions->value("allSettings"));
  menuYarock->addSeparator();  
  menuYarock->addAction(actions->value("quit"));
    
  //! Player Menu
  menuPlayer->addAction(actions->value("prev"));
  menuPlayer->addAction(actions->value("play"));
  menuPlayer->addAction(actions->value("stop"));
  menuPlayer->addAction(actions->value("next"));
  
  //! Playlist Menu
  menuPlaylist->addAction(actions->value("addfile"));
  menuPlaylist->addAction(actions->value("adddir"));    
  menuPlaylist->addAction(actions->value("addurl"));    
  menuPlaylist->addAction(actions->value("clear"));
  menuPlaylist->addAction(actions->value("save"));
  menuPlaylist->addAction(actions->value("autosave"));
  menuPlaylist->addSeparator();
    // Playlist Sub menu (mode)
    QMenu *menuMode = menuPlaylist->addMenu(tr("&Choose playlist mode"));
    menuMode->addAction(actions->value("playlistModeTitle"));
    menuMode->addAction(actions->value("playlistModeAlbum"));
    menuMode->addAction(actions->value("playlistModeArtist"));
  menuPlaylist->addSeparator();
  menuPlaylist->addAction(actions->value("showPlaylist"));

  //! Collection Menu
  menuCollection->addAction(actions->value("collectionSettings"));
  menuCollection->addAction(actions->value("forceRebuild"));
  menuCollection->addAction(actions->value("coverThread"));

  //! Radio Menu
  menuRadio->addAction(actions->value("radioSettings"));
  
  //! help about Menu
  menuHelp->addAction(actions->value("helpYaRock"));
  menuHelp->addAction(actions->value("helpQt"));

  this->setMenuBar(menuBar);
}


//! --------- connectSlots -----------------------------------------------------
void MainWindow::connectSlots()
{
  //! Global Yarock Actions
  QObject::connect(actions->value("quit"), SIGNAL(triggered()), SLOT(on_actionQuit_Yarock_triggered()));
  QObject::connect(actions->value("helpYaRock"), SIGNAL(triggered()), SLOT(on_actionAbout_YaRock_triggered()));
  QObject::connect(actions->value("helpQt"), SIGNAL(triggered()), SLOT(on_actionAbout_Qt_triggered()));
    
  //! Connection Actions player
  QObject::connect(actions->value("play"), SIGNAL(triggered()), SLOT(playOrPause()));
  QObject::connect(actions->value("next"), SIGNAL(triggered()), SLOT(playNext()));
  QObject::connect(actions->value("prev"), SIGNAL(triggered()), SLOT(playPrev()));
  QObject::connect(actions->value("stop"), SIGNAL(triggered()), SLOT(stopPlayer()));
  QObject::connect(_player, SIGNAL(trackFinished()), this, SLOT(playNext()));    
  QObject::connect(_player, SIGNAL(stateChanged(Phonon::State,Phonon::State)), this, SLOT(playerStateChanged(Phonon::State,Phonon::State)));

  //! Connection Actions playlist
  QObject::connect(actions->value("clear"), SIGNAL(triggered()), SLOT(clearPlaylist()));
  QObject::connect(actions->value("save"), SIGNAL(triggered()), SLOT(savePlaylist()));
  QObject::connect(actions->value("autosave"), SIGNAL(triggered()), SLOT(autoSavePlaylist()));
  QObject::connect(actions->value("addfile"), SIGNAL(triggered()), SLOT(addFileToPlaylist()));
  QObject::connect(actions->value("adddir"), SIGNAL(triggered()), SLOT(addDirToPlaylist()));
  QObject::connect(actions->value("addurl"), SIGNAL(triggered()), SLOT(addUrlToPlaylist()));
  QObject::connect(_playlistView, SIGNAL(signal_playlist_itemDoubleClicked()), this, SLOT(playTrackFromPlaylist()));    
  QObject::connect(_playlistModel, SIGNAL(signalAddUrls(QList<QUrl>)), this, SLOT(updatePlaylist(QList<QUrl>)));
  
  
  //! Connection Actions Database
  QObject::connect(actions->value("forceRebuild"), SIGNAL(triggered()), SLOT(forceRebuildDatabaseDlg()));
  QObject::connect(actions->value("coverThread"), SIGNAL(triggered()), SLOT(startCoverSearchThread()));

  //! Connection signaux COLLECTION
  QObject::connect(_virtualPlaylist, SIGNAL(signal_collection_playTrack()), this, SLOT(playTrackFromCollection()));    
  QObject::connect(_collectionScene,SIGNAL(signal_RepopulateCollection()),this,SLOT(populateCollectionModel()));
  QObject::connect(_collectionScene,SIGNAL(signal_RepopulatePlsFile()),this,SLOT(populatePlsFileModel()));

  //! Connection signaux USER SETTINGS
  QObject::connect(actions->value("collectionSettings"), SIGNAL(triggered()), SLOT(openCollectionSettingsDlg()));
  QObject::connect(actions->value("allSettings"), SIGNAL(triggered()), SLOT(openAllSettingsDlg()));
  QObject::connect(actions->value("radioSettings"), SIGNAL(triggered()), SLOT(openRadioSettingsDlg()));

  //! ThreadPool connection
  QObject::connect(_threadPool, SIGNAL(playlistSaved(QString)), this, SLOT(checkNewPlaylistSaved(QString)));
  QObject::connect(_threadPool, SIGNAL(databaseBuild()), this, SLOT(populateCollectionModel()));
  QObject::connect(_threadPool, SIGNAL(databasePlsBuild()), this, SLOT(populatePlsFileModel()));
  QObject::connect(_threadPool, SIGNAL(collectionModelPopulated()), this, SLOT(updateCollectionScene()));
  QObject::connect(_threadPool, SIGNAL(plsFileModelPopulated()), this, SLOT(updatePlsFileScene()));
}


//! --------- Quit Actions -----------------------------------------------------
void MainWindow::on_actionQuit_Yarock_triggered()
{
  _canClose = true;
  this->close();
}

void MainWindow::closeEvent(QCloseEvent *event)
{
  if(!_trayIcon) {
    this->close();
  }
  else {
    // only hide app is tray is running
    if (_trayIcon->isVisible() && (_canClose == false)) {
      this->hide();
      event->ignore();
    }
  }
} 

//! -------------- Systray methods ---------------------------------------------
void MainWindow::trayClicked(QSystemTrayIcon::ActivationReason reason)
{
    if (reason == QSystemTrayIcon::Trigger) {
        if (this->isHidden()) {
            this->show();
        } else {
            this->hide();
        }
    }
}

//! --------- Help About -------------------------------------------------------
void MainWindow::on_actionAbout_YaRock_triggered()
{
    QMessageBox aboutDlg;

    QStringList developers;
    developers << "Sébastien Amardeilh";

    QString str = "<h1>"+QApplication::applicationName()+"</h1>"
                  +tr("Version %1").arg(QApplication::applicationVersion())+
                  "<p>This program is free software; you can redistribute it and/or modify it under the terms of "
                  "the GNU General Public License as published by the Free Software Foundation; either version "
                  "2 of the License, or (at your option) any later version.</p>"
                  "<h2>"+tr("Developers")+":</h2>"
                  "<p>"+developers.join(", ")+"</p>"
                  "<p>&copy; 2010 <a href=\"mailto:sebastien.amardeilh@gmail.com\">Sebastien Amardeilh</a></p>";
    aboutDlg.about(this,tr("About YaRock"),str.toAscii());
}

void MainWindow::on_actionAbout_Qt_triggered()
{
  QMessageBox::aboutQt(this,tr("About Qt"));
}


//! --------- File management --------------------------------------------------
//! addFileToPlaylist
//! addDirToPlaylist
//! savePlaylist (m3u, pls, xspf)
void MainWindow::addFileToPlaylist()
{
  // TO DO : repartir du dernier repertoire ouvert 
    QStringList files = QFileDialog::getOpenFileNames(
		this, 
		tr("Add music files or playlist"), 
		QDesktopServices::storageLocation(QDesktopServices::MusicLocation), 
                tr("Audio files(*.mp3 *.wav *.ogg *.flac);;Playlists files(*.m3u *.pls *.xspf);;All files(*.*)"));
			    
    _threadPool->PlaylistAddFiles(files);
}


void MainWindow::addDirToPlaylist()
{
  // TO DO : repartir du dernier repertoire ouvert 
  QString dirName = QFileDialog::getExistingDirectory(
                      this,
                      tr("Add music directory"),
                      QDesktopServices::storageLocation(QDesktopServices::MusicLocation), 
                      QFileDialog::ShowDirsOnly);
  
  if (!dirName.isEmpty())
    _threadPool->PlaylistAddFile(dirName);
}

void MainWindow::addUrlToPlaylist()
{
  bool ok;
  QString url = QInputDialog::getText(this,  
                                      tr("Enter a URL"),
                                      "URL:",
                                      QLineEdit::Normal,
                                      "http://", &ok);
  if (ok && !url.isEmpty()) {
    _threadPool->PlaylistAddFile(url);
  }
}

void MainWindow::savePlaylist()
{
  QString filename = QFileDialog::getSaveFileName(
                this, 
		tr("Save playlist to file"), 
		QDir::homePath(), 
		tr("m3u playlist (m3u) (*.m3u);;pls playlist (pls) (*.pls);;xspf playlist (xspf) (*.xspf)")
		);

    if (!filename.isEmpty())    
      _threadPool->PlaylistSaveToFile(filename);
}

void MainWindow::autoSavePlaylist()
{
    //! check if playlist is not empty
    const int rowCount  = _playlistModel->rowCount(QModelIndex());
    const bool isPlaylistEmpty = rowCount < 1;
    if(isPlaylistEmpty) return;
    
    //! check internal playlist storage localtion
    QString storageLocation = _CONFIGDIR + "/playlists/";
    QDir dir;
    dir.mkpath(storageLocation);
    
    //! build filename
    QDateTime date = QDateTime::currentDateTime();
    QString filename = storageLocation + "playlist-" + date.toString("dd-MM-yyyy-hh:mm:ss") + ".m3u"; 

    //! internal playlist storage
    _threadPool->PlaylistSaveToFile(filename);
}

void MainWindow::checkNewPlaylistSaved(QString filename)
{
  qDebug() << "--- checkNewPlaylistSaved --- path = " << filename;
  
  // pour eviter de lancer cette procedure lors de la sauvegarde de la playlist 
  //lors d'un quit   
  if(_state == Closing) return;
  /* 
  l'enregistrement d'une playlist lors d'un save ou autosave est fait de manire 
  asynchrone. Il faut donc rcuprer le signal playlistSave et vrifier si 
  l'enregistrement a t fait dans les rpertoires de la collection ou 
  du rpertoire interne de sauvegarde 
  --> dans ce cas la il faut lancer un rebuild de la DatabasePlsFile et du modele
  */
   QString storageLocation = _CONFIGDIR + "/playlists/";
   QStringList listDir = QStringList() << _settings->value("Collections/SourcePaths",QStringList()).toStringList();

   QString path = QFileInfo(filename).absolutePath();  
   qDebug() << "--- checkNewPlaylistSaved --- path = " << path;

   if(listDir.contains(path) || storageLocation.contains(path)) {
         _statusBar->startAction(tr("Updating Playlist Files"));
         _threadPool->DatabasePlsBuild(listDir);
   }
}


void MainWindow::updatePlaylist(QList<QUrl> listUrl)
{
  if (!listUrl.isEmpty()) {
    _threadPool->PlaylistAddUrls(listUrl);
  }
}

//! --------- Playlist Widget --------------------------------------------------
void MainWindow::clearPlaylist()
{
    _playlistView->ClearPlaylist();
    _playlistView->StopPlaying();
    _playlistView->updatePlaylistActions();
    
    // on arrete la lecture seulement si il n'y a pas de lecture a partir 
    // de la collection 
    if(!_virtualPlaylist->isPlaying()) {
      _player->stop();
      _playerToolBar->StopTrackTime();
    }

}

//! --------- Play request -----------------------------------------------------
//! playTrackFromPlaylist (a file is activated from the playlist)
//! an item is activated from the collection
void MainWindow::playTrackFromPlaylist()
{
  QString file = _playlistView->GetActiveTrack();

  if(!file.isNull()) {
    _virtualPlaylist->StopPlaying();
    _player->setTrack(QUrl(file));
  }
}

void MainWindow::playTrackFromCollection()
{
  QString file = _virtualPlaylist->GetActiveTrack();

  if(!file.isNull()) {
    _playlistView->StopPlaying();
    _player->setTrack(QUrl(file));
  }
}

//! -------------- Player methods ----------------------------------------------
void MainWindow::playerStateChanged(Phonon::State newState, Phonon::State /* oldState */)
{
  QString playerError;

  switch (newState) {
      case Phonon::ErrorState:   
         playerError = _player->mediaObject()->errorString();
          qDebug() << "MainWindow::playerStateChanged Error State" << playerError;
          qDebug() << "MainWindow::playerStateChanged currentMediaSource" << _player->currentMediaSource();

	 if(playerError.isEmpty())
           playerError = tr("An error has been encountered during playback");
         
	 _collectionModel->stopPlayingItem();
	 _plsFileModel->stopPlayingItem();
	 _streamModel->stopPlayingItem();
         _statusBar->showMessage(playerError, 3000);
      break;
      
      case Phonon::PlayingState:
        (actions->value("stop"))->setEnabled(true); break;

      case Phonon::StoppedState:
        (actions->value("stop"))->setEnabled(false); break;
      
      case Phonon::PausedState:  break;
      case Phonon::BufferingState: break;
      default:break;
  }
}

void MainWindow::playOrPause()
{
  QString file;

  if (_player->playerState() == Phonon::PlayingState) {
    _player->pause();
    (actions->value("play"))->setIcon(QIcon(":/images/media-pause.png"));
  } 
  else {
     if (_player->currentSource().fileName().isEmpty()) {
        file = _playlistView->GetSelectedTrack();
        //qDebug() << "Source is empty, get selected track:" << file;
      
      	if(!file.isEmpty()) {
          _player->setTrack(QUrl(file));
          _playlistView->ActiveSelectedTrack();
        }
     }
     else {
      _player->play();
      (actions->value("play"))->setIcon(QIcon(":/images/media-play.png"));
     }
  }
}

void MainWindow::stopPlayer()
{    
    _player->stop();
    _playlistView->StopPlaying();
    _virtualPlaylist->StopPlaying();    
    _playerToolBar->StopTrackTime();
}



void MainWindow::playNext()
{
  QString file;
  
  //! We need to known if we are playing from collection or playlist
  if(_virtualPlaylist->isPlaying()) {
    file = _virtualPlaylist->GetNextTrack();
    if(!file.isEmpty())
      _player->setTrack(QUrl(file));
    else
      stopPlayer();
  }
  else if(_playlistView->isPlaying()) {
    file = _playlistView->GetNextTrack();
    if(!file.isEmpty())
      _player->setTrack(QUrl(file));
    else
      stopPlayer();
  }
}


void MainWindow::playPrev()
{
  QString file;

  //! We need to known if we are playing from collection or playlist
  if(_virtualPlaylist->isPlaying()) {
    
    file = _virtualPlaylist->GetPrevTrack();
    if(!file.isEmpty())
      _player->setTrack(QUrl(file));
    else
      stopPlayer();
  }
  else if(_playlistView->isPlaying()) {
    file = _playlistView->GetPrevTrack();
    if(!file.isEmpty())
      _player->setTrack(QUrl(file));
    else
      stopPlayer();
  }
}


//! -------------- User settings methods methods -------------------------------
void MainWindow::openCollectionSettingsDlg()
{
  //! Show settings dialog
  SettingsDialog *_settingsDialog = new SettingsDialog(this);
  _settingsDialog->setStartPage(SettingsPage::CollectionPage);

  connect(_settingsDialog,SIGNAL(signal_rebuildDatabase()),this,SLOT(rebuildDatabase()));
  connect(_settingsDialog,SIGNAL(signal_rebuildRadio()),this,SLOT(rebuildRadio()));
  _settingsDialog->exec();
}

void MainWindow::openRadioSettingsDlg()
{
  //! Show settings dialog
  SettingsDialog *_settingsDialog = new SettingsDialog(this);
  _settingsDialog->setStartPage(SettingsPage::RadioPage);

  connect(_settingsDialog,SIGNAL(signal_rebuildDatabase()),this,SLOT(rebuildDatabase()));
  connect(_settingsDialog,SIGNAL(signal_rebuildRadio()),this,SLOT(rebuildRadio()));
  _settingsDialog->exec();
}

void MainWindow::openAllSettingsDlg()
{
  //! Show settings dialog
  SettingsDialog *_settingsDialog = new SettingsDialog(this);
  _settingsDialog->setStartPage(SettingsPage::StartPage);

  connect(_settingsDialog,SIGNAL(signal_rebuildDatabase()),this,SLOT(rebuildDatabase()));
  connect(_settingsDialog,SIGNAL(signal_rebuildRadio()),this,SLOT(rebuildRadio()));
  _settingsDialog->exec();
}

//! -------------- Database methods --------------------------------------------
void MainWindow::firstStartDialog() 
{
    QString str = tr("<h2>Welcome to YaRock</h2>" 
        "<p>No collection seems to be setup</p>"
	"<p>Do you want to setup your collection Now</p>");
     
    QMessageBox firstStartDlg(this);
    //firstStartDlg.setIcon(QIcon(":/icon/64x64/yarock.png"));
    firstStartDlg.setWindowTitle(tr("Setup your music collection directory"));
    firstStartDlg.setText(str.toAscii());
    firstStartDlg.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
    
    QAbstractButton * okButton = firstStartDlg.button(QMessageBox::Ok);
    connect(okButton,SIGNAL(clicked()),this,SLOT(openCollectionSettingsDlg()));

    firstStartDlg.resize(445, 120);
    firstStartDlg.exec();
}

void MainWindow::rebuildDatabase()
{
    qDebug() << " ---- MainWindow --> rebuildDatabase";
    
    QStringList listDir = QStringList() << 
    _settings->value("Collections/SourcePaths",QStringList()).toStringList();
  
    // Database Building
    if (!listDir.isEmpty()) {
      _statusBar->startAction(tr("Updating Music Collection"));
      _statusBar->startAction(tr("Updating Playlist Files"));    
      _threadPool->DatabaseBuild(listDir);
      _threadPool->DatabasePlsBuild(listDir);
    }
    else {
      _statusBar->startAction(tr("Updating Playlist Files"));    
      _threadPool->DatabasePlsBuildInternal();
    }
}        

void MainWindow::forceRebuildDatabaseDlg() 
{
    QString str = tr("<h2>Warning</h2>" 
        "<p>All changes into collection database will be discarded !!</p>");
     
    QMessageBox forceRebuildDlg(this);
    forceRebuildDlg.setIcon(QMessageBox::Warning);
    forceRebuildDlg.setWindowTitle(tr("Force rebuild database YaRock"));
    forceRebuildDlg.setText(str.toAscii());
    forceRebuildDlg.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
    
    QAbstractButton * sb = forceRebuildDlg.button(QMessageBox::Ok);
    connect(sb,SIGNAL(clicked()),this,SLOT(forceRebuildDatabase()));

    forceRebuildDlg.resize(445, 120);
    forceRebuildDlg.exec();
}

void MainWindow::forceRebuildDatabase() 
{
  qDebug() << " ---- Entering forceRebuildDatabase";
  
  //! delete existing database
  QFile database(_CONFIGDIR + "/collection.db");
  if(database.exists()) database.remove();
    
  //! delete existing cover art 
  QString storageLocation = _CONFIGDIR + "/albums/";
  QDir dir(storageLocation);
 
  if (dir.exists(storageLocation)) {
    Q_FOREACH(QFileInfo info, dir.entryInfoList(QDir::NoDotAndDotDot | QDir::System | QDir::Hidden  | QDir::AllDirs | QDir::Files, QDir::DirsFirst)) 
    {
    QFile::remove(info.absoluteFilePath());
    }
  }
  
  //! rebuild initial database
  rebuildDatabase();
}
       
//! -------------- Collection population methods -------------------------------
void MainWindow::populateCollectionModel() 
{
  _collectionScene->clear();
  _statusBar->startAction(tr("Reading Music Collection"));
  _statusBar->stopAction(tr("Updating Music Collection"));
  _threadPool->PopulateCollectionModel();
}

void MainWindow::populatePlsFileModel() 
{
  _collectionScene->clear();
  _statusBar->startAction(tr("Reading Playlist Files"));
  _statusBar->stopAction(tr("Updating Playlist Files"));
  _threadPool->PopulatePlsFileModel();
}

void MainWindow::updateCollectionScene() 
{
  _collectionWidget->updateCollectionScene();
  _statusBar->stopAction(tr("Reading Music Collection"));
}

void MainWindow::updatePlsFileScene()
{
  _collectionWidget->updatePlsFileScene();
  _statusBar->stopAction(tr("Reading Playlist Files"));
}


//! -------------- Thread for cover searching ----------------------------------
void MainWindow::startCoverSearchThread() 
{
  if(!_coverThread) {
    _coverThread = new CoverThread();
    connect(_coverThread,SIGNAL(coverThreadEnd()),this,SLOT(stopCoverSearchThread()));
  }
  
  if(!_coverThread->isRunning()) {
    _statusBar->startAction(tr("Fetching Album Cover"));
    _coverThread->run();
  }
  else {
      qDebug() << " ---- Search Cover thread already running";
  }
}

void MainWindow::stopCoverSearchThread()
{
  _statusBar->stopAction(tr("Fetching Album Cover"));
}



//! -------------- Radio methods -----------------------------------------------
void MainWindow::rebuildRadio()
{
  qDebug() << " ---- MainWindow --> rebuildRadio";
  _menuModel->update();
  //! WARNING on repart de zro sur le breadcrumb
 /* _centralToolBar->_breadcrumb->setCurrentIndex(
          _menuModel->indexFromItem(_menuModel->invisibleRootItem()->child(0))
          );*/
}
